# fsql_action_vars.pl
#    Last change: PFB 2001-10-07 03:15:41
#############################################################################
# This is the action vars setup file for FutureSQL
#############################################################################
# POD Documentation

=head1 STRUCTURE OF THIS FILE

NOTE! Be careful of :!:tags:!: and system variables that DO things.
      I recommend saving a copy of this file somewhere.

      You can see the tags that are frequently parsed by the subroutine
      'parse_params' by looking at that subroutine in the main file.

NOTE! The order that the variables are listed in is important because
      some variables depend on previous variables.

This file contains the following:

        1. Previous and Next Buttons

        2. $default_external_template
           (usually 'fields.html')
           This allows the spec file to work. If you want to use different
           spec files for each table, then use the syntax:
           '[table_name]_fields.html' and the spec file will use them
           automatically. Place them in the $preferences directory.
           You can also specify a template file for each ACTION if you wish,
           by editing the variable in the spec file action.

           Note: under normal circumstances, the "fields.html" should never
           be used, because you'll use a more specific file.

        3. Menu Control Button Variables & Form Defaults
           (list BEFORE html variables
           so that buttons and form defaults will be seen)

           These can be used for all of the 'spec' files, to provide
           menu button controls when you use '_external_template_' files.
           Note that the html display of the buttons is integrated from
           THREE sources: 1) this file 2) the 'spec' file 3) the template file

        4. Default Field Display Variables for Actions
           - these variables allow the 'spec' files to run,
             even without modification. If you desire, you can
             then create [table_name] specific variables in THIS FILE,
             or at the TOP of the spec file. If they exist, they will
             overwrite the defaults for the spec file for that table

             The syntax for the user defined table_name variables that
             match the above defaults are:

             . $variable_name_tbl_[table_name]

             ... where [table_name] (including the brackets) is replaced, e.g:
             . $custom_page_fields_tbl_users (where 'users' is the table name)

        5. Admin Menu Vars
             . $admin_menu_main_text (main menu for $w_user_type ne 'admin')
               (this is one example - see the &admin_menu subroutine
                in the fsql_setup.pl file)
             . $admin_menu_extra_text (extra functions for
                             advanced user $w_user_type ne 'admin')

        6. YOUR CUSTOMIZATONS - following the patterns above
           See note about where to put TABLE specific vars.
           I recommend the 'spec' files.

=cut

#############################################################################
#############################################################################

# 1. Previous and Next Buttons
#############################################################################

$previous_submit_button = qq~
<input type="image" src="$image_url/previous.gif" border=0 alt="Previous" name="Previous">
~;

$next_submit_button = qq~
<input type="image" src="$image_url/next.gif" border=0 alt="Next" name="Next">
~;

# OR... you can use regular submit button code
# $previous_submit_button = qq~
# <input type="submit" value=" Previous ">
# ~;

# $next_submit_button = qq~
# <input type="submit" value=" Next ">
# ~;

# header for prev/next buttons (for multiple page output of records)

$prev_next_header = qq~
<div align=center>
<table width="100%" border="0" cellspacing="0" cellpadding="0">
<tr>
<td></td>
~;

# Text for the Prev Page href
# NOTE! if you place a :!:previous_page:!: tag in your
# template file, then this $previous_page variable will
# be ignored, and the tag in the template file will be used
# instead. The tag is replaced with the complete <form> tag.
# If there is NO form generated, the tag is replaced with a ''.
# You can also use the :!:prev_next_header:!: and :!:prev_next_footer:!:
# tags in your template file - if you do, then the header and footer
# tags will be replaced with the variables used in this setup file.

$previous_page = qq~
<td width="10">&nbsp;</td>
<td valign=top align=left>
<font face="Times New Roman,Georgia,Times" size="2">
<br>
<b>:!:previous_page:!:</b>
</font>
</td>
~;

# Text for the Next Page href
# NOTE! if you place a :!:next_page:!: tag in your
# template file, then this $next_page variable will
# be ignored, and the tag in the template file will be used
# instead. The tag is replaced with the complete <form> tag.
# If there is NO form generated, the tag is replaced with a ''.

$next_page = qq~
<td valign=top align=right>
<font face="Times New Roman,Georgia,Times" size="2">
<br>
<b>:!:next_page:!:</b>
</font>
</td>
<td width="10">&nbsp;</td>
~;

# footer for prev/next buttons (for multiple page output of records)

$prev_next_footer = qq~
</tr>
</table>
</div>
~;

# 2. Default External Template
#############################################################################

$default_external_template = 'fields.html';

# 3. Menu Control Button Variables & Form Defaults
#############################################################################

# fast_search_form
#....................

$fast_search_form = qq~
<table border="0" cellspacing="0" cellpadding="0">
<tr>
<td>
<form method="post" action=":!:futuresql_url:!:"></td>
</tr>
<tr>
<td>
:!:security_fields:!:
<input type="hidden" name="s_" value=":!:s_:!:">
<input type="hidden" name="dc_" value=":!:dc_:!:">
<input type="hidden" name="p_" value=":!:p_:!:">
<input type="hidden" name="rt_" value=":!:rt_:!:">
<input type="hidden" name="w_" value="1">
<input type="hidden" name="a_" value="view_custom_list">
<input type="hidden" name="r_" value=":!:default_rows_per_page:!:">
<input type="hidden" name="wh_" value=":!:key_field:!: > 0">
&nbsp;
:!:fast_search_fields:!:
<input type="hidden" name="qbf" value="yes">
<input type="hidden" name="select_param" value="like">
<input type="hidden" name="show_sort_params" value="no">
<input type="hidden" name="show_select_params" value="no">
:!:fast_search_order:!:
<input type="image" src=":!:image_url:!:/search.gif" border=0 alt="Fast Search" name="Find">
<input type="hidden" name="custom_list_header" value="<b>Fast Search</b>, <i>Sorted by Last Name, First Name</i>">
</td>
</tr>
<tr>
<td></form></td>
</tr>
</table>
~;

#..........................................................................
# admin_little_menu_text is the menu used at the bottom of the display
# of various menu functions. Modify it at will, but keep the variable
# name 'admin_little_menu_text' intact
# you can use any :!:tag:!: that is parsed by the subroutine '&parse_params'

# the tag => :!:admin_menu:!: will display this menu in a custom_page

$admin_little_menu_text = qq~
<p>
<div align="center">
<table border="0">
<tr>
<td>
<font size=-1>
<form method="post" action=":!:futuresql_url:!:">
:!:system_hidden_fields:!:
<input type="hidden" name="w_"  value=":!:w_:!:">
<input type="hidden" name="a_"  value="top_menu">
<input type="hidden" name="rt_" value=":!:rt_:!:">
<input type="submit" value=" Top Menu ">
</form>
</font>
</td>
<td>
<font size=-1>
<form method="post" action=":!:futuresql_url:!:">
:!:system_hidden_fields:!:
<input type="hidden" name="w_"  value=":!:w_:!:">
<input type="hidden" name="a_"  value="view_admin_list">
<input type="hidden" name="rt_" value=":!:rt_:!:">
<input type="hidden" name="wh_" value=":!:wh_:!:">
<input type="hidden" name="or_" value=":!:or_:!:">
<input type="submit" value=" View Admin List ">
</form>
</font>
</td>
<td>
<font size=-1>
<form method="post" action=":!:futuresql_url:!:">
:!:system_hidden_fields:!:
<input type="hidden" name="w_"  value=":!:w_:!:">
<input type="hidden" name="a_"  value="query_by_form">
<input type="hidden" name="rt_" value=":!:rt_:!:">
<input type="submit" value=" Query by Form ">
</form>
</font>
</td>
<td>
<font size=-1>
<form method="post" action=":!:futuresql_url:!:">
:!:system_hidden_fields:!:
<input type="hidden" name="w_"  value=":!:w_:!:">
<input type="hidden" name="a_"  value="add_record">
<input type="hidden" name="rt_" value=":!:rt_:!:">
<input type="hidden" name="from_admin_list" value="yes">
<input type="submit" value=" Add Record ">
</form>
</font>
</td>
</tr>
</table>
</div>
~;    

#............................................................................

$menu_control_header = qq~
<table border=0 cellpadding=0 cellspacing=0>
<tr><td height="5"></td></tr>
<tr><td>
<FORM method="post" action="$script_name">
<input type="hidden" name="rt_" value=":!:rt_:!:">
<input type="hidden" name="w_" value=":!:key_field:!:">
:!:system_hidden_fields:!:
~;

#............................................................................

$menu_control_footer = qq~
</td><td></form></td>
</tr></table>
~;

#............................................................................

# use like this:
# $common_search_form_start
# <input type="hidden" name="wh_" value="$key_field > 0">
# <input type="hidden" name="or_" value="$key_field">
# <input type="image" src="$image_url/find_all.gif" border=0 alt="Find All" name="Find_All">
# </form>

$common_search_form_start = qq~
<FORM method="post" action="$script_name">
:!:security_fields:!:
<input type="hidden" name="s_" value=":!:s_:!:">
<input type="hidden" name="dc_" value=":!:dc_:!:">
<input type="hidden" name="p_" value="m">
<input type="hidden" name="rt_" value=":!:rt_:!:">
<input type="hidden" name="w_" value="1">
<input type="hidden" name="a_" value="view_custom_list">
<input type="hidden" name="r_" value="$default_rows_per_page">
~;

#............................................................................
# place next vars AFTER the menu control vars above
# in other words, vars used above have to be created first
# save, cancel, create, view, home, edit, find

        # RELEVANT ACTIONS (in spec files):
        # 'query_by_form_custom'        - (find)
        # 'view_custom_list'            - (displays results of find)
        # 'view_custom_record'          - (view)
        # 'edit_record'                 - (edit)
        # 'update_record'               - (save - used with edit)
        # 'add_record'                  - (create)
        # 'save_new_record'             - (save - used with add)
        # 'admin_functions'

        # LESS RELEVANT ACTIONS:
        # 'top_menu'
        # 'query_by_form'
        # 'public_functions'
        # 'view_admin_list'
        # 'view_record'
        # 'batch_process'
        # 'confirm_batch_process'

#!!! Note that :!:key_field:!: gets replaced with the VALUE in the key field
#    while $key_field gets replaced with the NAME of the key field
#!!! THUS... this file must be loaded AFTER the dictionary file, where the
#    key field is defined.
#............................................................................
# sample:
# $home_menu $create_menu $edit_menu $find_menu
# paste the line above into the menu_control_text key, e.g:

# 'menu_control_text'    => qq~
#                           $home_menu $create_menu $edit_menu $find_menu
#                           ~,

# ... and/or alter as necessary
#............................................................................

$home_menu = qq~
<td valign=bottom align=right width="$button_cell_width">
<table border=0 cellpadding=0 cellspacing=0>
<tr><td height="5"></td></tr>
<tr><td>
<FORM method="post" action="$script_name">
<input type="hidden" name="rt_" value="t2">
<input type="hidden" name="w_" value=":!:key_field:!:">
:!:system_hidden_fields:!:
<input type="hidden" name="a_"  value="admin_functions_._admin_menu">
<input type="image" src="$image_url/home2.gif" border=0 alt="Home Menu" name="Home">
$menu_control_footer
</td>
~;

#............................................................................

$create_menu = qq~
<td valign=bottom align=right width="$button_cell_width">
$menu_control_header
<input type="hidden" name="a_" value="add_record">
<input type="image" src="$image_url/create.gif" border=0 alt="Create Record" name="create">
$menu_control_footer
</td>
~;

#............................................................................

$edit_menu = qq~
<td valign=bottom align=right width="$button_cell_width">
$menu_control_header
<input type="hidden" name="a_" value="edit_record">
<input type="image" src="$image_url/edit.gif" border=0 alt="Edit" name="edit">
$menu_control_footer
</td>
~;

#............................................................................

$find_menu = qq~
<td valign=bottom align=right width="$button_cell_width">

<table border=0 cellpadding=0 cellspacing=0>
<tr><td height="5"></td></tr>
<tr><td>
<FORM method="post" action="$script_name">
<input type="hidden" name="rt_" value="t1">
<input type="hidden" name="w_" value=":!:key_field:!:">
:!:system_hidden_fields:!:

<input type="hidden" name="a_" value="query_by_form_custom">
<input type="image" src="$image_url/find.gif" border=0 alt="Find" name="Find">
$menu_control_footer
</td>
~;

#............................................................................
# use find_all_menu when you want to skip the find and go to view_custom_list
# with all the records

# syntax for displaying special list file
# <input type="hidden" name="a_" value="view_custom_list_._02">

$special_sort_order = $key_field;
if ( $table_name eq 'readers' ){$special_sort_order = 'last_name';}

$find_all_menu = qq~
<td valign=bottom align=right width="$button_cell_width">
<table border=0 cellpadding=0 cellspacing=0>
<tr><td height="5"></td></tr>
<tr><td>
<FORM method="post" action="$script_name">
:!:security_fields:!:
<input type="hidden" name="s_" value=":!:s_:!:">
<input type="hidden" name="dc_" value=":!:dc_:!:">
<input type="hidden" name="p_" value="m">
<input type="hidden" name="rt_" value="t1">
<input type="hidden" name="w_" value=":!:key_field:!:">
<input type="hidden" name="a_" value="view_custom_list">
<input type="hidden" name="r_" value="$default_rows_per_page">
<input type="hidden" name="wh_" value="$key_field > 0">
<input type="hidden" name="or_" value="$special_sort_order">
<input type="image" src="$image_url/find.gif" border=0 alt="Find" name="Find">
</td><td></form></td>
</tr></table>
</td>
~;

#............................................................................
### special example loop for find_menu
### so that you can bypass the qbf for some tables

# if ( $table_name eq 'users' )
#         { $find_menu_text = $find_all_menu; }
# else
#         {

$find_menu_text = $find_menu;

# }
#............................................................................

$view_menu = qq~
<td valign=bottom align=right width="$button_cell_width">
$menu_control_header
<input type="hidden" name="a_" value="view_custom_record">
<input type="image" src="$image_url/view.gif" border=0 alt="View" name="View">
$menu_control_footer
</td>
~;

#............................................................................

if    ( $table_name eq 'authors' )
        {$view_custom_records_sort_order = qq~record_updated desc~;}
elsif ( $table_name eq 'books' )
        {
        # we use 'b_record_updated' because we join 3 tables
        # when we view the custom record (see the docs)
        $view_custom_records_sort_order = qq~b.record_updated desc~;
        }
else
        {$view_custom_records_sort_order = $default_custom_records_sort_order;}

$view_custom_records_menu = qq~
<td valign=bottom align=right width="$button_cell_width">
<table border=0 cellpadding=0 cellspacing=0>
<tr><td height="5"></td></tr>
<tr><td>
<FORM method="post" action="$script_name">
:!:security_fields:!:
<input type="hidden" name="s_" value=":!:s_:!:">
<input type="hidden" name="dc_" value=":!:dc_:!:">
<input type="hidden" name="p_" value="m">
<input type="hidden" name="r_" value="1">
<input type="hidden" name="rt_" value=":!:rt_:!:">
<input type="hidden" name="w_" value=":!:key_field:!:">
<input type="hidden" name="a_" value="view_custom_record">
<input type="hidden" name="wh_" value="$key_field > 0">
<input type="hidden" name="or_" value="$view_custom_records_sort_order">
<input type="image" src="$image_url/view.gif" border=0 alt="View Records" name="View">
</td><td></form></td>
</tr></table>
</td>
~;

#............................................................................

$cancel_menu = qq~
<table border=0 cellpadding=0 cellspacing=0>
<tr><td>
<FORM method="post" action="$script_name">
</td>
<td>
:!:security_fields:!:
<input type="hidden" name="s_" value=":!:s_:!:">
<input type="hidden" name="dc_" value=":!:dc_:!:">
<input type="hidden" name="p_" value="m">
<input type="hidden" name="r_" value="1">
<input type="hidden" name="rt_" value=":!:rt_:!:">
<input type="hidden" name="w_" value=":!:key_field:!:">
<input type="hidden" name="a_" value="view_custom_record">
<input type="hidden" name="wh_" value="$key_field > 0">
<input type="hidden" name="or_" value="$view_custom_records_sort_order">
<input type="image" src="$image_url/cancel.gif" border=0 alt="Cancel" name="Cancel">
</td>
<td></form></td>
</tr></table>
~;

#............................................................................

$save_button = qq~
<input type="image" src="$image_url/save.gif" border=0 alt="Save" name="Save">
~;

$search_button = qq~
<input type="image" src="$image_url/search.gif" border=0 alt="Search" name="Search">
~;

#............................................................................
# special var: $cancel_form_start_save_combo
# place this in the 'page_form_start' key for 'add_record' and 'edit_record'
# this interacts with the fields template file to
# place a 'cancel' button AND!!! begin the form for the regular save
# and then place a 'save' button at the top of the form

$cancel_form_start_save_combo = qq~
<table width="100%" border="0" cellpadding="0" cellspacing="0" bgcolor="$td_bg">
<tr bgcolor="$td_bg">

<td valign=top align=right width="$button_cell_width">
<FORM method="post" action="$script_name">
:!:security_fields:!:
<input type="hidden" name="s_" value=":!:s_:!:">
<input type="hidden" name="dc_" value=":!:dc_:!:">
<input type="hidden" name="p_" value="m">
<input type="hidden" name="rt_" value=":!:rt_:!:">
<input type="hidden" name="r_" value="1">
<input type="hidden" name="wh_" value="$key_field > 0">
<input type="hidden" name="or_" value="$view_custom_records_sort_order">
<input type="hidden" name="a_" value="view_custom_record">
<input type="image" src="$image_url/cancel.gif" border=0 alt="Cancel" name="Cancel">
</td>

<td valign=top align=left width="1" bgcolor="$td_bg">
</form>&nbsp;
</td>

<td valign=top align=left width="3" bgcolor="$td_bg">
<form method="post" action="$futuresql_url">&nbsp;
</td>

<td valign=top align=left width="$button_cell_width">
$save_button
</td>

<td valign=top align=left width="500">
&nbsp;
</td>
</tr>
</table>
~;

#............................................................................

# 4. Default Field Display Variables for Actions
#############################################################################

# ADMIN MENU BODY VAR
#............................................................................

$admin_menu_body_text = &include_file("$template_dir/fsql_admin_menu_template.html");

# CUSTOM PAGE FIELD VAR
#............................................................................
# Note table row syntax:
# <tr><td bgcolor="$lbl_bg" align=right>Textual Field Label</td><td bgcolor="$fld_bg">:!:technical_field_name:!:</td></tr>

$custom_page_fields = qq~
<table width="80%" border=1 cellpadding=5>
<tr><td bgcolor="$lbl_bg" align=right>$key_field</td><td bgcolor="$fld_bg">:!:key_field:!:</td></tr>
</table>
~;

# $custom_page_fields_tbl_users
$custom_fields = qq~This may be deprecated.~;

# QUERY BY FORM (STANDARD) VARS
#...........................................................................

$query_by_form_table_start = qq~
<table border="2" cellpadding="4" cellspacing="0" width="100%">
<tr bgcolor="$td_bg">
<td colspan="2" valign="top">
<font size=+1>Search for Data in [$ucfirst_table] using Query by Form</font><br>
<font size=-1>
Type values in the fields you want to query. Change 'ignore'
to one of the search types for each used field. Note that
fields are searched using the logic of 'AND'.<br>
e.g: select records where (field.1 = 'a') AND (field.2 = 'b')
</font>
</td>
</tr>
<tr>
<td colspan="2">
<form method="post" action="$futuresql_url">
</td>
</tr>
~;

#...........................................................................

$query_by_form_record_text = qq~
<tr>
<td width="150" valign="top">
<font face="Times New Roman,Georgia,Times"><b>
:!:field_label:!:
</b></font>
</td>
<td valign="top">
<font face="Times New Roman,Georgia,Times">
:!:field_value:!:
</font>
</td>
</tr>
~;

#...........................................................................

$query_by_form_table_end = qq~
<tr>
<td colspan="2">
<font face="Times New Roman,Georgia,Times" size=-1>
:!:system_hidden_fields:!:
<input type="hidden" name="rt_" value="$rt_">
<input type="hidden" name="w_" value="$w_">
<input type="hidden" name="qbf" value="yes">
<input type="hidden" name="a_" value="view_admin_list">
<div align=right>
:!:order_fields:!:
</div>
<input type="submit" value=" View Search Results ">
</font>
</form>
</td>
</tr>
</table>
~;

# PUBLIC FUNCTION RESPONSE VAR
#...........................................................................

$public_functions_response_body = qq~
<table border="2" cellpadding="10" cellspacing="0" width="100%">
<tr>
<td valign=top><font face="Times New Roman,Georgia,Times">
<br>
<br>
:!:response_text:!:
<br>
<br>
</font></td>
</tr>
</table>
~;

# VIEW ADMIN LIST VARS
#...........................................................................

$view_admin_list_table_start = qq~
<table border="2" cellpadding="4" cellspacing="0" width="100%">
<tr>
<td colspan=:!:colspan:!: align=center bgcolor="$td_bg">
<font size=-1 face="Times New Roman,Georgia,Times"><b>
Select One or More Records, and then click either<br>
View / Edit / or Batch Process</b><br>
<i>\~ selecting batch will take you to a "process menu" \~<br>
(checkboxed records are only used by batch process)
</i></font>
</td>
</tr>
<tr>
<td colspan=:!:colspan:!:>
<form method="post" action="$futuresql_url">
</td>
</tr>
<tr>
<td>
<font face="Times New Roman,Georgia,Times"><b>
Action
</b></font>
</td>
~;

#...........................................................................

$view_admin_list_table_header = qq~
<td>
<font face="Times New Roman,Georgia,Times"><b>
:!:field_label:!:
</b></font>
</td>
~;

#...........................................................................

$view_admin_list_record_text = qq~
</tr>
<tr>
<td>
<font face="Times New Roman,Georgia,Times" size=-1>
:!:system_hidden_fields:!:
<input type="hidden" name="rt_" value="$rt_">
<input type="radio"    name="w_" value=":!:key_field:!:">
<input type="checkbox" name="w_array" value=":!:key_field:!:">
</font>
</td>
~;

#...........................................................................

$view_admin_list_record_cols = qq~
<td><font size="2" face="Times New Roman,Georgia,Times">
:!:field_value:!:
</font></td>
~;

#...........................................................................

$view_admin_list_table_end = qq~
</tr>
<tr>
<td colspan=:!:colspan:!: align=center>
<font size=-1>
<input type="submit" name="a_" value="view_record">
<input type="submit" name="a_" value="view_custom_record">
<input type="submit" name="a_" value="edit_record">
<input type="hidden" name="from_admin_list" value="yes">
<input type="submit" name="a_" value="batch_process"><br>
Process ALL Records on Page? <input type="checkbox" name="batch_type" value="all">
<input type="hidden" name="w_array_all" value=":!:w_array_all:!:">
</font>
</td>
</tr>
<tr>
<td colspan=:!:colspan:!: align=center>
</form>
</td>
</tr>
</table>
~;

# MENU CONTROL VARS
#...........................................................................

$view_custom_record_menu_control_text = qq~
$home_menu $create_menu $edit_menu $find_menu_text
~;

#...........................................................................

$edit_record_menu_control_text = qq~
<td><img src="$image_url/edit.gif"></td>
~;

#...........................................................................

# $update_record_menu_control_text = qq~
# $home_menu $create_menu $edit_menu $find_menu_text
# ~;

$update_record_menu_control_text = qq~
$view_custom_records_menu
~;

#...........................................................................

$add_record_menu_control_text = qq~
<td><img src="$image_url/create.gif"></td>
~;

#...........................................................................

# $save_new_record_menu_control_text = qq~
# $home_menu $create_menu $edit_menu $find_menu_text
# ~;

$save_new_record_menu_control_text = qq~
$view_custom_records_menu
~;

# VIEW RECORD VARS
#...........................................................................

$view_record_table_start = qq~
<table border="2" cellpadding="4" cellspacing="0" width="100%">
<tr bgcolor="$td_bg">
<td colspan="2" valign="top">
<font size=+1>Record Data</font>
</td>
</tr>
<tr>
<td colspan="2">
</td>
</tr>
~;

#...........................................................................

$view_record_record_text = qq~
<tr>
<td width="150" valign="top" bgcolor="$td_bg">
<font face="Times New Roman,Georgia,Times"><b>
:!:field_label:!:
</b></font>
</td>
<td valign="top">
<font face="Times New Roman,Georgia,Times">
:!:field_value:!:
</font>
</td>
</tr>
~;

#...........................................................................

$view_record_table_end = qq~
</table>
~;

# VIEW CUSTOM RECORD VARS
#...........................................................................

$view_custom_record_page_title = qq~Viewing Record~;

#...........................................................................

$view_custom_record_page_header = qq~
<font size=+1>
<b>Viewing Record</b>
</font>
~;

#...........................................................................

$view_custom_record_custom_page = qq~
:!:page_header:!:
:!:menu_control_text:!:
:!:page_form_start:!:
<blockquote>
:!:custom_page_fields:!:
<p>
:!:page_form_system_fields:!:
:!:page_form_end:!:
</blockquote>
:!:page_footer:!:
~;

#...........................................................................

$view_custom_record_page_form_start = qq~~;

#...........................................................................

$view_custom_record_page_form_system_fields = qq~~;

#...........................................................................

$view_custom_record_page_form_end = qq~~;

#...........................................................................

$view_custom_record_page_footer = qq~~;

#...........................................................................

$view_custom_record_table_start = qq~
<table border="0" cellpadding="4" cellspacing="0" width="100%">
~;

#...........................................................................

$view_custom_record_record_text = qq~
<tr>
<td width="150" valign="top" bgcolor="$td_bg">
<FONT COLOR="#FF0000" SIZE="-1" FACE="Arial,Helvetica"><B>
place fields here<br>
</FONT></B>
<p>
<FONT COLOR="#000080">
</td>
</tr>
~;

#...........................................................................

$view_custom_record_table_end = qq~
</table>
~;

# BATCH PROCESS VARS
#...........................................................................

$batch_process_table_start = qq~
<table border="2" cellpadding="4" cellspacing="0" width="100%">
<tr>
<td colspan=:!:colspan:!: align=center bgcolor="$td_bg">
<font size=-1 face="Times New Roman,Georgia,Times"><b>
Batch Process</b><br>
<i>\~ You have selected the following records for processing: \~<br>
- - - Click "Process Type" and "Confirm Batch Process" to Continue - - -</i>
</font>
</td>
</tr>
<tr>
<td colspan=:!:colspan:!:>
<form method="post" action=":!:futuresql_url:!:">
</td>
</tr>
<tr>
<td></td>
~;

#...........................................................................

$batch_process_table_header = qq~
<td>
<font face="Times New Roman,Georgia,Times"><b>
:!:field_label:!:
</b></font>
</td>
~;

#...........................................................................

$batch_process_record_text = qq~
<td></td>
</tr>
<tr>
<td>
<font size="2" face="Times New Roman,Georgia,Times">
:!:system_hidden_fields:!:
<input type="hidden" name="rt_" value=":!:rt_:!:">
<input type="hidden" name="w_array" value=":!:key_field:!:">
</font>
</td>
~;

#...........................................................................

$batch_process_record_cols = qq~
<td>
<font size="2" face="Times New Roman,Georgia,Times">
:!:field_value:!:
</font>
</td>
~;

#...........................................................................

$batch_process_table_end = qq~
<td></td>
</tr>
<tr>
<td colspan=:!:colspan:!: align=center>
<font size=-1>
[ <input type="radio" name="process_name" value="delete" checked> Delete ]
[ <input type="radio" name="process_name" value="sample_process"> Sample Process (displays records) ]
<!--
[ <input type="radio" name="process_name" value="approve"> Approve ]
-->
<br>
<input type="submit" name="a_" value="confirm_batch_process">
</font>
</td>
</tr>
<tr>
<td colspan=:!:colspan:!: align=center>
</form>
</td>
</tr>
</table>
~;

# CONFIRM BATCH PROCESS VARS
#...........................................................................

$confirm_batch_process_table_start = qq~
<table border="2" cellpadding="4" cellspacing="0" width="100%">
<tr>
<td colspan=:!:colspan:!: align=center bgcolor="$td_bg">
<font size=-1 face="Times New Roman,Georgia,Times"><b>
Batch [:!:process_name:!:] in Progress</b><br>
<i>\~ the following records are being processed: \~
</font>
</td>
</tr>
<tr>
~;

#...........................................................................

$confirm_batch_process_table_header = qq~
<td>
<font face="Times New Roman,Georgia,Times"><b>
:!:field_label:!:
</b></font>
</td>
~;

#...........................................................................

$confirm_batch_process_record_text = qq~
</tr>
<tr>
~;

#...........................................................................

$confirm_batch_process_record_cols = qq~
<td>
<font size="2" face="Times New Roman,Georgia,Times">
:!:field_value:!:
</font>
</td>
~;

#...........................................................................

$confirm_batch_process_table_end = qq~
</tr>
<tr>
<td colspan=:!:colspan:!: align=center bgcolor="$td_bg">
<font size=-1 face="Times New Roman,Georgia,Times"><b>
Batch [:!:process_name:!:] Finished!</b><br>
:!:batch_response:!:
</font>
</td>
</tr>
</table>
~;

#...........................................................................
# EDIT RECORD VARS

$edit_record_page_title = qq~
Edit a Record
~;

#...........................................................................

$edit_record_page_header = qq~
<font size=+1>
<b>Edit a Record</b>
</font>
~;

#...........................................................................

$edit_record_page_form_start = qq~
$cancel_form_start_save_combo
~;

#...........................................................................

$edit_record_custom_page = qq~
:!:page_header:!:
:!:menu_control_text:!:
:!:page_form_start:!:
<p>
<blockquote>
:!:custom_page_fields:!:
<p>
:!:missing_fields_made_hidden:!:
:!:page_form_system_fields:!:
:!:page_form_end:!:
</blockquote>
:!:page_footer:!:
~;

#...........................................................................

$edit_record_page_form_system_fields = qq~
<font face="Times New Roman,Georgia,Times" size=-1>
:!:system_hidden_fields:!:
<input type="hidden" name="rt_" value=":!:rt_:!:">
<input type="hidden" name="w_" value=":!:key_field:!:">
<input type="hidden" name="a_" value="update_record">
$save_button
</font>
~;

#...........................................................................

$edit_record_page_form_end = qq~
</form>
~;

#...........................................................................

$edit_record_page_footer = qq~
<br>
~;

#...........................................................................

$edit_record_table_start = qq~
<table border="2" cellpadding="4" cellspacing="0" width="100%">
<tr bgcolor="$td_bg">
<td colspan="2" valign="top">
<font size=+1>Edit Record</font>
</td>
</tr>
<tr>
<td colspan="2">
<form method="post" action="$futuresql_url">
</td>
</tr>
~;

#...........................................................................

$edit_record_record_text = qq~
<tr>
<td width="150" valign="top" bgcolor="$td_bg">
<font face="Times New Roman,Georgia,Times"><b>
:!:field_label:!:
</b></font>
</td>
<td valign="top">
<font face="Times New Roman,Georgia,Times">
:!:field_value:!:
</font>
</td>
</tr>
~;

#...........................................................................

$edit_record_table_end = qq~
<tr>
<td colspan="2">
<font face="Times New Roman,Georgia,Times" size=-1>
:!:system_hidden_fields:!:
<input type="hidden" name="rt_" value=":!:rt_:!:">
<input type="hidden" name="w_" value=":!:key_field:!:">
<input type="hidden" name="from_admin_list" value="yes">
<input type="hidden" name="a_" value="update_record">
<input type="submit" value=" Save Record Update ">
</font>
</form>
</td>
</tr>
</table>
~;

#...........................................................................
# UPDATE RECORD VARS

$update_record_page_title = qq~
Record Updated
~;

#...........................................................................

$update_record_page_header = qq~
<font size=+1>
<b>Record Updated</b>
</font>
<font size=-1 color='red'><i>
&nbsp;&nbsp;&nbsp; :: Click VIEW to leave Update Mode =></i></font>
~;

#...........................................................................

$update_record_page_form_start = qq~
~;

#...........................................................................

$update_record_custom_page = qq~
:!:page_header:!:
:!:menu_control_text:!:
:!:page_form_start:!:
<blockquote>
:!:custom_page_fields:!:
<p>
:!:page_form_system_fields:!:
:!:page_form_end:!:
</blockquote>
:!:page_footer:!:
~;

#...........................................................................

$update_record_page_form_system_fields = qq~
~;

#...........................................................................

$update_record_page_form_end = qq~
~;

#...........................................................................

$update_record_page_footer = qq~
~;

#...........................................................................

$update_record_table_start = qq~
<table border="2" cellpadding="4" cellspacing="0" width="100%">
<tr bgcolor="$td_bg">
<td colspan="2" valign="top">
<font size=+1>Record Updated</font>
</td>
</tr>
~;

#...........................................................................

$update_record_record_text = qq~
<tr>
<td width="150" valign="top">
<font face="Times New Roman,Georgia,Times"><b>
:!:field_label:!:
</b></font>
</td>
<td valign="top">
<font face="Times New Roman,Georgia,Times">
:!:field_value:!:
</font>
</td>
</tr>
~;

#...........................................................................

$update_record_table_end = qq~
</table>
~;

#...........................................................................
# ADD RECORD VARS

$add_record_page_title = qq~
Add a Record
~;

#...........................................................................

$add_record_page_header = qq~
<font size=+1>
<b>Add a Record</b>
</font>
~;

#...........................................................................

$add_record_page_form_start = qq~
$cancel_form_start_save_combo
~;

#...........................................................................

$add_record_custom_page = qq~
:!:page_header:!:
:!:menu_control_text:!:
:!:page_form_start:!:
<p>
<blockquote>
:!:custom_page_fields:!:
<p>
:!:missing_fields_made_hidden:!:
:!:page_form_system_fields:!:
:!:page_form_end:!:
</blockquote>
:!:page_footer:!:
~;

#...........................................................................

$add_record_page_form_system_fields = qq~
<font face="Times New Roman,Georgia,Times" size=-1>
:!:system_hidden_fields:!:
<input type="hidden" name="rt_" value=":!:rt_:!:">
<input type="hidden" name="w_" value="0">
<input type="hidden" name="a_" value="save_new_record">
$save_button
</font>
~;

#...........................................................................

$add_record_page_form_end = qq~
</form>
~;

#...........................................................................

$add_record_page_footer = qq~
<br>
~;

#...........................................................................

$add_record_table_start = qq~
<table border="2" cellpadding="4" cellspacing="0" width="100%">
<tr bgcolor="$td_bg">
<td colspan="2" valign="top">
<font size=+1>Add New Record</font>
</td>
</tr>
<tr>
<td colspan="2">
<form method="post" action=":!:futuresql_url:!:">
</td>
</tr>
~;

#...........................................................................

$add_record_record_text = qq~
<tr>
<td width="150" valign="top">
<font face="Times New Roman,Georgia,Times"><b>
:!:field_label:!:
</b></font>
</td>
<td valign="top">
<font face="Times New Roman,Georgia,Times">
:!:field_value:!:
</font>
</td>
</tr>
~;

#...........................................................................

$add_record_table_end = qq~
<tr>
<td colspan="2">
<font face="Times New Roman,Georgia,Times" size=-1>
:!:system_hidden_fields:!:
<input type="hidden" name="rt_" value=":!:rt_:!:">
<input type="hidden" name="w_" value="0">
<input type="hidden" name="a_" value="save_new_record">
<input type="submit" value=" Save New Record ">
</font>
</form>
</td>
</tr>
</table>
~;

#...........................................................................
# SAVE NEW RECORD VARS

$save_new_record_page_title = qq~
Record Saved
~;

#...........................................................................

$save_new_record_page_header = qq~
<font size=+1>
<b>Record Saved</b>
</font>
<font size=-1 color='red'><i>
&nbsp;&nbsp;&nbsp; :: Click VIEW to leave Save Mode =></i></font>
~;

#...........................................................................

$save_new_record_page_form_start = qq~
~;

#...........................................................................

$save_new_record_custom_page = qq~
:!:page_header:!:
:!:menu_control_text:!:
:!:page_form_start:!:
<blockquote>
:!:custom_page_fields:!:
<p>
:!:page_form_system_fields:!:
:!:page_form_end:!:
</blockquote>
:!:copyright:!:
:!:admin_menu:!:
:!:page_footer:!:
~;

#...........................................................................

$save_new_record_page_form_system_fields = qq~
~;

#...........................................................................

$save_new_record_page_form_end = qq~
~;

#...........................................................................

$save_new_record_page_footer = qq~
~;

#...........................................................................

$save_new_record_table_start = qq~
<table border="2" cellpadding="4" cellspacing="0" width="100%">
<tr bgcolor="$td_bg">
<td colspan="2" valign="top">
<font size=+1>New Record Added</font>
</td>
</tr>
~;

#...........................................................................

$save_new_record_record_text = qq~
<tr>
<td width="150" valign="top">
<font face="Times New Roman,Georgia,Times"><b>
:!:field_label:!:
</b></font>
</td>
<td valign="top">
<font face="Times New Roman,Georgia,Times">
:!:field_value:!:
</font>
</td>
</tr>
~;

#...........................................................................

$save_new_record_table_end = qq~
</table>
~;

#...........................................................................

# CUSTOM VARS (see hash at bottom of file)
############################################################################

$query_by_form_custom_table_start = qq~
<table border="0" cellpadding="4" cellspacing="0" width="92%">
<tr>

<td>

<table border="0" width="100%">
<tr>

<td align=left width="30%" bgcolor="$td_bg" colspan=3>
<font size=+1 face="Arial, Helvetica, sans-serif" color="darkblue">
<b>$ucfirst_table Table - Custom Find & Query by Form</b>
</font>
</td>

<!--
<td width="1" bgcolor=""></td>
<td align=right bgcolor="">
<img src="$image_url/find.gif" alt="Find">
</td>
 -->

</tr>
</table>

</td></tr>
<tr>
<td valign=top align=left colspan=2>
<!--
COMMENT
You can use href links like this, or forms as is done below.
Note that if you want to use this function for an ADMIN level,
then you have to pass the connection security information, thus
it would be better to use a FORM, so that the data doesn't show up
on the url line.
This link is a sample of a way to access a public function via a url.
[ e.g: <a href="futuresql.cgi?s_=users.1&dc_=users.1&r_=10&p_=m&rt_=t1&user_level=public&w_=add&a_=public_functions_._user_login_page">Add or Access a Record</a> ]
END OF COMMENT
-->
</td></tr>
</table>
~;

#............................................................................

$query_by_form_custom_record_text = qq~
<table border="0" cellpadding="4" cellspacing="0" width="80%">
<tr>
<td valign=top>
$common_search_form_start
<input type="hidden" name="wh_" value="$key_field > 0">
<input type="hidden" name="or_" value="$key_field">
<input type="image" src="$image_url/arrow.gif" border=0 alt="Find All" name="Find_All">
Find All
</form>
</td>
</tr>
<tr bgcolor="$td_bg">
<td colspan="2" valign="top">
<font size=+1>Search for Data using Query by Form</font><br>
</td>
</tr>

<tr>
<td valign="top" align="left" width="$button_cell_width">
$cancel_menu
</td>
<td valign="top" align="left">
<form method="post" action="$futuresql_url">
$search_button
</td>
</tr>

<tr>
<td width="20%" valign="top">
<font face="Times New Roman,Georgia,Times">
ID:
</font>
</td>
<td valign="top">
<font face="Times New Roman,Georgia,Times">
 :!:$key_field:!:
</font>
</td>
</tr>
~;

#............................................................................

$query_by_form_custom_table_end = qq~
<tr>
<td>&nbsp;</td>
<td>
<font face="Times New Roman,Georgia,Times" size=-1>
:!:system_hidden_fields:!:
<input type="hidden" name="rt_" value="$rt_">
<input type="hidden" name="w_" value="$w_">
<input type="hidden" name="qbf" value="yes">
<input type="hidden" name="a_" value="view_custom_list">
:!:search_sort_params:!:
<div align=left>
:!:order_fields:!:
</div>
$search_button
</font>
</form>
</td>
</tr>
</table>
~;

#............................................................................

$view_custom_list_table_start = qq~
<table border="0" cellpadding="4" cellspacing="0" width="100%">
<tr bgcolor="$td_bg">
<td>
<font size=-1 face="Times New Roman,Georgia,Times"><b>
Select
</b></font>
</td>
<td></td>
<td>
<font size=-1 face="Times New Roman,Georgia,Times"><b>
ID
</b></font>
</td>
</tr>
<tr>
<td colspan=3>
</td>
</tr>
~;

#............................................................................

$view_custom_list_record_text = qq~
<tr>
<td colspan=1>
<FONT COLOR="#FF0000" SIZE="-1" FACE="Arial,Helvetica"><B>
/ ID :!:$key_field:!: /
</FONT></B>
<br>
<font size=-1>
<form method="post" action="$futuresql_url">
:!:system_hidden_fields:!:
<input type="hidden" name="rt_" value="$rt_">
<input type="hidden" name="w_" value=":!:key_field:!:">
<input type="hidden" name="a_" value="view_custom_record">
<input type="submit" value="View Details">
</form>
</font>
<hr>
</td>
</tr>
~;

#............................................................................

$view_custom_list_table_end = qq~
</table>
~;

#............................................................................

# 5. Admin Menu Vars
#############################################################################

# ADMIN FUNCTION RESPONSE VAR
#...........................................................................
# this var is simple because we also have an admin menu var
# (not to be confused with the complex administration page)

$admin_functions_response_body = qq~
:!:response_text:!:
$copyright
~;

#...........................................................................

if ( -e "$template_dir/admin_menu_main_text.html" )
        {
        $admin_menu_main_text = &include_file("$template_dir/admin_menu_main_text.html");
        }
else
        {
        $admin_menu_main_text = qq~
        Place Your Home Menu Functions Here - or use an HTML file.
        ~;
}

#............................................................................

if ( -e "$template_dir/admin_menu_extra_text.html" )
        {
        $admin_menu_extra_text = &include_file("$template_dir/admin_menu_extra_text.html");
        }
else
        {
        $admin_menu_extra_text = qq~
        <hr>
        Place Your Super User Home Menu Functions Here,
        or use an HTML file.

        Note that you'll have to change the hidden fields in the form below.

        <FORM method="post" action=":!:futuresql_url:!:">
        :!:security_fields:!:
        <input type="hidden" name="s_" value="u.1">
        <input type="hidden" name="dc_" value="u.1">
        <input type="hidden" name="r_" value=":!:default_rows_per_page:!:">
        <input type="hidden" name="p_" value="m">
        <input type="hidden" name="rt_" value="t1">
        <input type="hidden" name="w_" value="1">
        <input type="hidden" name="a_" value="top_menu">
        <input type="submit" value="Administration Menu">
        </form>
        ~;
        }

#............................................................................

# 6. YOUR CUSTOMIZATONS - following the patterns above
#    Default Field Display Variables for Actions
#############################################################################
#############################################################################
# create your secondary vars below, or place the values in a FILE
# (you can then design the page in an external program)
# this also prevents a lot of html from being loaded in the hash below
# when it might not be necessary.

# => you can also leave any or all of the values BLANK, and set to NO
# => and then the DEFAULTS (above) will be used.
#............................................................................
# !!! NOTE !!! Some of the vars below can and should use different variables
#              that have been declared PRIOR to the initialization of the
#              variables below. IF you use include files, then the
#              PREVIOUSLY declared variables
#              won't be available UNLESS you do a replace of a TAG.

# note that $key_field can also be wrapped with :!: tags, e.g: :!:$key_field:!:
# so that in your file, it would look like this: :!::!:key_field:!::!:

# ALSO note that in Netscape, tables that don't end properly (as many of
# the tables here do not, will not display. However, most will display in IE.
# We don't end all of the tables, because they're integrated with other
# end tags elsewhere in the program.

# NOTE ON TABLE SPECIFIC VARS

# You define the vars here, but if you have many tables, there's usually
# no reason to load the vars for all of the tables, since only one
# table is accessed at once.

# Thus, it's more efficient to TURN ON table specific vars at the
# top of the 'specvar' files, which get loaded just before the
# 'spec' files. It's better to put them in the 'specvar' files,
# rather than the 'spec' file, because you may not have a 'spec'
# file for each table - unless you're customizing them.

# CUSTOM_HTML TAGS
# ...............................
# these are the tags that are replaced in the custom html values
# each value here can be placed in the html using the :!:tag:!: delimiters
# they are then replaced by the $tag variable value.
# => you can add tags as you wish.
# => if your html doesn't include a tag, the tag is ignored.
# => Note! A number of system tags are replaced deeper in the script,
#          in the parse_params subroutine, and should not be replaced here.
# ...............................

@custom_html_tags = qw[
                       key_field      default_rows_per_page
                       lbl_bg         fld_bg
                       home_menu      save_menu       cancel_menu
                       find_menu      find_all_menu   create_menu
                       edit_menu      view_menu       search_button
                       common_search_form_start
                      ];

@custom_prev_next_tags = qw[
                       custom_list_header
                      ];

# CUSTOM_HTML VALUES
# ...............................
# these are the defaults that will be used for all tables UNLESS you overwrite
# these values by placing copies of these arrays in the 'specvar' files
# => Thus, you normally would set them to 'no' (n) here, and turn them on for
#    individual tables, unless you want them on for all tables.

# note the usage of the $table_name variable to create different variable
# names depending on the table in use.
# => one easy method is to just copy the 'usage' hash, and set various keys
#    to 'y' or 'n'

# => Note that you can copy JUST THE KEYS you want to the specvar file,
#    and the program will only look at those keys.

# => also note that you can use a FILE NAME, or just place the html in the
#    html key value.
# => If you use a FILE NAME, then a different value will be loaded for
#    each table (in other words, the physical file name will have to have
#    the table name in it.)
# => If you place the html in the hash, the same html will be used for all
#    tables.
#..........................................
# => Note that the two hashes are split below, even though it might seem
#    cleaner (with less duplication) to combine them, for two reasons:
#    1. It's easier to copy the 'usage' hash by itself, to a specvar file
#    2. By copying the 'custom_html' hash to a specvar file, you could
#       completely customize the values because it would be redeclared.
#..........................................

# VARIABLE NAME SYNTAX: $variable_name_tbl_[table_name]
# (the variable name is 'split' on '_tbl_', so that the program can
# copy the custom value into the 'default' variable (named on the
# left side of '_tbl_').


# Note again that a preferred method is to set these to 'n' (no) here,
# and turn them on with a 'y' (yes) in the specvar files, as you need them.

# Whichever ones you use, of course, need corresponding values in the
# %custom_html hash below
#............................................................................

%custom_html_usage = (
             "query_by_form_custom_table_start_tbl_$table_name" => 'n',
             "query_by_form_custom_record_text_tbl_$table_name" => 'n',
             "query_by_form_custom_table_end_tbl_$table_name"   => 'n',

             "view_custom_list_table_start_tbl_$table_name"     => 'n',
             "view_custom_list_record_text_tbl_$table_name"     => 'n',
             "view_custom_list_table_end_tbl_$table_name"       => 'n',

             # custom page fields is not needed if you
             # use external templates or the standard method
             # the standard method is important, because it
             # allows things to work if neither the custom page or
             # external template methods are defined

             # => this may be deprecated (below)

             # "custom_page_fields_tbl_$table_name"               => 'n',

             # => (above)

             # just copy the keys above to the specvar files,
             # if you're not going to customize the standard
             # functions listed below.

             # (remove the # comment character from the front of
             #  the rows below that you want to use.)
             # ANY THAT YOU DON'T UNCOMMENT will use the defaults
             # ................................................................

             # MENU CONTROL VARIABLES
             # "$view_custom_record_menu_control_text_tbl_$table_name"       => 'n',
             # "$edit_record_menu_control_text_tbl_$table_name"              => 'n',
             # "$update_record_menu_control_text_tbl_$table_name"            => 'n',
             # "$add_record_menu_control_text_tbl_$table_name"               => 'n',
             # "$save_new_record_menu_control_text_tbl_$table_name"          => 'n',

             # "$query_by_form_table_start_tbl_$table_name"                  => 'n',
             # "$query_by_form_record_text_tbl_$table_name"                  => 'n',
             # "$query_by_form_table_end_tbl_$table_name"                    => 'n',

             # "$public_functions_response_body_tbl_$table_name"             => 'n',
             # "$admin_functions_response_body_tbl_$table_name"              => 'n',

             # "$view_admin_list_table_start_tbl_$table_name"                => 'n',
             # "$view_admin_list_table_header_tbl_$table_name"               => 'n',
             # "$view_admin_list_record_text_tbl_$table_name"                => 'n',
             # "$view_admin_list_record_cols_tbl_$table_name"                => 'n',
             # "$view_admin_list_table_end_tbl_$table_name"                  => 'n',

             # "$view_record_table_start_tbl_$table_name"                    => 'n',
             # "$view_record_record_text_tbl_$table_name"                    => 'n',
             # "$view_record_table_end_tbl_$table_name"                      => 'n',

             # "$view_custom_record_page_title_tbl_$table_name"              => 'n',
             # "$view_custom_record_page_header_tbl_$table_name"             => 'n',
             # "$view_custom_record_custom_page_tbl_$table_name"             => 'n',
             # "$view_custom_record_page_form_start_tbl_$table_name"         => 'n',
             # "$view_custom_record_page_form_system_fields_tbl_$table_name" => 'n',
             # "$view_custom_record_page_form_end_tbl_$table_name"           => 'n',
             # "$view_custom_record_page_footer_tbl_$table_name"             => 'n',
             # "$view_custom_record_table_start_tbl_$table_name"             => 'n',
             # "$view_custom_record_record_text_tbl_$table_name"             => 'n',
             # "$view_custom_record_table_end_tbl_$table_name"               => 'n',

             # "$edit_record_page_title_tbl_$table_name"                     => 'n',
             # "$edit_record_page_header_tbl_$table_name"                    => 'n',
             # "$edit_record_page_form_start_tbl_$table_name"                => 'n',
             # "$edit_record_custom_page_tbl_$table_name"                    => 'n',
             # "$edit_record_page_form_system_fields_tbl_$table_name"        => 'n',
             # "$edit_record_page_form_end_tbl_$table_name"                  => 'n',
             # "$edit_record_page_footer_tbl_$table_name"                    => 'n',
             # "$edit_record_table_start_tbl_$table_name"                    => 'n',
             # "$edit_record_record_text_tbl_$table_name"                    => 'n',
             # "$edit_record_table_end_tbl_$table_name"                      => 'n',

             # "$update_record_page_title_tbl_$table_name"                   => 'n',
             # "$update_record_page_header_tbl_$table_name"                  => 'n',
             # "$update_record_page_form_start_tbl_$table_name"              => 'n',
             # "$update_record_custom_page_tbl_$table_name"                  => 'n',
             # "$update_record_page_form_system_fields_tbl_$table_name"      => 'n',
             # "$update_record_page_form_end_tbl_$table_name"                => 'n',
             # "$update_record_page_footer_tbl_$table_name"                  => 'n',
             # "$update_record_table_start_tbl_$table_name"                  => 'n',
             # "$update_record_record_text_tbl_$table_name"                  => 'n',
             # "$update_record_table_end_tbl_$table_name"                    => 'n',

             # "$add_record_page_title_tbl_$table_name"                      => 'n',
             # "$add_record_page_header_tbl_$table_name"                     => 'n',
             # "$add_record_page_form_start_tbl_$table_name"                 => 'n',
             # "$add_record_custom_page_tbl_$table_name"                     => 'n',
             # "$add_record_page_form_system_fields_tbl_$table_name"         => 'n',
             # "$add_record_page_form_end_tbl_$table_name"                   => 'n',
             # "$add_record_page_footer_tbl_$table_name"                     => 'n',
             # "$add_record_table_start_tbl_$table_name"                     => 'n',
             # "$add_record_record_text_tbl_$table_name"                     => 'n',
             # "$add_record_table_end_tbl_$table_name"                       => 'n',

             # "$save_new_record_page_title_tbl_$table_name"                 => 'n',
             # "$save_new_record_page_header_tbl_$table_name"                => 'n',
             # "$save_new_record_page_form_start_tbl_$table_name"            => 'n',
             # "$save_new_record_custom_page_tbl_$table_name"                => 'n',
             # "$save_new_record_page_form_system_fields_tbl_$table_name"    => 'n',
             # "$save_new_record_page_form_end_tbl_$table_name"              => 'n',
             # "$save_new_record_page_footer_tbl_$table_name"                => 'n',
             # "$save_new_record_table_start_tbl_$table_name"                => 'n',
             # "$save_new_record_record_text_tbl_$table_name"                => 'n',
             # "$save_new_record_table_end_tbl_$table_name"                  => 'n',

             # "$batch_process_table_start_tbl_$table_name"                  => 'n',
             # "$batch_process_table_header_tbl_$table_name"                 => 'n',
             # "$batch_process_record_text_tbl_$table_name"                  => 'n',
             # "$batch_process_record_cols_tbl_$table_name"                  => 'n',
             # "$batch_process_table_end_tbl_$table_name"                    => 'n',

             # "$confirm_batch_process_table_start_tbl_$table_name"          => 'n',
             # "$confirm_batch_process_table_header_tbl_$table_name"         => 'n',
             # "$confirm_batch_process_record_text_tbl_$table_name"          => 'n',
             # "$confirm_batch_process_record_cols_tbl_$table_name"          => 'n',
             # "$confirm_batch_process_table_end_tbl_$table_name"            => 'n',

             );

# note that you can use 'file' set to 'n' (no) and then use
# standard html or a VARIABLE name from the default section above
# (not to be confused with a patched together variable name which
# is actually the name of the FILE, e.g: 'custom.$table_name\_fields.html')
#............................................................................

%custom_html = (
             "query_by_form_custom_table_start_tbl_$table_name"    => {
                           'file' => 'y',  # use 'n' with $query_by_form_custom_table_start
                           'html' => qq~
                                     custom.$table_name\.qbf_table_start.html
                                     ~,},
             "query_by_form_custom_record_text_tbl_$table_name"    => {
                           'file' => 'y',
                           'html' => qq~
                                     custom.$table_name\.qbf_record_text.html
                                     ~,},
             "query_by_form_custom_table_end_tbl_$table_name"  => {
                           'file' => 'y', # use 'n' with $query_by_form_custom_table_end
                           'html' => qq~
                                     custom.$table_name\.qbf_table_end.html
                                     ~,},
             "view_custom_list_table_start_tbl_$table_name"    => {
                           'file' => 'y',
                           'html' => qq~
                                     custom.$table_name\.view_list_table_start.html
                                     ~,},
             "view_custom_list_record_text_tbl_$table_name"    => {
                           'file' => 'y',
                           'html' => qq~
                                     custom.$table_name\.view_list_record_text.html
                                     ~,},
             "view_custom_list_table_end_tbl_$table_name"      => {
                           'file' => 'y', # use 'n' with $view_custom_list_table_end
                           'html' => qq~
                                     custom.$table_name\.view_list_table_end.html
                                     ~,},

             # (this may be deprecated)
             # "custom_page_fields_tbl_$table_name"                  => {
             #               'file' => 'n', # this may be deprecated
             #               'html' => qq~
             #                         $custom_fields
             #                         ~,},

             );

###############################################################################

1;
