# fsql_dictionary_vars.pl
#    Last change: PFB 2001-09-04 14:48:24
#############################################################################
# This is a setup file for FutureSQL
#############################################################################
# POD Documentation

=head1 SUMMARY OF FIELD RULES

! Note that this file MUST be loaded before the dictionary file
  because the dictionary file will use popup values from this file

format:

Note: I generally keep the first value BLANK so that
      when a record is updated, a value is not inadvertently
      placed in the field if the PREVIOUS value of the field
      was blank.

@name_array = (
          '',
          '',
          ''
          );

%name_hash  = (
          '' => '',
          '' => '',
          '' => ''
          );

NOTE! The values that are stored in the fields are the values
      in the 'arrays', which must match the values on the left
      side of the 'hashes'. The right side column of the hashes
      are simply 'labels' and are not stored.

=cut

####################################################################
# USER DEFINED ARRAYS AND HASHES
####################################################################

# EXAMPLE OF A "MULTI_TAG" FIELD - @a_tags_array and %a_tags_hash
# ....................................................................
# author tags array (field: a_tags)
# example of what I call a 'multi_tag' field, where each character
# or letter represents something unique, based on the lists below
# Because the field is a text field, it can fit 65k+ chars.
# The characters can be in any order, and are stored via a
# 'scrolling_list' definition in the data dictionary file for the table.
# Searches are done using the SQL 'like' syntax, e.g:
# "where a_tags like '%g%'"
# Using this type of field allows one to store a large number of
# 'attributes' about the record, that are dynamically based on the
# list definition, not the field name.

@a_tags_array = (
                '',
                'g',
                'h',
                'm',
                'r',
                'b',
                's'
                );

%a_tags_hash  = (
                ''  => '',
                'g' => 'General Topics',
                'h' => 'Historical Writing',
                'm' => 'Mysteries',
                'r' => 'Romance Writing',
                'b' => 'Biographies',
                's' => 'Science Fiction Writing'
                );

# values for authors.a_category

@a_category_array = (
                    '',
                    'really_famous',
                    'well_known',
                    'who_dat'
                    );

%a_category_hash  = (
                    ''              => '',
                    'really_famous' => 'Really Famous',
                    'well_known'    => 'Well Known',
                    'who_dat'       => 'Who Dat?'
                    );

# user validation

@user_status_array = (
                     '',
                     'active',
                     'deny'
                     );

%user_status_hash  = (
                     ''           => '',
                     'active'     => 'Active User',
                     'deny'       => 'Access Denied'
                     );

@user_type_array = (
                   '',
                   'admin',
                   'reader'
                   );

%user_type_hash  = (
                   ''          => '',
                   'admin'     => 'Administrator',
                   'reader'    => 'Reader'
                   );

@book_rating_array = (
                     '',
                     'A',
                     'B',
                     'C',
                     'D',
                     'F'
                     );

%book_rating_hash  = (
                     '',  => '',
                     'A', => 'A - We Wax Superlative',
                     'B', => 'B - Better than a Sharp Stick in the Eye',
                     'C', => 'C - Yawn. Quite Average',
                     'D', => 'D - A Reel Dawg',
                     'F'  => 'F - Ick. Gag. Blech.'
                     );

# used for state fields

@state_array =  (
                '',
                'AL',
                'AK',
                'AZ',
                'AR',
                'CA',
                'CO',
                'CT',
                'DE',
                'FL',
                'GA',
                'HI',
                'ID',
                'IL',
                'IN',
                'IA',
                'KS',
                'KY',
                'LA',
                'ME',
                'MD',
                'MA',
                'MI',
                'MN',
                'MS',
                'MO',
                'MT',
                'NE',
                'NV',
                'NH',
                'NJ',
                'NM',
                'NY',
                'NC',
                'ND',
                'OH',
                'OK',
                'OR',
                'PA',
                'RI',
                'SC',
                'SD',
                'TN',
                'TX',
                'UT',
                'VT',
                'VA',
                'WA',
                'WV',
                'WI',
                'WY'
                );

%state_hash  = (
                ''    => '',
                'AL'  => 'Alabama',
                'AK'  => 'Alaska',
                'AZ'  => 'Arizona',
                'AR'  => 'Arkansas',
                'CA'  => 'California',
                'CO'  => 'Colorado',
                'CT'  => 'Connecticut',
                'DE'  => 'Delaware',
                'FL'  => 'Florida',
                'GA'  => 'Georgia',
                'HI'  => 'Hawaii',
                'ID'  => 'Idaho',
                'IL'  => 'Illinois',
                'IN'  => 'Indiana',
                'IA'  => 'Iowa',
                'KS'  => 'Kansas',
                'KY'  => 'Kentucky',
                'LA'  => 'Louisiana',
                'ME'  => 'Maine',
                'MD'  => 'Maryland',
                'MA'  => 'Massachusetts',
                'MI'  => 'Michigan',
                'MN'  => 'Minnesota',
                'MS'  => 'Mississippi',
                'MO'  => 'Missouri',
                'MT'  => 'Montana',
                'NE'  => 'Nebraska',
                'NV'  => 'Nevada',
                'NH'  => 'New Hampshire',
                'NJ'  => 'New Jersey',
                'NM'  => 'New Mexico',
                'NY'  => 'New York',
                'NC'  => 'North Carolina',
                'ND'  => 'North Dakota',
                'OH'  => 'Ohio',
                'OK'  => 'Oklahoma',
                'OR'  => 'Oregon',
                'PA'  => 'Pennsylvania',
                'RI'  => 'Rhode Island',
                'SC'  => 'South Carolina',
                'SD'  => 'South Dakota',
                'TN'  => 'Tennessee',
                'TX'  => 'Texas',
                'UT'  => 'Utah',
                'VT'  => 'Vermont',
                'VA'  => 'Virginia',
                'WA'  => 'Washington',
                'WV'  => 'West Virginia',
                'WI'  => 'Wisconsin',
                'WY'  => 'Wyoming'
                );

# required field values

@readers_required_fields = qw[
                            user_type
                            first_name
                            last_name
                            user
                            password
                            status
                            city
                            state
                            work_phone
                            work_email
                           ];

# special, useful arrays

@duplicate_field_check = qw[
                     first_name
                     last_name
                     work_email
                     ];

@yes_array = (
             'y'
             );

%yes_hash = (
             'y' => ''
             );

@yesno_array = (
             '',
             'y',
             'n'
             );

%yesno_hash = (
             ''  => '',
             'y' => 'Yes',
             'n' => 'No'
             );

#.................................................................
# useful arrays and hashes for time and dates
# (used for data entry in some cases)
# (NOT USED IN THE CURRENT DEMO)
#  but left for those who might need them

@month_3_array = (
                 '',
                 'Jan',
                 'Feb',
                 'Mar',
                 'Apr',
                 'May',
                 'Jun',
                 'Jul',
                 'Aug',
                 'Sep',
                 'Oct',
                 'Nov',
                 'Dec'
                );

%month_3_hash = (
                 ''    => '',
                 'Jan' => '01',
                 'Feb' => '02',
                 'Mar' => '03',
                 'Apr' => '04',
                 'May' => '05',
                 'Jun' => '06',
                 'Jul' => '07',
                 'Aug' => '08',
                 'Sep' => '09',
                 'Oct' => '10',
                 'Nov' => '11',
                 'Dec' => '12'
                );

@day_array = (
            '',
            '00',
            '01',
            '02',
            '03',
            '04',
            '05',
            '06',
            '07',
            '08',
            '09',
            '10',
            '11',
            '12',
            '13',
            '14',
            '15',
            '16',
            '17',
            '18',
            '19',
            '20',
            '21',
            '22',
            '23',
            '24',
            '25',
            '26',
            '27',
            '28',
            '29',
            '30',
            '31'
             );

@year_array = (
            '',
            '2000',
            '2001',
            '2002',
            '2003',
            '2004',
            '2005'
             );

# all hours hh array/hash

@hh_array = (
            '',
            '00',
            '01',
            '02',
            '03',
            '04',
            '05',
            '06',
            '07',
            '08',
            '09',
            '10',
            '11',
            '12',
            '13',
            '14',
            '15',
            '16',
            '17',
            '18',
            '19',
            '20',
            '21',
            '22',
            '23'
            );

%hh_hash  = (
            ''   => '',
            '00' => '12am',
            '01' => '1 am',
            '02' => '2 am',
            '03' => '3 am',
            '04' => '4 am',
            '05' => '5 am',
            '06' => '6 am',
            '07' => '7 am',
            '08' => '8 am',
            '09' => '9 am',
            '10' => '10am',
            '11' => '11am',
            '12' => '12pm',
            '13' => '1 pm',
            '14' => '2 pm',
            '15' => '3 pm',
            '16' => '4 pm',
            '17' => '5 pm',
            '18' => '6 pm',
            '19' => '7 pm',
            '20' => '8 pm',
            '21' => '9 pm',
            '22' => '10pm',
            '23' => '11pm'
            );

@mm_array = (
            '',
            '00',
            '15',
            '30',
            '45'
            );

%mm_hash  = (
            ''   => '',
            '00' => '00',
            '15' => '15',
            '30' => '30',
            '45' => '45'
            );

###############################################################################

1;
