#!/usr/bin/perl
#    Last change: PFB 2001-10-07 19:55:01
##########################################################################
# POD Documentation

=head1 PROGRAM NAME AND AUTHOR

FutureSQL - Version 2.2
Build Date: October 7, 2001

fsql_main.cgi
by Peter F. Brown
peterbrown@worldcommunity.com

! SEE Instructions and Notes in the
documentation file: 'fsql_doc.txt'
and the setup file: 'fsql_setup.pl'

=head1 COPYRIGHT

Copyright 2001 Peter F. Brown (The World Community Network)
This program complies with the GNU GENERAL PUBLIC LICENSE
and is released as "Open Source Software".
NO WARRANTY IS OFFERED FOR THE USE OF THIS SOFTWARE

FutureSQL Version 1.0 was originally commissioned by
Remy Arteaga as a support script for PetJam.com.

=head1 BUG REPORTS AND SUPPORT

Send bug reports to peterbrown@worldcommunity.com.
Visit the author's web site at 'worldcommunity.com'
to view information about support, customer quotes,
a resume link, and fees for custom Perl/MySQL programming.

=head1 OBTAINING THE LATEST VERSION

==> Get the most recent version of this program at:
    http://worldcommunity.com/opensource

=head1 REQUIREMENTS

MySQL (www.mysql.com)
DBI:DBD (at the MySQL site)
CGI.pm
Perl 5

=cut

##############################################################################

use DBI;
use Cwd;

use CGI qw(:all -private_tempfiles escapeHTML unescapeHTML);
use CGI::Carp qw(fatalsToBrowser cluck);
$CGI::POST_MAX=1024 * 50;  # max 50K posts
$q = new CGI;

# use: confess "error"; in any subroutine to get stack trace, e.g:
# uncomment the 'confess' line in the sub 'error_message' for output

no strict refs;

print "Content-type: text/html\n\n";

$version        = 'v2.2';
$copyright_year = '2001';

# load Site DB file - which then loads the main setup file
#...............................................................................

if ( -e $site_db_file )
     {
     require $site_db_file;
     }
else
     {
     print qq~
     <p><br>Error!<p>
     Site DB file ($site_db_file\) not found.<br>
     Please contact the system administrator.
     <p>
     ~;
     exit;
     }

$copyright = qq~
<div align=center>
<font size=-1>
<b>FutureSQL $version &copy $copyright_year
The World Community Network&nbsp;&nbsp;:&nbsp;&nbsp;</b>
<i><a href="http://worldcommunity.com">worldcommunity.com</a></i>
</font>
</div>
~;

# check and store incoming parameters first,
# so that params can be used in any setup file
#..................................................

&check_params;

$s_  = param('s_');  # s_  = Key part of name of spec file
$dc_ = param('dc_'); # dc_ = Key part of name of data dictionary file

$a_  = param('a_');  # a_  = ACTION
$r_  = param('r_');  # r_  = NO OF RESULTS DISPLAYED (5, 10, 25)                              
$p_  = param('p_');  # p_  = SINGLE OR MULTIPLE PAGES (s, m)                                  
$rt_ = param('rt_'); # rt_ = RESULTS TEMPLATE e.g. 't1.htm' (without the .htm)

$w_  = param('w_');  # w_  = WORD - search item
$wh_ = param('wh_'); # wh_ = run time where string
$or_ = param('or_'); # or_ = run time order string

# AFTER loading setup file... load secondary files - note ORDER of loading
# first load dictionary vars file, to prep for dictionary file
#...............................................................................

if ( -e "$futuresql_dictionary_vars_file" )
        {
        require "$futuresql_dictionary_vars_file";
        }
else
        {
        &error_message(qq~Error!<p>
        The dictionary variables file [$futuresql_dictionary_vars_file] does not exist.
        ~);
        }

# next load dictionary, in order to get $table_name and $key_field
#...............................................................................

$dictionary_file = $futuresql_dict_prefix . $dc_ . $futuresql_specfile_suffix;

if ( -e "$dictionary_file" )
        {
        require "$dictionary_file";
        }
else
        {
        &error_message(qq~
        Error!<p>
        The dictionary file [$dictionary_file] does not exist.
        ~);
        }

# process dictionary file and get vars, such as $key_field and $table_name
# ******************************************************************************

&get_dictionary_hash_vars;

# set custom_template_dir, according to table name

if ( $use_table_subdir eq 'yes' )
     {
     $custom_template_dir = "$template_dir/$table_name";
     }
else
     {
     $custom_template_dir = "$template_dir";
     }

$ucfirst_table = ucfirst($table_name);

# ******************************************************************************

# next load action vars file, to prep for action spec file
#...............................................................................

if ( -e "$futuresql_action_vars_file" )
        {
        require "$futuresql_action_vars_file";
        }
else
        {
        &error_message(qq~
        Error!<p>
        The action variables file [$futuresql_action_vars_file] does not exist.
        ~);
        }

# next load user sub file
#...............................................................................

if ( -e "$futuresql_user_subs_file" )
     {
     require "$futuresql_user_subs_file";
     }
else
     {
     &error_message(qq~
     <p><br>Error!<p>
     User Subroutine file ($futuresql_user_subs_file\) not found.<br>
     Please contact the system administrator.
     <p>
     ~);
     }

# next load specvar file, to prep for action spec file
#...............................................................................
### modified July 11, 2001
### requires that $futuresql_specvars_default be set to
### something like 'default' and then use a default specvars file for
### any tables where you don't want a unique specvars file.

$search_specvars_file = $futuresql_specvars_prefix . $dc_ . $futuresql_specfile_suffix;

if ( -e "$search_specvars_file" )
        {
        require "$search_specvars_file";
        }
else
        {
        $search_specvars_file = $futuresql_specvars_prefix  .
                                $futuresql_specvars_default .
                                $futuresql_specfile_suffix;

        if ( -e "$search_specvars_file" )
                {
                require "$search_specvars_file";
                }
        else
                {
                &error_message(qq~
                Error!<p>
                The setup file [$search_specvars_file] does not exist.
                ~);
                }
        }

### end of modified July 11, 2001

# get custom html as set to 'yes' or 'no' in specvar file

&get_custom_html;

# load action spec file last
#...............................................................................

$search_spec_file = $futuresql_spec_prefix . $s_ . $futuresql_specfile_suffix;

if ( -e "$search_spec_file" )
        {
        require "$search_spec_file";
        }
else
        {
        &error_message(qq~
        Error!<p>
        The setup file [$search_spec_file] does not exist.
        ~);
        }

#...............................................................................

$user_level = param('user_level');

if ( $show_copyright eq 'no' and $user_level eq 'public' )
        {
        $copyright = '';
        }

# parse $a_ for sub_function, split by _._
# we do this for ease of use in forms.

if ( $w_ eq '' or $w_ eq ':!:key_field:!:' ){$w_ = '1';}

if ( $a_ =~ /\_\.\_/ )
        {
        # e.g: 'admin_functions_._admin_menu
        # e.g: 'view_custom_list_._01

        ($a_, $sub_function) = split /\_\.\_/, $a_;
        }

# &get_dictionary_hash_vars (and inside that routine, &field_setup)
# MUST be run before spec_setup
# in order to grab critical dictionary variables used in the spec file

&spec_setup;    # this processes the action specs

# now process external report files, if they exist
#................................................................

if ( $a_ eq 'view_custom_list' and $sub_function =~ /\w+/ )
     {
     #HASH LOOKUP#
     if ( $s_spec{action}{view_custom_list}{custom_list_files}{$sub_function} )
             {$custom_list_file = $s_spec{action}{view_custom_list}{custom_list_files}{$sub_function};}
     else    {&hash_error("$table_name - view_custom_list file $sub_function key error");}

     if ( -e "$custom_template_dir/$custom_list_file" )
             {
             require "$custom_template_dir/$custom_list_file";
             }
     else
             {
             &error_message(qq~
             Error!<p>
             The custom list file [$custom_list_file] does not exist.~);
             }

     for $attrib_name ( keys %{$s_spec{action}{$a_}} )
          {
          $attrib_value = $s_spec{action}{$a_}{$attrib_name};
          $$attrib_name = $attrib_value;
          }

     # we grab the external file, and process tags, which needs checking
     # (tags have to be processed in the correct order,
     #  for things like key_field values, etc.)

     if ( $external_table_start eq 'yes' )
          { 
          $table_start =~ s/\s+//g;
          $table_start = &include_file("$custom_template_dir/$table_start");
          foreach $tag ( @custom_html_tags )
                { $table_start =~ s/:!:$tag:!:/$$tag/g; }
          }

     if ( $external_record_text eq 'yes' )
          {
          $record_text =~ s/\s+//g;
          $record_text = &include_file("$custom_template_dir/$record_text");
          foreach $tag ( @custom_html_tags )
                { $record_text =~ s/:!:$tag:!:/$$tag/g; }
          }

     if ( $external_table_end eq 'yes' )
          {
          $table_end =~ s/\s+//g;
          $table_end = &include_file("$custom_template_dir/$table_end");
          foreach $tag ( @custom_html_tags )
                { $table_end =~ s/:!:$tag:!:/$$tag/g; }
          }

     }
else
     {
     # if external reports don't exist, then process keys as usual
     #.......................................................................
     for $attrib_name ( keys %{$s_spec{action}{$a_}} )
          {
          $attrib_value = $s_spec{action}{$a_}{$attrib_name};
          $$attrib_name = $attrib_value;
          }
     }

# assemble where and order strings based on input
#...................................................................
# we can pass run time where and order strings,
# via $wh_ and $or_ , or we can use $where_str and $order_str
# from the setup file.
# This way, we have more flexibility.

#!! we use unescapeHTML because unescape() had the
#   problem of turning %fa into the accented u character
#   (e.g: the sql query "like '%fast%'")

if ( $wh_ )
        {
        $wh_ = unescapeHTML($wh_);
        $where_str = $wh_;
        }

if ( $or_ )
        {
        $or_ = unescapeHTML($or_);
        $order_str = $or_;
        }

# connect to db

&connect_to_db;

# assemble header (do this after connecting to db)

&assemble_header;

# assemble system_hidden_fields (after connecting to db)
# note $security_fields - this var contains connection/validation values
# that are initialized inside connect_to_db

$system_hidden_fields = qq~
<input type="hidden" name="s_"         value="$s_">
<input type="hidden" name="dc_"        value="$dc_">
<input type="hidden" name="r_"         value="$r_">
<input type="hidden" name="p_"         value="$p_">
$security_fields
~;

# set up prefix string to check for joined fields in certain subroutines
$first_dc_prefix = '';

# all variables should be initialized before this line, if they're needed
# for any of the subroutines below.
#...............................................................................
#...............................................................................
# finally, check incoming parameters and branch

if ( $a_ eq "" )                        {&top_menu($q);}
if ( $a_ eq "top_menu" )                {&top_menu($q);}

if ( $a_ eq "query_by_form" )           {&query_by_form($q);}
if ( $a_ eq "query_by_form_custom" )    {&query_by_form($q);}

if ( $a_ eq "view_admin_list" )         {&get_records($q);}
if ( $a_ eq "view_custom_list" )        {&get_records($q);}
if ( $a_ eq "view_record" )             {&get_records($q);}
if ( $a_ eq "view_custom_record" )      {&get_records($q);}
if ( $a_ eq "edit_record" )             {&get_records($q);}

if ( $a_ eq "batch_process" )           {&batch_process($q);}
if ( $a_ eq "confirm_batch_process" )   {&batch_process($q);}

if ( $a_ eq "add_record" )              {&add_record($q);}

if ( $a_ eq "save_new_record" )         {&save_record($q);}
if ( $a_ eq "update_record" )           {&save_record($q);}

if ( $a_ eq "public_functions" )        {&do_function($q);}
if ( $a_ eq "admin_functions" )         {&do_function($q);}

#.............

&logout;

exit;

##################################################################
# top_menu
sub top_menu
{

my($q) = @_;

&do_pre_op;

unless ( $where_str ){$where_str = "$key_field > 0";}
unless ( $order_str ){$order_str = "$key_field";}

$field_order =~ s/\s+//g;
@field_order = split /,/, $field_order;

@where_strings = ();
%where_strings = ();

@order_strings = ();
%order_strings = ();

for $attrib_name ( keys %{$s_spec{action}{$a_}{where_strings}} )
        {
        $attrib_value = $s_spec{action}{$a_}{where_strings}{$attrib_name};

        # added in v1.2
        $attrib_value = escapeHTML($attrib_value);

        push (@where_strings, $attrib_value);
        $where_strings{$attrib_value} = $attrib_name;
        }

for $attrib_name ( keys %{$s_spec{action}{$a_}{order_strings}} )
        {
        $attrib_value = $s_spec{action}{$a_}{order_strings}{$attrib_name};

        # added in v1.2
        $attrib_value = escapeHTML($attrib_value);

        push (@order_strings, $attrib_value);
        $order_strings{$attrib_value} = $attrib_name;
        }

unless ( $r_ ){ $r_ = 20; }

$body_all = $menu_body;

# first assemble form variables

# form ....................................................
$select_spec_form_begin = qq~
<form method="post" action="$futuresql_url">
<input type="hidden" name="w_"  value="$w_">
<input type="hidden" name="r_"  value="$r_">
<input type="hidden" name="p_"  value="$p_">
<input type="hidden" name="rt_" value="$rt_">
<input type="hidden" name="a_"  value="top_menu">
$security_fields
~;

$select_spec_popup = $q->popup_menu(
                             -name     => 's_',
                             -values   => \@table_spec_array,
                             -labels   => \%table_spec_hash,
                             -default  => $s_
                             );

$select_dict_popup = $q->popup_menu(
                             -name     => 'dc_',
                             -values   => \@table_dict_array,
                             -labels   => \%table_dict_hash,
                             -default  => $dc_
                             );

# form ....................................................
$add_record_form = qq~
<form method="post" action="$futuresql_url">
$system_hidden_fields
<input type="hidden" name="rt_" value="$rt_">
<input type="hidden" name="w_" value="$w_">
<input type="hidden" name="a_" value="add_record">
<input type="submit" value=" Add New Record ">
</form>
~;

# form ....................................................
$view_default_list_form = qq~
<form method="post" action="$futuresql_url">
$system_hidden_fields
<input type="hidden" name="w_"  value="$w_">
<input type="hidden" name="rt_" value="$rt_">
<input type="hidden" name="wh_" value="$where_str">
<input type="hidden" name="or_" value="$order_str">
<input type="hidden" name="a_" value="view_admin_list">
<input type="submit" value=" View Default List of Records "><br>
<i>(where [$where_str] order by [$order_str])</i>
</form>
~;

# form ....................................................
$qbf_form = qq~
<form method="post" action="$futuresql_url">
$system_hidden_fields
<input type="hidden" name="rt_" value="$rt_">
<input type="hidden" name="w_" value="$w_">
Standard QBF <input type="radio" name="a_" value="query_by_form" checked> \~
Custom QBF <input type="radio" name="a_" value="query_by_form_custom"><br>
<input type="submit" value=" Search using Query by Form ">
</form>
~;

# form - search - custom .................................................

$search_form_begin = qq~
<form method="post" action="$futuresql_url">
<input type="hidden" name="s_"  value="$s_">
<input type="hidden" name="dc_"  value="$dc_">
<input type="hidden" name="w_"  value="$w_">
<input type="hidden" name="rt_" value="$rt_">
$security_fields
~;

$page_display_popup = $q->popup_menu(
                             -name     => 'p_',
                             -values   => ['s', 'm'],
                             -labels   => {
                                          's' => 'Single Page Display',
                                          'm' => 'Multiple Page Display'
                                          },
                             -default  => $p_
                             );

$sql_where_popup = $q->popup_menu(
                             -name     => 'wh_',
                             -values   => \@where_strings,
                             -labels   => \%where_strings,
                             -default  => ''
                             );

$sql_order_popup = $q->popup_menu(
                             -name     => 'or_',
                             -values   => \@order_strings,
                             -labels   => \%order_strings,
                             -default  => ''
                             );

# form - mysql backup ....................................................

$mysql_backup_form = qq~
<form method="post" action="$futuresql_url">
$system_hidden_fields
<input type="hidden" name="w_"  value="$w_">
<input type="hidden" name="rt_" value="$rt_">
<input type="hidden" name="a_" value="admin_functions_._mysql_backup">
<input type="submit" value=" BackUp Now ">
</form>
~;

# form - custom update ....................................................

$update_form_begin = qq~
<form method="post" action="$futuresql_url">
$system_hidden_fields
<input type="hidden" name="w_"  value="$w_">
<input type="hidden" name="rt_" value="$rt_">
<input type="hidden" name="or_" value="$order_str">
<input type="hidden" name="a_" value="admin_functions_._global_update">
Update [$table_name]<br>
Set ~;

$update_field = $q->popup_menu(
                             -name     => 'update_field',
                             -values   => \@field_order,
                             -labels   => '',
                             -default  => ''
                             );

# form - custom delete ....................................................

$delete_form_begin = qq~
<form method="post" action="$futuresql_url">
$system_hidden_fields
<input type="hidden" name="w_"  value="$w_">
<input type="hidden" name="rt_" value="$rt_">
<input type="hidden" name="or_" value="$order_str">
<input type="hidden" name="a_" value="admin_functions_._global_delete">
~;

# now replace tags

$body_all =~ s/:!:copyright:!:/$copyright/g;
$body_all =~ s/:!:login_data:!:/$login_data/g;
$body_all =~ s/:!:where_order_line:!:/$where_order_line/g;
$body_all =~ s/:!:top_menu_left_bg_color:!:/$top_menu_left_bg_color/g;
$body_all =~ s/:!:top_menu_right_bg_color:!:/$top_menu_right_bg_color/g;
$body_all =~ s/:!:top_menu_backup_bg_color:!:/$top_menu_backup_bg_color/g;
$body_all =~ s/:!:top_menu_backup_fg_color:!:/$top_menu_backup_fg_color/g;
$body_all =~ s/:!:wrap:!:/$wrap/g;
$body_all =~ s/:!:table_name:!:/$table_name/g;
$body_all =~ s/:!:futuresql_url:!:/$futuresql_url/g;

$body_all =~ s/:!:select_spec_form_begin:!:/$select_spec_form_begin/g;
$body_all =~ s/:!:select_spec_popup:!:/$select_spec_popup/g;
$body_all =~ s/:!:select_dict_popup:!:/$select_dict_popup/g;
$body_all =~ s/:!:add_record_form:!:/$add_record_form/g;
$body_all =~ s/:!:view_default_list_form:!:/$view_default_list_form/g;
$body_all =~ s/:!:qbf_form:!:/$qbf_form/g;
$body_all =~ s/:!:search_form_begin:!:/$search_form_begin/g;
$body_all =~ s/:!:page_display_popup:!:/$page_display_popup/g;
$body_all =~ s/:!:sql_where_popup:!:/$sql_where_popup/g;
$body_all =~ s/:!:sql_order_popup:!:/$sql_order_popup/g;
$body_all =~ s/:!:mysql_backup_form:!:/$mysql_backup_form/g;
$body_all =~ s/:!:update_form_begin:!:/$update_form_begin/g;
$body_all =~ s/:!:update_field:!:/$update_field/g;
$body_all =~ s/:!:delete_form_begin:!:/$delete_form_begin/g;

$body_all =~ s/:!:system_hidden_fields:!:/$system_hidden_fields/g;

&do_post_op;

$template_text = &process_template;
$template_text = &parse_params($template_text);

print $template_text;

}
################################################
# do_function
sub do_function
{

my($q) = @_;

&do_pre_op;

# changed in 2.0 (split out into functions)
# $body_all = $login_data . $where_order_line . $response_body . $copyright;
$body_all = $response_body;

unless ( $response_text ){ $response_text = 'Process Completed!'; }

$body_all =~ s/:!:response_text:!:/$response_text/g;

&do_post_op;

$template_text = &process_template;
$template_text = &parse_params($template_text);

print $template_text;

}
################################################
# add_record
sub add_record
{
    
my($q) = @_;

&do_pre_op;

$field_order   =~ s/\s+//g;
@field_order   = split /,/, $field_order;

# 'display_method'    => (_standard_, _custom_page_, _external_template_)

$body_all = '';

if ( $display_method eq '_standard_' )
     {
     $body_all      = $login_data . $where_order_line . $table_start;
     $record2       = $record_text;
     }
elsif ( $display_method eq '_custom_page_' )
     {
     $record2 = $custom_page;
     $record2 =~ s/:!:custom_page_fields:!:/$custom_page_fields/g;
     }
elsif ( $display_method eq '_external_template_' )
     {
     $record2 = &include_file("$custom_template_dir/$external_template");
     }

$record2       =~ s/:!:page_header:!:/$page_header/g;
$record2       =~ s/:!:menu_control_text:!:/$menu_control_text/g;
$record2       =~ s/:!:page_form_start:!:/$page_form_start/g;
$record2       =~ s/:!:page_form_system_fields:!:/$page_form_system_fields/g;
$record2       =~ s/:!:page_form_end:!:/$page_form_end/g;
$record2       =~ s/:!:page_footer:!:/$page_footer/g;

$record2       =~ s/:!:key_field:!://g;
$table_end     =~ s/:!:key_field:!://g;

$record_repeat = '';

foreach $field_name ( @field_order )
        {
        $dc_field_name = $field_name;
        &field_setup;
        &get_top_field_attributes;
        &get_field_setup_attributes;
        &get_field_default_values;
        &process_field_setup_subs;

        # now do special field processing and create html, etc.

        $field_html = '';

        if ( $display_only =~ /yes/ )
                {
                $field_html = qq~
                <input type="hidden" name="$field_name" value="$field_value">
                $field_value
                ~;
                }
        else
                {
                # 'html_type'         => 'text',
                # 'html_size'         => '10',
                # 'max_length'        => '10',

                if ( $html_type eq 'text' )
                        {
                        $field_html  = qq~<input type="text" name="$field_name" size="$html_size" maxlength="$max_length" value="$field_value">~;
                        }
                elsif ( $html_type eq 'password' )
                        {
                        $field_value = $default;
                        $field_html  = qq~<input type="password" name="$field_name" size="$html_size" maxlength="$max_length" value="$field_value">~;
                        }
                elsif ( $html_type eq 'textarea' )
                        {
                        $field_html = qq~<textarea $html_size name="$field_name" wrap="$wrap">$field_value</textarea>~;
                        }
                elsif ( $html_type eq 'scrolling_list' )
                        {
                        $field_html = $q->scrolling_list(
                                                     -name     => $field_name,
                                                     -values   => $values,
                                                     -default  => $field_value,
                                                     -labels   => $labels,
                                                     -size     => $size,
                                                     -multiple => $multiple
                                                     );
                        }
                elsif ( $html_type eq 'popup_menu' )
                        {
                        $field_html = $q->popup_menu(
                                                     -name     => $field_name,
                                                     -values   => $values,
                                                     -default  => $field_value,
                                                     -labels   => $labels
                                                     );
                        }
                elsif ( $html_type eq 'radio_group' )
                        {
                        $field_html = $q->radio_group(
                                                     -name     => $field_name,   
                                                     -values   => $values, 
                                                     -default  => $field_value,
                                                     -labels   => $labels, 
                                                     -linebreak=> $linebreak
                                                     );
                        }
                elsif ( $html_type eq 'checkbox_group' )
                        {
                        $field_html = $q->checkbox_group(
                                                     -name     => $field_name,   
                                                     -values   => $values, 
                                                     -default  => $field_value,
                                                     -labels   => $labels, 
                                                     -linebreak=> $linebreak
                                                     );
                        }

                }

        # ... now branch, depending on method of display
        #..................................................................

        if ( $display_method eq '_standard_' )
             {
             $record_repeat .= $record2;
             $record_repeat =~ s/:!:field_label:!:/$label/g;
             $record_repeat =~ s/:!:field_value:!:/$field_html/g;
             }
        else
             {
             if ( $record2 =~ /:!:$field_name:!:/ )
                   {
                   $record2 =~ s/:!:$field_name:!:/$field_html/g;
                   }
             else
                   {
                   # fixed on Aug. 18, 2001
                   # commented this out: on 'add' a record, use default_value
                   # instead of the 'default' in the field setup
                   # (field_value has already been declared)

                   # $field_value = $default;
                   
                   $field_html = qq~
                   <input type="hidden" name="$field_name" value="$field_value">
                   :!:missing_fields_made_hidden:!:
                   ~;
                   $record2 =~ s/:!:missing_fields_made_hidden:!:/$field_html/g;
                   }
             }
        
        }
        # end of field loop

if ( $display_method eq '_standard_' )
     {
     $record2 = $record_repeat;
     $body_all .= $record2;
     $body_all .= $table_end . $copyright;
     $body_all .= &admin_little_menu;
     }
else
     {
     $body_all .= $record2;
     $admin_menu = &admin_little_menu;
     }

&do_post_op;

$template_text = &process_template;
$template_text = &parse_params($template_text);

print $template_text;

}
################################################
# batch_process
sub batch_process
{
    
my($q) = @_;

$process_name = param('process_name');
&do_pre_op;

# assemble field string
#....................................

$field_str = $user_field_order;
$user_field_order =~ s/\s+//g;
@user_field_order = split /,/, $user_field_order;
$colspan = scalar(@user_field_order) + 2;

$body_all = $login_data . $where_order_line . $table_start;
$body_all .= &assemble_table_headers;

# get records from array
#....................................

@w_array     = param('w_array');
@w_array_all = split /\|/, param('w_array_all');
$batch_type  = param('batch_type');
if ( $batch_type eq 'all' ){@w_array = @w_array_all;}

foreach $w_ ( @w_array )
{

&spec_setup;
for $attrib_name ( keys %{$s_spec{action}{$a_}} )
        {
        $attrib_value = $s_spec{action}{$a_}{$attrib_name};
        $$attrib_name = $attrib_value;
        }

# finalize select string
#....................................

$select_str = "
               select
               $field_str
               from
               $table_name
               where
               $where_str
              ";
    
# now get rows
#........................................

$sth = $dbh->prepare("$select_str")
                     or die
                     &error_message(qq~Error!<p>
                     Can't do select: $DBI::errstr~);

$rv = $sth->execute or die
                       &error_message(qq~Error!<p>
                       Can't execute the query: $DBI::errstr~);

# get records
#....................................

$hash_ref = $sth->fetchrow_hashref;

$record2 = $record_text;
$key_value = $hash_ref->{$key_field};
$record2 =~ s/:!:key_field:!:/$key_value/g;
$table_end =~ s/:!:key_field:!:/$key_value/g;

$record_cols2 = $record_cols;
$record_repeat = '';

foreach $field_name ( @user_field_order )
        {
        $field_value = $hash_ref->{$field_name};

        $dc_field_name = $field_name;
        &field_setup;
        &get_top_field_attributes;

        if ( $html_type eq 'textarea' )
                {
                $field_value =~ s/\n/<br>/g;
                }

        $record_repeat .= $record_cols2;
        $record_repeat =~ s/:!:field_value:!:/$field_value/g;
        }

$record_cols2 = $record_repeat;
$record2 = $record2 . $record_cols2;

#..........................................................

if ( $a_ eq 'confirm_batch_process' )
        {
        # process command in hash
        #HASH LOOKUP#
        if ( $s_spec{action}{confirm_batch_process}{batch_commands}{$process_name} )
                {
                $sub_name = $s_spec{action}{confirm_batch_process}{batch_commands}{$process_name}; 
                &$sub_name if defined &$sub_name;                                                           
                }
        else
                {
                &error_message(qq~
                Error!
                <p>
                Are you sure that you clicked on a Batch Process Button?
                <p>
                Please hit BACK and try again.
                ~);
                }
        }

$body_all .= $record2;

# end of record process
#....................................

} # end of w_ loop

$body_all .= $table_end . $copyright;
$body_all =~ s/:!:colspan:!:/$colspan/g;
$body_all =~ s/:!:batch_response:!:/$batch_response/g;

$body_all =~ s/:!:process_name:!:/$process_name/g;
$body_all .= &admin_little_menu;

&do_post_op;

$template_text = &process_template;
$template_text = &parse_params($template_text);

print $template_text;

}
################################################
# query_by_form
sub query_by_form
{

my($q) = @_;

&do_pre_op;

if ( $a_ eq 'query_by_form_custom' )
     {
     if ( $use_join_str eq 'yes' )
          {
          $search_fields = $join_field_order;
          }
     else
          {
          # reset search fields to ALL fields, to
          # parse through them all and check if the tags exist
          $search_fields = $field_order;
          }

     $body_all      = $table_start;

     # $login_data . $where_order_line
     # can be placed in the spec file
     }
else
     {
     $body_all      = $login_data . $where_order_line . $table_start;
     }

$search_fields =~ s/\s+//g;
@search_fields = split /,/, $search_fields;

$record2       = $record_text;
$record_repeat = '';

foreach $field_name ( @search_fields )
        {
        # process joined or regular fields, based on $use_join_str
        #..........................................................
        $dc_field_name = $field_name;
        if ( $use_join_str eq 'yes' ){&prep_joined_fields;}

        &field_setup;
        &get_top_field_attributes;
        &get_field_setup_attributes;

        $field_value = '';
        if ( $use_field_defaults_for_qbf eq 'yes' )
               {
               &get_field_default_values;
               }

        # process anonymous subroutines
        &process_field_setup_subs;

        # now do special field processing and create html, etc.
        # also create subfields for search parameters
        # i.e. =, like, <, >, <>

        # new in 2.0
        # 'show_select_params'    => 'yes',
        # 'select_param'          => 'like',
        # 'show_sort_params'      => 'yes',
        # 'sort_order'            => qq~$key_field desc~,

        $field_html   = '';

        if ( $a_ eq 'query_by_form_custom' and $show_select_params ne 'yes' )
          {
          $field_html2  = qq~<font size=-1>~;
          }
        else
          {

          $field_html2  = qq~<br><font size=-1>~;

          $field_html2 .= $q->popup_menu(
                               -name     => "$field_name\_search_type",
                               -values   => [
                                            'ignore',
                                            '=',
                                            '!=',
                                            'like',
                                            'not like',
                                            '<',
                                            '<=',
                                            '>',
                                            '>='
                                            ],
                               -labels   => {
                                            'ignore'   => 'ignore',
                                            '='        => 'equals',
                                            '!='       => 'does not equal',
                                            'like'     => 'contains within',
                                            'not like' => 'does not contain within',
                                            '<'        => 'less than',
                                            '<='       => 'less than or equal',
                                            '>'        => 'greater than',
                                            '>='       => 'greater than or equal'
                                            },
                               -default  => 'ignore'
                               );

          }

          $field_html2 .= qq~</font>~;
        #...................................................
        
        # 'html_type'         => 'text',
        # 'html_size'         => '10',
        # 'max_length'        => '10',

        if ( $html_type eq 'text' )
                {
                $field_html  = qq~<input type="text" name="$field_name" size="$html_size" maxlength="$max_length" value="$field_value">~;
                $field_html .= $field_html2;
                }
        elsif ( $html_type eq 'password' )
                {
                $field_value = $default;
                $field_html  = qq~<input type="password" name="$field_name" size="$html_size" maxlength="$max_length" value="$field_value">~;
                }
        elsif ( $html_type eq 'textarea' )
                {
                $field_html = qq~<textarea $html_size name="$field_name" wrap="$wrap">$field_value</textarea>~;
                $field_html .= $field_html2;
                }
        elsif ( $html_type eq 'scrolling_list' )
                {
                $field_html = $q->scrolling_list(
                                             -name     => $field_name,
                                             -values   => $values,
                                             -default  => $field_value,
                                             -labels   => $labels,
                                             -size     => $size,
                                             -multiple => $multiple
                                             );
                $field_html .= $field_html2;
                }
        elsif ( $html_type eq 'popup_menu' )
                {
                $field_html = $q->popup_menu(
                                             -name     => $field_name,
                                             -values   => $values,
                                             -default  => $field_value,
                                             -labels   => $labels
                                             );
                $field_html .= $field_html2;
                }
        elsif ( $html_type eq 'radio_group' )
                {
                $field_html = $q->radio_group(
                                             -name     => $field_name,
                                             -values   => $values, 
                                             -default  => $field_value,
                                             -labels   => $labels, 
                                             -linebreak=> $linebreak
                                             );
                $field_html .= $field_html2;
                }
        elsif ( $html_type eq 'checkbox_group' )
                {
                $field_html = $q->checkbox_group(
                                             -name     => $field_name,   
                                             -values   => $values, 
                                             -default  => $field_value,
                                             -labels   => $labels, 
                                             -linebreak=> $linebreak
                                             );
                $field_html .= $field_html2;
                }

        if ( $a_ eq 'query_by_form_custom' )
                {
                $record2 =~ s/:!:$field_name:!:/$field_html/g;
                }
        else
                {
                $record_repeat .= $record2;
                $record_repeat =~ s/:!:field_label:!:/$label/g;
                $record_repeat =~ s/:!:field_value:!:/$field_html/g;
                }

        }

# parse sort order fields
# ................................................

$order_fields = qq~
<font size=-1>Select the Sort Order for
your Query by choosing up to 3 Different Fields,<br>
and the Direction of the Sort.
</font>
<br>
~;

# set the first field to have a value
# so it doesn't crash

$order_fields .= $q->popup_menu(
                             -name     => 'sort_field_1',
                             -values   => \@search_fields,
                             -default  => ''
                             );

$order_fields .= $q->radio_group(
                             -name     => 'sort_field_dir_1',   
                             -values   => ['asc', 'desc'],
                             -default  => 'asc',
                             -labels   =>
                                          {
                                          'asc'    => 'asc',
                                          'desc'   => 'desc'
                                          }
                             );

$order_fields .= qq~<br>~;

$order_fields .= $q->popup_menu(
                             -name     => 'sort_field_2',
                             -values   => \@search_fields,
                             -default  => ''
                             );

$order_fields .= $q->radio_group(
                             -name     => 'sort_field_dir_2',   
                             -values   => ['ignore', 'asc', 'desc'],
                             -default  => 'ignore',
                             -labels   =>
                                          {
                                          'ignore' => 'ignore',
                                          'asc'    => 'asc',
                                          'desc'   => 'desc'
                                          }
                             );

$order_fields .= qq~<br>~;

$order_fields .= $q->popup_menu(
                             -name     => 'sort_field_3',
                             -values   => \@search_fields,
                             -default  => ''
                             );

$order_fields .= $q->radio_group(
                             -name     => 'sort_field_dir_3',
                             -values   => ['ignore', 'asc', 'desc'],
                             -default  => 'ignore',
                             -labels   =>
                                          {
                                          'ignore' => 'ignore',
                                          'asc'    => 'asc',
                                          'desc'   => 'desc'
                                          }
                             );

$order_fields .= qq~
<br>
<font size=-1>
(note that 'asc' means 'ascending order' e.g: 'a-z',
while 'desc' means 'descending order' e.g: 'z-a'.)
<p>
</font>
~;

#.........................................

$search_sort_params = '';

if ( $a_ eq 'query_by_form_custom' )
     {
     $body_all .= $record2;
     $body_all .= $table_end;

     # new in 2.0
     # 'show_select_params'    => 'yes',
     # 'select_param'          => 'like',
     # 'show_sort_params'      => 'yes',
     # 'sort_order'            => qq~$key_field desc~,

     if ( $show_select_params ne 'yes' )
          {
          $search_sort_params .= qq~
                  <input type="hidden" name="show_select_params" value="no">
                  <input type="hidden" name="select_param" value="$select_param">
                  ~;
          }
     if ( $show_sort_params ne 'yes' )
          {
          $order_fields = '';
          $search_sort_params .= qq~
                  <input type="hidden" name="show_sort_params" value="no">
                  <input type="hidden" name="sort_order" value="$sort_order">
                  ~;
          }
     }
elsif ( $a_ eq 'query_by_form' )
     {
     $record2   = $record_repeat;
     $body_all .= $record2;
     $body_all .= $table_end . $copyright;
     $body_all .= &admin_little_menu;
     }

$body_all =~ s/:!:search_sort_params:!:/$search_sort_params/g;
$body_all =~ s/:!:order_fields:!:/$order_fields/g;
$body_all =~ s/:!:key_field:!:/1/g;

&do_post_op;

$template_text = &process_template;
$template_text = &parse_params($template_text);

print $template_text;
    
}
################################################
# get_records
sub get_records
{

my($q) = @_;

&do_pre_op;

# set up limit string for single or multiple pages
#............................................................
# $max_rows = 200;
# $max_rows_per_page = 25;

if ( $r_ < 1 or $r_ > $max_rows_per_page ){$r_ = $max_rows_per_page;}

if ( $p_ eq 'm' )
        {
        # setup for multiple pages
        $o_ = param('o_');

        unless ( $o_ )
                {
                # this means that it's the first iteration
                $offset = 0;
                }
        else
                {
                $offset = $o_;
                }

        if ( ($offset + $r_) > $max_rows )
                {
                $limit_str = "limit $offset," . ($max_rows - $offset);
                }
        else
                {
                $limit_str = "limit $offset,$r_";
                }
        }
else
        {
        # setup for single page
        if ( $r_ < 1 or $r_ > $max_rows ){$r_ = $max_rows;}
        $limit_str = "limit $r_";
        }

# assemble field string
#....................................

# test for view_admin_list first
if ( $a_ eq 'view_admin_list' )
     {
     $field_order = $user_field_order;
     }

# test for $use_join_str. this can be in any pre_op involving get_records
#..........................................................................

if ( $use_join_str eq 'yes' )
     {
     $field_order    = $join_field_order;
     $table_name_str = $join_table_names;
     $field_str      = "
                       $join_field_expressions,
                       $field_expressions
                       ";
     $field_order =~ s/\s+//g;
     @field_order = split /,/, $field_order;
     @field_order = sort (@field_order);
     }
else
     {
     $table_name_str = $table_name;
     $field_str      = "
                       $field_order,
                       $field_expressions
                       ";
     $field_order =~ s/\s+//g;
     @field_order = split /,/, $field_order;
     }

# now process as usual.
# join issues below are: reloading the dict files
#..........................................................................

$colspan = scalar(@field_order) + 2;

$field_expression_names =~ s/\s+//g;
@field_expression_names = split /,/, $field_expression_names;

# finalize select string
#....................................

$select_str = "
               select
               $field_str
               from
               $table_name_str
               where

               ( $hard_coded_where_str )
               and
               ( $where_str )

               order by 
               $order_str
               $limit_str
              ";

$select_count = "
               select
               count(*)
               from
               $table_name_str
               where

               ( $hard_coded_where_str )
               and
               ( $where_str )

               limit $max_rows
              ";

# first check count(*)
#........................................

$sth = $dbh->prepare("$select_count")
                     or die
                     &error_message(qq~Error!<p>
                     Can't do select: $DBI::errstr~);

$rv = $sth->execute or die
                       &error_message(qq~Error!<p>
                       Can't execute the query: $DBI::errstr~);

( $total_rows ) = $sth->fetchrow_array;

$zero_records_text_new = $zero_records_text;
$zero_records_text_new =~ s/:!:select_count:!:/$select_count/g;
$zero_records_text_new =~ s/:!:zero_records_custom_message:!:/$zero_records_custom_message/g;

if ( $total_rows == 0 )
     {
     &error_message(qq~
                    $zero_records_text_new
                    ~);
     }

# now prepare sql to get rows
#........................................

$sth = $dbh->prepare("$select_str")
                     or die
                     &error_message(qq~Error!<p>
                     Can't do select: $DBI::errstr~);

$rv = $sth->execute or die
                    &error_message(qq~Error!<p>
                    Can't execute the query: $DBI::errstr~);

if (
     ( $show_sql_header eq 'no' and $user_level eq 'public' )
     or
     ( $show_sql_header eq 'no' and $a_ =~ /custom/ )
   )
        {
        $body_all = $table_start;
        }
else
        {
        $body_all = qq~
            $login_data
            $where_order_line
            <font size=-1>
            <i>$limit_str</i><br>
            &gt;&gt;&gt; <b>$total_rows found</b> &lt;&lt;&lt;
            </font>
            <br>
            $table_start
            ~;
        }

# test for $a_ eq 'view_admin_list' - then assemble table headers
#............................................................

if ( $a_ eq 'view_admin_list' )
        {
        $body_all .= &assemble_table_headers;
        }
#............................................................

$sub_head_test_field_str = '';
$page_recs_displayed = 0;

@w_array_all = ();

# get records
#............................................................

# requires a more recent DBI library
# while( $hash_ref = $sth->fetchrow_hashref ('NAME_lc') )

while( $hash_ref = $sth->fetchrow_hashref )
        {
        $key_value = $hash_ref->{$key_field};

        # new in v2.0

        if ( $display_method eq '_standard_' or $display_method eq '' )
             {
             $record2 = $record_text;
             }
        elsif ( $display_method eq '_custom_page_' )
             {
             $record2 = $custom_page;
             $record2 =~ s/:!:custom_page_fields:!:/$custom_page_fields/g;
             }
        elsif ( $display_method eq '_external_template_' )
             {
             $record2 = &include_file("$custom_template_dir/$external_template");
             }
        
        $record2 =~ s/:!:page_header:!:/$page_header/g;
        $record2 =~ s/:!:menu_control_text:!:/$menu_control_text/g;
        $record2 =~ s/:!:page_form_start:!:/$page_form_start/g;
        $record2 =~ s/:!:page_form_system_fields:!:/$page_form_system_fields/g;
        $record2 =~ s/:!:page_form_end:!:/$page_form_end/g;
        $record2 =~ s/:!:page_footer:!:/$page_footer/g;
        #...................

        # $record2 = $record_text;
        $record2   =~ s/:!:key_field:!:/$key_value/g;
        $table_end =~ s/:!:key_field:!:/$key_value/g;

        push (@w_array_all, $key_value);

        #...............................

        if ( $a_ eq 'view_admin_list' )
                {
                # @field_order copied from @user_field_order
                # :!:colspan:!:
                # :!:field_label:!:
                # :!:field_value:!:
                # <tr></tr> around table_header
                # <tr> before record_text
                # </tr> after record_cols
                # table_header and record_cols multiplied based on @user_field_order

                $record_cols2 = $record_cols;
                $record_repeat = '';

                foreach $field_name ( @field_order )
                        {
                        $field_value = $hash_ref->{$field_name};

                        $dc_field_name = $field_name;
                        &field_setup;
                        &get_top_field_attributes;

                        # new in 2.0
                        # note use of '$display_field_value' below

                        $display_field_value = &get_display_field_value($dc_field_name, $field_value);
                        #..........................................................

                        if ( $view_with_html_escape eq 'yes' )
                                {
                                $display_field_value = escapeHTML($display_field_value);
                                }

                        if ( $html_type eq 'textarea' )
                                {
                                $display_field_value =~ s/\n/<br>/g;
                                }

                        $record_repeat .= $record_cols2;
                        $record_repeat =~ s/:!:field_value:!:/$display_field_value/g;
                        }

                $record_cols2 = $record_repeat;
                $record2 .= $record_cols2;
                }

        #..........................................................

        elsif ( $a_ eq 'view_custom_list' )
                {
                $custom_list_header = param('custom_list_header');

                foreach $field_name ( @field_order )
                        {
                        $field_value = $hash_ref->{$field_name};

                        # process joined or regular fields, based on $use_join_str
                        #..........................................................
                        $dc_field_name = $field_name;
                        if ( $use_join_str eq 'yes' ){&prep_joined_fields;}

                        &field_setup;
                        &get_top_field_attributes;

                        # new in 2.0
                        # note use of '$display_field_value' below

                        $display_field_value = &get_display_field_value($dc_field_name, $field_value);
                        #..........................................................

                        if ( $view_with_html_escape eq 'yes' )
                                {
                                $display_field_value = escapeHTML($display_field_value);
                                }

                        if ( $html_type eq 'textarea' )
                                {
                                $display_field_value =~ s/\n/<br>/g;
                                }

                        $record2 =~ s/:!:$field_name:!:/$display_field_value/g;
                        }

                foreach $field_name ( @field_expression_names )
                        {
                        $field_value = $hash_ref->{$field_name};
                        &field_setup;

                        if ( $view_with_html_escape eq 'yes' )
                                {
                                $field_value = escapeHTML($field_value);
                                }

                        $record2 =~ s/:!:$field_name:!:/$field_value/g;
                        }
                }

        #...............................

        elsif ( $a_ eq 'view_record' )
                {
                $record_repeat = '';

                foreach $field_name ( @field_order )
                        {
                        $field_value = $hash_ref->{$field_name};

                        $dc_field_name = $field_name;
                        &field_setup;
                        &get_top_field_attributes;

                        # new in 2.0
                        # note use of '$display_field_value' below

                        $display_field_value = &get_display_field_value($dc_field_name, $field_value);
                        #.................................................................................

                        if ( $view_with_html_escape eq 'yes' )
                                {
                                $display_field_value = escapeHTML($display_field_value);
                                }

                        if ( $html_type eq 'textarea' )
                                {
                                $display_field_value =~ s/\n/<br>/g;
                                }

                        $record_repeat .= $record2;
                        $record_repeat =~ s/:!:field_label:!:/$label/g;
                        $record_repeat =~ s/:!:field_value:!:/$display_field_value/g;
                        }

                # ? put this in the table begin?
                # foreach $field_name ( @field_expression_names )
                #         {
                #         $record2 =~ s/:!:$field_name:!:/$hash_ref->{$field_name}/g;
                #         }

                $record2 = $record_repeat;

                }

        #...............................................

        elsif ( $a_ eq 'view_custom_record' )
                {
                foreach $field_name ( @field_order )
                        {
                        $field_value = $hash_ref->{$field_name};

                        # process joined or regular fields, based on $use_join_str
                        #..........................................................
                        $dc_field_name = $field_name;
                        if ( $use_join_str eq 'yes' ){&prep_joined_fields;}

                        &field_setup;
                        &get_top_field_attributes;

                        # new in 2.0
                        # note use of '$display_field_value' below

                        $display_field_value = &get_display_field_value($dc_field_name, $field_value);
                        #.................................................................................

                        if ( $view_with_html_escape eq 'yes' )
                                {
                                $display_field_value = escapeHTML($display_field_value);
                                }

                        if ( $html_type eq 'textarea' )
                                {
                                $display_field_value =~ s/\n/<br>/g;
                                }

                        $record2 =~ s/:!:$field_name:!:/$display_field_value/g;
                        }

                foreach $field_name ( @field_expression_names )
                        {
                        $field_value = $hash_ref->{$field_name};
                        &field_setup;

                        if ( $view_with_html_escape eq 'yes' )
                                {
                                $field_value = escapeHTML($field_value);
                                }

                        $record2 =~ s/:!:$field_name:!:/$field_value/g;
                        }

                }

        #...............................................

        elsif ( $a_ eq 'edit_record' )
                {
                $record_repeat = '';

                foreach $field_name ( @field_order )
                        {
                        $field_value = $hash_ref->{$field_name};

                        $dc_field_name = $field_name;
                        &field_setup;
                        &get_top_field_attributes;
                        &get_field_setup_attributes;
                        &process_field_setup_subs;

                        # now do special field processing and create html, etc.

                        $field_html = '';

                        if ( $display_only =~ /yes/ )
                                {
                                # new in 2.0
                                $field_value = escapeHTML($default);
                                $display_field_value = $field_value;

                                if ( $html_type eq 'textarea' )
                                        {
                                        $display_field_value =~ s/\n/<br>/g;
                                        }

                                $field_html = qq~
                                <input type="hidden" name="$field_name" value="$field_value">
                                $display_field_value
                                ~;
                                # end of new in 2.0
                                }
                        else
                                {
                                # 'html_type'         => 'text',
                                # 'html_size'         => '10',
                                # 'max_length'        => '10',

                                if ( $html_type eq 'text' )
                                        {
                                        $field_value = $default;
                                        $field_value = escapeHTML($field_value);

                                        $field_html  = qq~<input type="text" name="$field_name" size="$html_size" maxlength="$max_length" value="$field_value">~;
                                        }
                                elsif ( $html_type eq 'password' )
                                        {
                                        $field_value = $default;
                                        $field_html  = qq~<input type="password" name="$field_name" size="$html_size" maxlength="$max_length" value="$field_value">~;
                                        }
                                elsif ( $html_type eq 'textarea' )
                                        {
                                        $field_value = $default;
                                        $field_value = escapeHTML($field_value);
                                        $field_html  = qq~<textarea $html_size name="$field_name" wrap="$wrap">$field_value</textarea>~;
                                        }
                                elsif ( $html_type eq 'scrolling_list' )
                                        {
                                        $field_html = $q->scrolling_list(
                                                                     -name     => $field_name,
                                                                     -values   => $values,
                                                                     -default  => $default,
                                                                     -labels   => $labels,
                                                                     -size     => $size,
                                                                     -multiple => $multiple
                                                                     );
                                        }
                                elsif ( $html_type eq 'popup_menu' )
                                        {
                                        $field_html = $q->popup_menu(
                                                                     -name     => $field_name,
                                                                     -values   => $values,
                                                                     -labels   => $labels,
                                                                     -default  => $default
                                                                     );
                                        }
                                elsif ( $html_type eq 'radio_group' )
                                        {
                                        $field_html = $q->radio_group(
                                                                     -name     => $field_name,   
                                                                     -values   => $values, 
                                                                     -default  => $default,
                                                                     -labels   => $labels, 
                                                                     -linebreak=> $linebreak
                                                                     );
                                        }
                                elsif ( $html_type eq 'checkbox_group' )
                                        {
                                        $field_html = $q->checkbox_group(
                                                                     -name     => $field_name,   
                                                                     -values   => $values, 
                                                                     -default  => $default,
                                                                     -labels   => $labels, 
                                                                     -linebreak=> $linebreak
                                                                     );
                                        }

                                                  

                                }
  
                        # new in v2.0

                        # ... now branch, depending on method of display
                        #..................................................................
                
                        if ( $display_method eq '_standard_' or $display_method eq '' )
                             {
                             $record_repeat .= $record2;
                             $record_repeat =~ s/:!:field_label:!:/$label/g;
                             $record_repeat =~ s/:!:field_value:!:/$field_html/g;
                             }
                        else
                             {
                             if ( $record2 =~ /:!:$field_name:!:/ )
                                   {
                                   $record2 =~ s/:!:$field_name:!:/$field_html/g;
                                   }
                             else
                                   {
                                   $field_value = $default;
                                   $field_html = qq~
                                   <input type="hidden" name="$field_name" value="$field_value">
                                   :!:missing_fields_made_hidden:!:
                                   ~;
                                   $record2 =~ s/:!:missing_fields_made_hidden:!:/$field_html/g;
                                   }
                             }
                
                        #........................

                        }

                if ( $display_method eq '_standard_' or $display_method eq '' )
                    {
                    $record2 = $record_repeat;
                    }

                }

        #...............................................

        $sub_head2 = $sub_head;
        #HASH LOOKUP#
        if ( $hash_ref->{$sub_head_test_field} ne $sub_head_test_field_str )
                {
                $sub_head_test_field_str = $hash_ref->{$sub_head_test_field};
                $sub_head2 =~ s/:!:$sub_head_print_field:!:/$hash_ref->{$sub_head_print_field}/g;
                $body_all .= $sub_head2
                }

        if ( $display_method eq '_standard_' or $display_method eq '' )
          {
          $body_all .= $record2;
          }
        else
          {
          $body_all = $record2;
          }

        $offset++;
        $page_recs_displayed++;

        }
#....................................

if ( $display_method eq '_standard_' or $display_method eq '' )
     {
     if ( 
          ( $show_copyright eq 'no' and $user_level eq 'public' )
          or
          ( $show_copyright eq 'no' and $a_ =~ /custom/ )
        )
          {
          $body_all .= $table_end;
          }
     else
          {
          $body_all .= $table_end . $copyright;
          $body_all .= &admin_little_menu;
          }
     }
else
     {
     $admin_menu = &admin_little_menu;
     }

$body_all =~ s/:!:colspan:!:/$colspan/g;

$w_array_all = join '|', @w_array_all;
$body_all =~ s/:!:w_array_all:!:/$w_array_all/g;

# set up previous and next links
#..........................................................................

$use_previous = 'no';
$use_next     = 'no';

if ( $p_ eq 'm' )
        {

        if ( $a_ eq 'view_custom_list' and $sub_function =~ /\w+/ )
               {
               $prev_next_a_ = 'view_custom_list_._' . $sub_function;
               }
        else
               {
               $prev_next_a_ = $a_;
               }

        $wh_esc_str = '';
        if ( $wh_ )
                {
                $wh_esc = escapeHTML($wh_);
                $wh_esc_str = qq~
                <input type="hidden" name="wh_" value="$wh_esc">
                ~;
                }

        $or_esc_str = '';
        if ( $or_ )
                {
                $or_esc = escapeHTML($or_);
                $or_esc_str = qq~
                <input type="hidden" name="or_" value="$or_esc">
                ~;
                }

        if ( $offset > $r_ )
                {

                $offset_p = ($offset - $page_recs_displayed) - $r_;
                if ( $offset_p < 0 ){$offset_p = 0;}

                $offsetlink_p = qq~
                <table border=0 cellpadding=0 cellspacing=0>
                <tr><td>
                <form method="post" action="$futuresql_url">
                </td><td>
                <input type="hidden" name="w_" value="$w_">
                $wh_esc_str
                $or_esc_str
                <input type="hidden" name="s_"  value="$s_">
                <input type="hidden" name="dc_"  value="$dc_">
                <input type="hidden" name="r_"  value="$r_">
                <input type="hidden" name="p_"  value="$p_">
                <input type="hidden" name="rt_" value="$rt_">
                <input type="hidden" name="a_" value="$prev_next_a_">
                <input type="hidden" name="o_" value="$offset_p">
                $security_fields
                $previous_submit_button
                ~;

### -> new
                foreach $custom_tag ( @custom_prev_next_tags )
                    {
                    $custom_tag_value = escapeHTML($$custom_tag);
                    $offsetlink_p .= qq~
                    <input type="hidden" name="$custom_tag" value="$custom_tag_value">
                    ~;
                    }

                $offsetlink_p .= qq~
                </td><td>
                </form>
                </td></tr></table>
                ~;

                $use_previous = 'yes';
                }

        if ( $offset < $total_rows )
                {
                $offsetlink = qq~
                <table border=0 cellpadding=0 cellspacing=0>
                <tr><td>
                <form method="post" action="$futuresql_url">
                </td><td>
                <input type="hidden" name="w_" value="$w_">
                $wh_esc_str
                $or_esc_str
                <input type="hidden" name="s_"  value="$s_">
                <input type="hidden" name="dc_"  value="$dc_">
                <input type="hidden" name="r_"  value="$r_">
                <input type="hidden" name="p_"  value="$p_">
                <input type="hidden" name="rt_" value="$rt_">
                <input type="hidden" name="a_" value="$prev_next_a_">
                <input type="hidden" name="o_" value="$offset">
                $security_fields
                $next_submit_button
                ~;

### -> new
                foreach $custom_tag ( @custom_prev_next_tags )
                    {
                    $custom_tag_value = escapeHTML($$custom_tag);
                    $offsetlink .= qq~
                    <input type="hidden" name="$custom_tag" value="$custom_tag_value">
                    ~;
                    }

                $offsetlink .= qq~
                </td><td>
                </form>
                </td></tr></table>
                ~;

                $use_next = 'yes';
                }

        }

# now parse tags or append forms
#...........................................................

&do_post_op;

$template_text = &process_template('get_records');
$body_next_previous_string = '';

if ( $use_previous eq 'yes' or $use_next eq 'yes' )
     {
     # assemble header
     #...........................................................................
     if ( $template_text =~ /:!:prev_next_header:!:/ )
             { $template_text =~ s/:!:prev_next_header:!:/$prev_next_header/g; }
     else    { $body_next_previous_string .= $prev_next_header; }

     # assemble previous string
     #...........................................................................
     if ( $use_previous eq 'yes' )
          {
          if ( $template_text =~ /:!:previous_page:!:/ )
                  { $template_text =~ s/:!:previous_page:!:/$offsetlink_p/g; }
          else    {
                    $previous_page2 = $previous_page;
                    $previous_page2 =~ s/:!:previous_page:!:/$offsetlink_p/g;
                    $body_next_previous_string .= $previous_page2;
                  }
          }
     else
          {
          # get rid of tag
          $template_text =~ s/:!:previous_page:!://g;
          }               

     # assemble next string
     #...........................................................................
     if ( $use_next eq 'yes' )
          {
          if ( $template_text =~ /:!:next_page:!:/ )
                  { $template_text =~ s/:!:next_page:!:/$offsetlink/g; }
          else    { 
                    $next_page2 = $next_page;
                    $next_page2 =~ s/:!:next_page:!:/$offsetlink/g;
                    $body_next_previous_string .= $next_page2; 
                  }
          }
     else
          {
          # get rid of tag
          $template_text =~ s/:!:next_page:!://g;
          }               

     # assemble footer
     #...........................................................................
     if ( $template_text =~ /:!:prev_next_footer:!:/ )
             { $template_text =~ s/:!:prev_next_footer:!:/$prev_next_footer/g; }
     else    { $body_next_previous_string .= $prev_next_footer; }


     $template_text =~ s/:!:next_previous_check:!:/$body_next_previous_string/g;

     }
else
     {
     # get rid of tags if they exist
     $template_text =~ s/:!:prev_next_header:!://g;
     $template_text =~ s/:!:previous_page:!://g;
     $template_text =~ s/:!:next_page:!://g;
     $template_text =~ s/:!:prev_next_footer:!://g;
     $template_text =~ s/:!:next_previous_check:!://g;
     }

#........................................................................

# &do_post_op is done above

$template_text = &parse_params($template_text);

print $template_text;

}
##################################################################
# save_record
sub save_record
{
    
my($q) = @_;

&do_pre_op;

# we first copy $field_order for our redisplay at the bottom
$field_str_redisplay = $field_order;

# assemble field string
#....................................

$field_order =~ s/\s+//g;
@field_order = split /,/, $field_order;

%field_values = ();

# create save or update string
#....................................

$field_str = "";
$value_str = "";
$update_str = "";

foreach $field_name ( @field_order )
        {
        @value_array = param($field_name);

        $dc_field_name = $field_name;
        &field_setup;
        &get_top_field_attributes;

if    ( $a_ eq "save_new_record" )
        {
        #HASH LOOKUP#
        if ( $dictionary{fields}{$dc_field_name}{insert_expression} )
           {$sub_name = $dictionary{fields}{$dc_field_name}{insert_expression};
            $value = &$sub_name if defined &$sub_name;                                                           
           } else {&hash_error("dictionary error - $dc_field_name - insert_expression");}
        }

elsif ( $a_ eq "update_record" )
        {
        #HASH LOOKUP#
        if ( $dictionary{fields}{$dc_field_name}{update_expression} )
           {$sub_name = $dictionary{fields}{$dc_field_name}{update_expression};
            $value = &$sub_name if defined &$sub_name;                                                           
           } else {&hash_error("dictionary error - $dc_field_name - update_expression");}
        }

        $field_values{$field_name} = $value;

        if ( $field_name eq $key_field )
                {
                $update_id = $value;
                }

        # this will process records marked as 'yes_but_process'

        if ( $display_only eq 'yes' )
                {
                next;
                }
        else
                {
                # updated for v1.3
                $value = $dbh->quote($value);
                $field_str .= "$field_name,";
                #................................................

                # updated for v1.3
                $value_str .= "$value,";
                $update_str .= "$field_name = $value,";
                #................................................
                }
        }

#..................................

if ( substr($field_str,-1,1) eq ',')
        {
        substr($field_str, -1,1) = '';
        }

if ( substr($value_str,-1,1) eq ',')
        {
        substr($value_str, -1,1) = '';
        }

if ( substr($update_str,-1,1) eq ',')
        {
        substr($update_str, -1,1) = '';
        }

if    ( $a_ eq "save_new_record" )
        {
        $sth_i = $dbh->do("insert into $table_name ($field_str) values ($value_str)")
                       or die
                       &error_message(qq~Error!<p>
                       Can't save data: $DBI::errstr~);

        #  $last_insert_id = $dbh->func("_InsertID");
        $last_insert_id = $dbh->{'mysql_insertid'};

        $field_values{$key_field} = $last_insert_id;
        $w_ = $last_insert_id;

        $response_msg = qq~
        <font color="darkred"><b>
        &nbsp; - Record [$last_insert_id] has been Added.
        </b></font>
        ~;

        }
elsif ( $a_ eq "update_record" )
        {
        $fin_update_str = "update $table_name
                           set $update_str
                           where
                           $where_str
                          ";

        $sth_i = $dbh->do("$fin_update_str")
                       or die
                       &error_message(qq~Error!<p>
                       Can't save data: $DBI::errstr~);

        $w_ = $update_id;

        $response_msg = qq~
        <font color="darkred"><b>
        &nbsp; - Record [$update_id] has been Updated.
        </b></font>
        ~;

        }

for $field_name ( keys %field_values )
        {
        $field_value = $field_values{$field_name};
        $$field_name = $field_value;
        }

$body_all = '';

&do_post_op;

# now redisplay data, using the view_record syntax
#.....................................................................

# finalize select string
#....................................

$select_str = "
               select
               $field_str_redisplay
               from
               $table_name
               where
               $key_field = '$w_'
              ";

# now get row
#........................................

$sth = $dbh->prepare("$select_str")
                     or die
                     &error_message(qq~Error!<p>
                     Can't do select: $DBI::errstr~);

$rv = $sth->execute or die
                    &error_message(qq~Error!<p>
                    Can't execute the query: $DBI::errstr~);

# get record
#....................................

$hash_ref = $sth->fetchrow_hashref;
$key_value = $hash_ref->{$key_field};

# 'display_method'    => (_standard_, _custom_page_, _external_template_)

if ( $display_method eq '_standard_' )
     {
     $body2 = $login_data . $where_order_line . $response_msg . $table_start;
     $record2 = $record_text;
     }
elsif ( $display_method eq '_custom_page_' )
     {
     $record2 = $custom_page;
     $record2 =~ s/:!:custom_page_fields:!:/$custom_page_fields/g;
     }
elsif ( $display_method eq '_external_template_' )
     {
     $record2 = &include_file("$custom_template_dir/$external_template");
     }

$record2       =~ s/:!:page_header:!:/$page_header/g;
$record2       =~ s/:!:menu_control_text:!:/$menu_control_text/g;
$record2       =~ s/:!:page_form_start:!:/$page_form_start/g;
$record2       =~ s/:!:page_form_system_fields:!:/$page_form_system_fields/g;
$record2       =~ s/:!:page_form_end:!:/$page_form_end/g;
$record2       =~ s/:!:page_footer:!:/$page_footer/g;

$record2       =~ s/:!:key_field:!:/$key_value/g;
$table_end     =~ s/:!:key_field:!:/$key_value/g;

$record_repeat = '';

foreach $field_name ( @field_order )
        {
        $field_value = $hash_ref->{$field_name};

        $dc_field_name = $field_name;
        &field_setup;
        &get_top_field_attributes;

        # new in 2.0
        # note use of '$display_field_value' below

        $display_field_value = &get_display_field_value($dc_field_name, $field_value);
        #..............................................................................

        if ( $view_with_html_escape eq 'yes' )
                {
                $display_field_value = escapeHTML($display_field_value);
                }

        if ( $html_type eq 'textarea' )
                {
                $display_field_value =~ s/\n/<br>/g;
                }

        # ... now branch, depending on method of display
        #..................................................................

        if ( $display_method eq '_standard_' )
             {
             $record_repeat .= $record2;
             $record_repeat =~ s/:!:field_label:!:/$label/g;
             $record_repeat =~ s/:!:field_value:!:/$display_field_value/g;
             }
        else
             {
             $record2 =~ s/:!:$field_name:!:/$display_field_value/g;
             }

        }
        # end of field loop

if ( $display_method eq '_standard_' )
     {
     $record2 = $record_repeat;
     $body_all .= $body2;
     $body_all .= $record2;
     $body_all .= $table_end . $copyright;
     $body_all .= &admin_little_menu;
     }
else
     {
     $body_all .= $record2;
     $admin_menu = &admin_little_menu;
     }

$template_text = &process_template;
$template_text = &parse_params($template_text);

# &do_post_op is done above

print $template_text;

}
##################################################################

# PLACE ACTION SUBS ABOVE THIS LINE FOR CLARITY

##################################################################
# logout
sub logout
{
    
warn $DBI::errstr if $DBI::err;
if ( $dbh ){$rcdb = $dbh->disconnect;}
    
}
###################################################################
# get_dictionary_hash_vars
sub get_dictionary_hash_vars
{

# get table structure
#HASH LOOKUP#

&field_setup;

if ( $dictionary{table_name} ){$table_name = $dictionary{table_name};}
else             {&hash_error("$dc_ - table_name");}

if ( $dictionary{key_field}    )
        {$key_field = $dictionary{key_field};}
else    {&hash_error("$table_name - key_field");}

if ( $dictionary{field_order}    )
        {$field_order = $dictionary{field_order};}
else    {&hash_error("$table_name - field_order");}

if ( $dictionary{user_field_order}    )
        {$user_field_order = $dictionary{user_field_order};}
else    {&hash_error("$table_name - user_field_order");}

if ( $dictionary{search_fields}    )
        {$search_fields = $dictionary{search_fields};}
else    {&hash_error("$table_name - search_fields");}

}
##################################################################
# hash_error
sub hash_error
{

my ($hash_error) = @_;

&error_message(qq~Error!<p>
Configuration Error! Undefined Value in:
<p>
[ $hash_error ]
<p>
Please contact the system admin.
~);
    
}
##################################################################
# check_params
sub check_params
{

if ( 
     param('s_')  eq '' or
     param('dc_')  eq '' or
     param('r_')  eq '' or
     param('p_')  eq '' or
     param('rt_') eq '' or
     param('a_')  eq ''
   )
        {
        &missing_param;
        }

if (
    param('w_')          eq '' and
    param('w_array')     eq '' and
    param('w_array_all') eq ''
   )
        {
        $w_ = 1;
        # &missing_param;
        }

# clean up insecure characters?

# foreach $i (param)
#         {
#         $tmp = param($i);
#         $tmp =~ s/\`//g;
#         $tmp =~ s/\|//g;
#         $tmp =~ s/\<//g;
#         $tmp =~ s/\>//g;
# 
#         param($i,$tmp);
#         }

}
##################################################################
# missing_param
sub missing_param
{

$user_level = param('user_level');
    
my $error_message = qq~
Error!<p>
A Search Parameter is missing.
<p>
The problem could be:
<p>
a - you didn't select a record<br>
b - you didn't select the right menu option<br>
c - there's a problem with the link or button going to the script<br>
d - there's some other problem with the script.
<p>
Please hit BACK and try again.
<p>
~;

if ( param('user_level') eq 'admin' )
     {
     $error_message .= qq~
                       <font size=3>
                       <div align=left><blockquote><pre>Parameters Passed:<p>~;

     my @missing_parms = qw[s_ dc_ r_ p_ rt_ a_];
     foreach my $mp ( @missing_parms )
          {
          if ( param("$mp") !~ /\w+/ )
               {
               $error_message .= "<b><font color='red'>$mp" . (' ' x (5 - length($mp))) . " - [" . param("$mp") . "]</font></b><br>";
               }
          else {
               $error_message .= "$mp" .    (' ' x (5 - length($mp))) . " - [" . param("$mp") . "]<br>";
               }
          }

     $error_message .= qq~</pre></blockquote></div></font>~;
     }

&error_message($error_message);
exit;
    
}
###################################################################
# admin_little_menu
sub admin_little_menu
{

my ($menu_text);

$menu_text = ' ';

if ( $user_level eq $admin_access )
     {
     $menu_text = $admin_little_menu_text;
     $menu_text = &parse_params($menu_text);
     }

return($menu_text);
    
}
##################################################################
# convert_to_array
sub convert_to_array
{

# this is used for popup fields, using CGI.pm

# &convert_to_array(' ')
# $field_value is declared in main code

my ($delim) = @_;
my (@array);

@array = split /$delim/, $field_value;

return(\@array);

}
########################################################################
# join_array
sub join_array
{

# this is used for popup fields, using CGI.pm

# &join_array('|') or &join_array(' '), etc.
# @value_array is declared in main code

my ($delim) = @_;
my ($string);

$string = join $delim, @value_array;

return($string);

}
########################################################################
# include_file
sub include_file
{

my($i_file) = @_;
my($filesize);
my($i_text);

#obtain size of file
$filesize = -s $i_file;

# open file
unless ( open(INCLUDE_FILE, $i_file ))
{
    die
    &error_message(qq~Error!<p>
    Can't open File $i_file.~);
}

# read whole file into variable
read (INCLUDE_FILE, $i_text, $filesize);

close(INCLUDE_FILE);

return($i_text);
    
}
###################################################################
# get_date
sub get_date
{
    
my ($day, $month, $year) = (localtime)[3,4,5];
my ($datestr) = sprintf("%04d-%02d-%02d", $year+1900, $month+1, $day);

return($datestr);    
    
}
#############################################################
# do_pre_op
sub do_pre_op
{
#HASH LOOKUP#

if ( $s_spec{action}{$a_}{pre_op} )
   {$sub_name = $s_spec{action}{$a_}{pre_op}; 
    &$sub_name if defined &$sub_name;                                                           
   } else {&hash_error("$s_ - $a_ - pre_op");}

}
#############################################################
# do_post_op
sub do_post_op
{
#HASH LOOKUP#
    
if ( $s_spec{action}{$a_}{post_op} )
   {$sub_name = $s_spec{action}{$a_}{post_op}; 
    &$sub_name if defined &$sub_name;                                                           
   } else {&hash_error("$s_ - $a_ - post_op");}
    
}
#############################################################
# parse_params
sub parse_params
{

# parse all parameters
# $body_all = &parse_params($body_all);

my ($body_all) = @_;

$body_all =~ s/:!:a_:!:/$a_/g;
$body_all =~ s/:!:s_:!:/$s_/g;
$body_all =~ s/:!:dc_:!:/$dc_/g;
$body_all =~ s/:!:r_:!:/$r_/g;
$body_all =~ s/:!:p_:!:/$p_/g;
$body_all =~ s/:!:rt_:!:/$rt_/g;
$body_all =~ s/:!:w_:!:/$w_/g;
$body_all =~ s/:!:wh_:!:/$wh_/g;
$body_all =~ s/:!:or_:!:/$or_/g;
$body_all =~ s/:!:where_str:!:/$where_str/g;
$body_all =~ s/:!:order_str:!:/$order_str/g;

$body_all =~ s/:!:system_hidden_fields:!:/$system_hidden_fields/g;
$body_all =~ s/:!:futuresql_url:!:/$futuresql_url/g;
$body_all =~ s/:!:script_name:!:/$script_name/g;

$body_all =~ s/:!:security_fields:!:/$security_fields/g;

$body_all =~ s/:!:copyright:!:/$copyright/g;
$body_all =~ s/:!:admin_menu:!:/$admin_menu/g;

# we assume here that all missing fields (in edits or adds) have been
# assembled prior to this delete

$body_all =~ s/:!:missing_fields_made_hidden:!://g;

# get rid of previous / next tags if they exist
# (these are parsed in &get_records, prior to this,
#  and they're not needed anywhere else)

$body_all =~ s/:!:prev_next_header:!://g;
$body_all =~ s/:!:previous_page:!://g;
$body_all =~ s/:!:next_page:!://g;
$body_all =~ s/:!:prev_next_footer:!://g;
$body_all =~ s/:!:next_previous_check:!://g;

return($body_all);

}
#############################################################
# process_template
sub process_template
{

my ($routine) = @_;
my ($template_text);

if ( $display_method eq '_external_template_' )
     {
     $template_text = $body_all;
     if ( $routine eq 'get_records' )
          {
          $template_text .= ':!:next_previous_check:!:';
          }
     }
else
     {
     $template_text = &include_file("$template_dir/$rt_\.$template_ext");
     if ( $routine eq 'get_records' )
          {
          $body_final = $body_all . ':!:next_previous_check:!:';
          $template_text =~ s/$text_tag/$body_final/g;
          }
     else
          {
          $template_text =~ s/$text_tag/$body_all/g;
          }
     }

if ( $page_title !~ /\w+/ )
     {
     $page_title = $default_page_title;
     }

$template_text =~ s/:!:title:!:/$page_title/g;

foreach $tag_name ( keys %template_tags )
        {
        for $attrib_name ( keys %{$template_tags{$tag_name}} )
                {
                $attrib_value = $template_tags{$tag_name}{$attrib_name};
                $$attrib_name = $attrib_value;
                }

                if ( $tag_process_status eq 'active' )
                        {
                        $template_text =~ s/$tag_code_in_template/$$tag_value_variable_name/g;
                        }
                else
                        {
                        $template_text =~ s/$tag_code_in_template//g;
                        }
        }

return($template_text);

}
#############################################################
# assemble_table_headers
sub assemble_table_headers
{

$table_header2 = $table_header;
$record_repeat = '';

foreach $field_name ( @field_order )
        {
        for $attrib_name ( keys %{$dictionary{fields}{$field_name}} )
                {
                $attrib_value = $dictionary{fields}{$field_name}{$attrib_name};
                $$attrib_name = $attrib_value;
                }

        $record_repeat .= $table_header2;
        $record_repeat =~ s/:!:field_label:!:/$label/g;
        }

$table_header2 = $record_repeat;

return($table_header2);

}
#############################################################
# check_for_qbf
sub check_for_qbf
{

my ($field_name, $search_param, $search_type, $search_value);
my ($sort_field_1, $sort_field_dir_1);
my ($sort_field_2, $sort_field_dir_2);
my ($sort_field_3, $sort_field_dir_3);

if ( param('qbf') eq 'yes' )
       {
       # new
       # 'show_select_params'    => 'yes',
       # 'select_param'          => 'like',
       # 'show_sort_params'      => 'yes',
       # 'sort_order'            => qq~$key_field desc~,

       if ( param('show_sort_params') eq 'no' )
          {
          $or_ = param('sort_order');
          }

       else
          {
          $sort_field_1     = param('sort_field_1');
          $sort_field_dir_1 = param('sort_field_dir_1');
          
          $sort_field_2     = param('sort_field_2');
          $sort_field_dir_2 = param('sort_field_dir_2');
          
          $sort_field_3     = param('sort_field_3');
          $sort_field_dir_3 = param('sort_field_dir_3');
          
          $order_str = '';
          $or_ = "$sort_field_1 $sort_field_dir_1\, ";
          
          if ( $sort_field_dir_2 ne 'ignore' )
                  {
                  $or_ .= "$sort_field_2 $sort_field_dir_2\, ";
                  }
          
          if ( $sort_field_dir_3 ne 'ignore' )
                  {
                  $or_ .= "$sort_field_3 $sort_field_dir_3\, ";
                  }
          
          if ( substr($or_ , -2, 2) eq ', ')
                  {
                  substr($or_ , -2, 2) = '';
                  }
          
          }

       #...........................

       $order_str = $or_;

       #...........................

          $wh_ = '';
          $where_str = '';
   
          if ( $use_join_str eq 'yes' )
               {
               $search_fields = $join_field_order;
               }

          $search_fields =~ s/\s+//g;
          @search_fields = split /,/, $search_fields;
          
          $show_select_params = param('show_select_params');
          $select_param       = param('select_param');

          foreach $field_name ( @search_fields )
                  {
                  $wh_field_name = $field_name;

                  # replace dc_prefix with sql join syntax prefix ('.')
                  #............................................................
                  if ( $use_join_str eq 'yes' )
                  {
                  foreach $alias_key ( keys %join_fields_alias_hash )
                        {
                        if ( $field_name =~ /^$alias_key/ )
                             {
                             $wh_alias = $join_fields_alias_hash{$alias_key};
                             $wh_field_name =~ s/^$alias_key/$wh_alias\./;
                             next;
                             }
                        }
                  }
                  #............................................................

                  if ( $show_select_params eq 'no' )
                         {
                         $search_value = param($field_name);
          
                         # new in v2.0 - changed \w+ to \S+
                         # so that qbf can match non word characters in searches
                         
                         if ( $search_value =~ /\S+/ )
                             {
                             if ( $select_param =~ /like/ )
                                  {
                                  $search_value = '%' . $search_value . '%';
                                  }
                             $wh_ .= qq~( $wh_field_name $select_param '$search_value' ) and~;
                             }

                         }
                  else
                         {
                         $search_param = $field_name . '_search_type';
                         $search_type  = param($search_param);
                         $search_value = param($field_name);
          
                         if ( $search_type =~ /like/ )
                             {
                             $search_value = '%' . $search_value . '%';
                             }
          
                         if ( ($search_type ne 'ignore') and ($search_type) )
                             {
                             $wh_ .= qq~( $wh_field_name $search_type '$search_value' ) and~;
                             }
                         }
                  }
   
           if ( $wh_ eq '' )
                   {
                   &error_message(qq~
                   Error!
                   <p>
                   You must use at least one of the selection
                   fields.
                   <p>
                   Please hit BACK and try again.
                   ~);
                   }
   
           if ( substr($wh_, -4, 4) eq ' and')
                   {
                   substr($wh_, -4, 4) = '';
                   }
   
       #....................................................

       $where_str = $wh_;

        if ( $where_str and $order_str )
                {
                $where_order_line = qq~
                <br><i>where <font color="darkblue">$where_str</font>
                \~ order by <font color="darkred">$order_str</font></i>
                ~;
                }

       }

}
#############################################################
# global_update
sub global_update
{
    
my($q) = @_;
my ($response_text);
    
$update_field = param('update_field');
$new_value    = param('new_value');

$wh_esc = escapeHTML($where_str);
$new_value_esc = escapeHTML($new_value);

# modified on 2001-08-18
# to allow blank values

unless ( $update_field =~ /\w+/ )
        {
        $response_text = qq~
        <font size=+1><b>
        Update Parameters Not Selected
        </b></font>
        <blockquote>
        You must select a field and type in a valid value.
        </blockquote>
        ~;
        return ($response_text);
        }

$fin_update_str = qq~update $table_name
                   set $update_field = $new_value
                   where
                   $where_str
                  ~;

$response_text = qq~
<font size=+1><b>
Update Confirmation
</b></font>
<blockquote>
<font size=-1>
<b>NOTE!</b> This function should be used only if
you understand MySQL syntax. For example, when replacing
a field with a literal value, like 'Mr. Smith', the literal
value must be surrounded by single quote marks, as it is above.
On the other hand, you do not need the quote marks when you're
replacing one field with another; instead you just use the replacing field
name without quotes. Please consult
<a href="http://www.mysql.com">www.mysql.com</a> for reference. SQL 
(which stands for 'Structured Query Language') is not <b>that</b>
intimidating, but you can easily globally replace or delete data by
mistake. <b>The key point: backup your table first.</b>
</font>
<p>
<form method="post" action="$futuresql_url">
$system_hidden_fields
<input type="hidden" name="w_"  value="$w_">
<input type="hidden" name="rt_" value="$rt_">
<input type="hidden" name="or_" value="$order_str">
<input type="hidden" name="a_" value="admin_functions_._confirm_global_update">
<input type="hidden" name="update_field" value="$update_field">
<input type="hidden" name="new_value" value="$new_value_esc">
<input type="hidden" name="wh_" value="$wh_esc">

Your Update String will be processed as:<br>
[$fin_update_str]
<p>
Is that correct?
<p>
<font size=-1>
<input type="submit" value=" Confirm Global Update ">
</form>
</font>
</blockquote>
~;    
  
return ($response_text);    

}
#############################################################
# global_delete
sub global_delete
{
    
my ($q) = @_;
my ($response_text);

$wh_esc = escapeHTML($where_str);

unless ( $wh_ =~ /\w+/ )
        {
        $response_text = qq~
        <font size=+1><b>
        Delete Parameters Not Selected
        </b></font>
        <blockquote>
        You must type in a valid where string value.
        </blockquote>
        ~;
        return ($response_text);
        }
    
$delete_str = qq~delete from $table_name
                 where
                 $where_str
                ~;

$response_text = qq~
<font size=+1><b>
Delete Confirmation
</b></font>
<blockquote>
<font size=-1>
<b>NOTE!</b> This function should be used only if
you understand MySQL syntax. Please consult
<a href="http://www.mysql.com">www.mysql.com</a> for reference. SQL 
(which stands for 'Structured Query Language') is not <b>that</b>
intimidating, but you can easily globally delete data by
mistake. <b>The key point: backup your table first.</b>
</font>
<p>
<form method="post" action="$futuresql_url">
$system_hidden_fields
<input type="hidden" name="w_"  value="$w_">
<input type="hidden" name="rt_" value="$rt_">
<input type="hidden" name="or_" value="$order_str">
<input type="hidden" name="a_" value="admin_functions_._confirm_global_delete">
<input type="hidden" name="wh_" value="$wh_esc">

Your Delete String is:<br>
[$delete_str]
<p>
Is that correct?
<p>
<font size=-1>
<input type="submit" value=" Confirm Global Delete ">
</form>
</font>
</blockquote>
~;    
  
return ($response_text);    
    
}
#############################################################
# confirm_global_update
sub confirm_global_update
{
    
my($q) = @_;
my ($response_text);
    
$update_field = param('update_field');
$new_value    = param('new_value');
$new_value    = unescapeHTML($new_value);

$fin_update_str = qq~update $table_name
                   set $update_field = $new_value
                   where
                   $where_str
                  ~;

$sth_u = $dbh->do("$fin_update_str")
               or die
               &error_message(qq~Error!<p>
               Can't save data: $DBI::errstr~);

$response_text = qq~
<font size=+1><b>
Update Completed
</b></font>
<blockquote>
Your Update String was:<br>
[$fin_update_str]
<p>
The Update is Now DONE.
<p>
<form method="post" action="$futuresql_url">
$system_hidden_fields
<input type="hidden" name="w_"  value="$w_">
<input type="hidden" name="rt_" value="$rt_">
<input type="hidden" name="wh_" value="$where_str">
<input type="hidden" name="a_" value="view_admin_list">
<input type="submit" value=" View Updated Records ">
</form>
</blockquote>
~;    

$response_text .= &admin_little_menu;
  
return ($response_text);    

}
#############################################################
# confirm_global_delete
sub confirm_global_delete
{
    
my ($q) = @_;
my ($response_text);

$delete_str = qq~delete from $table_name
                 where
                 $where_str
                ~;

$sth_d = $dbh->do("$delete_str")
               or die
               &error_message(qq~Error!<p>
               Can't save data: $DBI::errstr~);
    
$response_text = qq~
<font size=+1><b>
Delete Completed
</b></font>
<blockquote>
Your Delete String was:<br>
[$delete_str]
<p>
</blockquote>
~;    

$wh_ = qq~$key_field > 0~;

$response_text .= &admin_little_menu;
  
return ($response_text);    
    
}
#############################################################
# mysql_backup
sub mysql_backup
{
    
my ($response_text);

$response_text = qq~
<font size=+1><b>
MySQL Backup of [$table_name] Table Completed
</b></font>
<hr>
Results:
<p>
~;    

$datetext = &get_backup_date_string;

$response_text .= qq~
<pre>
Directory: $mysql_backup_dir

~;

$script_dir = cwd();

chdir ("$mysql_backup_dir");

$backup_file    = "bak." . $datetext . "_" . $db_main . "." . $table_name . "." . $mysql_dump_file_ext;

if ( $backup_type eq 'mysqldump' )
        {
        $mysqldumpcommand = "$mysqlbin/mysqldump --user=$w_user --password=$w_password $mysqldump_params";
        $response_text .= `$mysqldumpcommand $db_main $table_name > $mysql_backup_dir/$backup_file`;
        }
elsif ( $backup_type eq 'outfile' )
        {
        $backup_str = qq~
                      select * from $table_name
                      into outfile '$mysql_backup_dir/$backup_file'
                      $outfile_params
                      ~;

        $sth_b = $dbh->do("$backup_str")
                       or die
                       &error_message(qq~Error!<p>
                       Can't backup data: $DBI::errstr~);

        }
else
        {
        unless ( open(FILE, ">$mysql_backup_dir/$backup_file" ))
                {
                die
                &error_message(qq~Error!<p>
                Can't open File $mysql_backup_dir/$backup_file.~);
                }

        $sth_b = $dbh->prepare("select * from $table_name")
                 or die
                 &error_message(qq~Error!<p>
                 Can't do select for backup: $DBI::errstr~);

        $rv    = $sth_b->execute
                 or die
                 &error_message(qq~Error!<p>
                 Can't execute the query: $DBI::errstr~);

        while ( @row = $sth_b->fetchrow_array )
                {
                $row_string = '';

                foreach $field_value (@row)
                        {
                        $row_string .= $backup_field_enclosed_by .
                                       $field_value .
                                       $backup_field_enclosed_by .
                                       $backup_field_terminate;
                        }
                
                $len_field_terminate = length($backup_field_terminate);
                if ( substr($row_string,-$len_field_terminate,$len_field_terminate) eq $backup_field_terminate)
                        {
                        substr($row_string, -$len_field_terminate,$len_field_terminate) = '';
                        }

                $row_string .= $backup_line_terminate;

                print FILE $row_string;
                }        

        close(FILE);

        }

if ( $chmod_backup_file eq 'yes' )
        {
        chmod 0600, $backup_file;
        }

$response_text .= qq~File Just Backed Up:
~;

$response_text .= `ls -la ./$backup_file`;

$response_text .= qq~
All Files in Backup Dir:
~;

$response_text .= `ls -lat *`;
    
$response_text .= qq~
</pre>
~;    

chdir ("$script_dir");

$response_text .= &admin_little_menu;
    
return ($response_text);    
    
}
#############################################################
# get_backup_date_string
sub get_backup_date_string
{
    
my ($sec, $min, $hour, $day, $month, $year) = (localtime)[0,1,2,3,4,5];

  if ($hour < 10)
    {
    $hour = "0$hour";
    }
  if ($min < 10) 
    {
    $min = "0$min";
    }
  if ($sec < 10)
    { $sec = "0$sec";
    }

my ($backup_date_string) = sprintf("%04d-%02d-%02d", $year+1900, $month+1, $day) . "_$hour.$min.$sec";

return($backup_date_string);
    
}
#######################################################################
# process_field_setup_subs
sub process_field_setup_subs
{
#HASH LOOKUP#

if ( $dictionary{fields}{$dc_field_name}{field_setup}{values} )
   {$sub_name = $dictionary{fields}{$dc_field_name}{field_setup}{values};
    $values = &$sub_name if defined &$sub_name;                                                           
   } else {&hash_error("dictionary error - $dc_field_name - field_setup - values");}

if ( $dictionary{fields}{$dc_field_name}{field_setup}{default} )
   {$sub_name = $dictionary{fields}{$dc_field_name}{field_setup}{default};
    $default = &$sub_name if defined &$sub_name;                                                           
   } else {&hash_error("dictionary error - $dc_field_name - field_setup - default");}

if ( $dictionary{fields}{$dc_field_name}{field_setup}{labels} )
   {$sub_name = $dictionary{fields}{$dc_field_name}{field_setup}{labels};
    $labels = &$sub_name if defined &$sub_name;                                                           
   } else {&hash_error("dictionary error - $dc_field_name - field_setup - labels");}

}
#######################################################################
# assemble_header
sub assemble_header
{

$login_data = '';
$where_order_line = '';

unless ( $show_user_header eq 'no' and $user_level eq 'public' )
        {
        $login_data = qq~
        <font size=-1><b>
        Table [$table_name] / User [$w_user] / User_Level [$user_level]
        </b></font>
        ~;
        }

unless ( $show_sql_header eq 'no' and $user_level eq 'public' )
        {
        if ( $where_str and $order_str )
                {
                $where_order_line = qq~
                <font size=-1>
                <br><i>where <font color="darkblue">$where_str</font><br>
                order by <font color="darkred">$order_str</font></i>
                </font>
                ~;
                }
        }

}
#######################################################################
# get_custom_html
sub get_custom_html
{

# get custom html and SET default html var, if $var_name eq 'yes'

my ($var_value, $var_name, $file_value, $html_value, $tag);
my ($default_var_name, $suffix);

if ( -e "$custom_template_dir/$table_name\_fields.html" )
   { $default_external_template = "$table_name\_fields.html"; }

foreach $var_name ( keys %custom_html_usage )
        {
        if ( $custom_html_usage{$var_name} eq 'y' )
                {
                if ( $custom_html{$var_name} )
                    {

                    if ( $custom_html{$var_name}{file} )
                       { $file_value = $custom_html{$var_name}{file}; }
                
                    if ( $custom_html{$var_name}{html} )
                       { $html_value = $custom_html{$var_name}{html}; }
                
                    if ( $file_value eq 'y' )
                       {
                       $html_value =~ s/\s+//g;
                       $var_value = &include_file("$custom_template_dir/$html_value");
                       }
                    else
                       {
                       $var_value = $html_value;
                       }

                    foreach $tag ( @custom_html_tags )
                       {
                       $var_value =~ s/:!:$tag:!:/$$tag/g;
                       }

                    ($default_var_name, $suffix) = split /_tbl_/, $var_name, 2;
                    $$default_var_name = $var_value;

                    }
                }
        }

}
###############################################################################
# error_message
sub error_message
{

# &error_message($error_text);

# uncomment this line to get stack trace
# confess "Error: ";
     
my ($error_text) = @_;

if ( -e "$error_file" )
     {
     $template_text = &include_file("$error_file");
     $template_text =~ s/$text_tag/$error_text/g;
     print $template_text;
     }
else
     {
     print qq~
     $error_text
     ~;
     }

exit;
  
}
##################################################################
# print_message
sub print_message
{

# &print_message($message_text);
     
my ($message_text) = @_;

unless ( $title )
     {
     $title = $default_page_title;
     }

if ( -e "$message_file" )
     {
     $template_text = &include_file("$message_file");
     $template_text =~ s/$text_tag/$message_text/g;
     $template_text =~ s/:!:title:!:/$title/g;
     print $template_text;
     }
else
     {
     print qq~
     $message_text
     ~;
     }

exit;
  
}
##################################################################
# prep_joined_fields
sub prep_joined_fields
{

# read correct dictionary file when joined field prefix
# changes. this method seems a bit slow, but I want to avoid
# creating custom joined dictionary files, in order to
# avoid having to update more than one copy

# this routine also strips off the prefix in the dc_field_name,
# if necessary.

foreach $dc_join ( keys %join_dc_prefix_hash )
      {
      if ( $field_name =~ /^$dc_join/ )
           {
           $dc_field_name =~ s/^$dc_join//;
           $dc_prefix = $join_dc_prefix_hash{$dc_join};
           next;
           }
      }

if ( $dc_prefix ne $first_dc_prefix )
   {
   &read_dc_file($dc_prefix);
   $first_dc_prefix = $dc_prefix;
   }

}
##################################################################
# read_dc_file
sub read_dc_file
{

# &read_dc_file($dc_prefix);

my ($dc_prefix) = @_;

$dictionary_file = $futuresql_dict_prefix . $dc_prefix . $futuresql_specfile_suffix;

if ( -e "$dictionary_file" )
        {
        # we use 'do' instead of require because we load the file more than once
        do "$dictionary_file";
        }
else
        {
        &error_message(qq~Error!<p>
        The dictionary file [$dictionary_file] does not exist.
        <p>
        ~);
        }

# NOTE that the following variables must be set for a join to work:
# -> in pre_op: $use_join_str = 'yes';
# ->            $join_field_expressions = qq~
#                                         t.t_id as t_t_id,
#                                         ... etc ...~,
#               NOTE that we use aliases here because fetchrow_hashref
#               needs unique field names and the leading t. qualifier
#               gets chopped off, as well as subsequent duplicates getting
#               discarded. (see page 115 of Programming the Perl DBI)
# ->            $join_field_order = qq~
#                                   t_t_id,
#                                   ... etc ...~,
#               NOTE: these are ALIAS FIELD NAMES that correspond with the
#               expressions above. The names in $join_field_order will be
#               used to store field values and will be the field names that
#               need corresponding :!:field_names:!: in template files.
# ->            $join_table_names = qq~ 
#                                   table_name_1 t,
#                                   table_name_2 a
#                                   ... etc ...~;
#
# ->            %join_dc_prefix_hash = (
#                                      't_' => 't.1',
#                                      'a_' => 'a.1'
#                                      ); # (for example)
#               NOTE: this hash is used to reread the correct dictionary file
#               for each joined field, based on the PREFIX of each field getting
#               mapped to the correct dictionary atom.
#
# ->            In addition, the following keys in the 'spec' file need to be
#               adjusted to account for joined fields:
#               (these are just examples)
# ->            'hard_coded_where_str' => qq~(t.a_id = a.a_id)~,
# ->            'where_str'            => qq~( 1 == 1 )~,
# ->            'order_str'            => qq~a.last_name~,
#..........................................................................

}
##################################################################
# get_top_field_attributes
sub get_top_field_attributes
{

for $attrib_name ( keys %{$dictionary{fields}{$dc_field_name}} )
     {
     $attrib_value = $dictionary{fields}{$dc_field_name}{$attrib_name};
     $$attrib_name = $attrib_value;
     }

}
##################################################################
# get_field_setup_attributes
sub get_field_setup_attributes
{

for $attrib_name ( keys %{$dictionary{fields}{$dc_field_name}{'field_setup'} } )
        {
        $attrib_value = $dictionary{fields}{$dc_field_name}{'field_setup'}{$attrib_name};
        $$attrib_name = $attrib_value;
        }

}
##################################################################
# get_field_default_values
sub get_field_default_values
{
#HASH LOOKUP#

if ( $dictionary{fields}{$dc_field_name}{default_value} )
   {$sub_name = $dictionary{fields}{$dc_field_name}{default_value};
    $field_value = &$sub_name if defined &$sub_name;                                                           
   } else {&hash_error("dictionary error - $dc_field_name - default_value");}

}
##########################################################################
# get_display_field_value
sub get_display_field_value
{

# $display_field_value = &get_display_field_value($dc_field_name, $field_value);

my ($field_name, $field_value) = @_;
my $field_value_out;
my $display_field_value;

#HASH LOOKUP#
if ( $dictionary{fields}{$dc_field_name}{display_process} )
        {
        $sub_name = $dictionary{fields}{$dc_field_name}{display_process};

        # To work correctly, the anonymous subroutine in 'display_process'
        # should return the original value first ('field_value_out')
        # and then the new value ('display_field_value')

        # We then check to see if the value going INTO the subroutine ('field_value')
        # is the same as 'field_value_out'. (The subroutine also should return
        # a second value 'display_field_value', which should contain the newly
        # modified value.)
        
        # If 'field_value_out' is NOT the same as 'field_value' (the value going in)
        # then there's either NO subroutine, or there's some other problem, so we
        # revert back to the original 'field_value' and return that instead, as our
        # display value.

        # => In other words, we have to do our data integrity checking
        #    in THIS subroutine, inside the main file, rather than rely
        #    on an associated dictionary file that may or may not be setup right.
        #    This also allows us to leave the 'display_process' subroutine key
        #    BLANK, if we wish, with no ill effect.

        # SAMPLE ENTRY IN DATA DICTIONARY FILE FOR 'DISPLAY_PROCESS' KEY
        # ..............................................................................
        # 'display_process'   => sub {
        #                        my ($field_name, $field_value) = @_;
        #                        my $return_value = $field_value;
        # 
        #                        # now do something, either locally here,
        #                        # like this (this example converts a military
        #                        # time field to 12 hour time)
        #
        #                        $return_value = $hh_hash{$field_value};
        #
        #                        # or, using a a common subroutine
        #                        # $return_value = &military_to_standard_time($field_name, $field_value);
        # 
        #                        return($field_value, $return_value);
        #                        },
        # ..............................................................................

        ($field_value_out, $display_field_value) = &$sub_name($field_name, $field_value) if defined &$sub_name;

        }
else    {
        &hash_error("dictionary error - $dc_field_name - display_process");
        }

if ( "$field_value_out" ne "$field_value" )
        {
        # reset if not the same
        $display_field_value = $field_value;
        }

return($display_field_value);

}
##########################################################################
# notes on subroutine calls that use dictionary files
# $dc_field_name = $field_name;
# if ( $use_join_str eq 'yes' ){&prep_joined_fields;}
# &field_setup;
# &get_top_field_attributes;
# &get_field_setup_attributes;
# &get_field_default_values;
# &process_field_setup_subs;
# &get_display_process_values;

# we put a 1; so we can require this from a stub file in the web tree

1;
