#!/usr/bin/perl
#    Last change: PFB 2001-10-07 19:47:56
##########################################################################
# POD Documentation

=head1 PROGRAM NAME AND AUTHOR

fsql_make.cgi (this file is v2.1, October 7, 2001)
(makes a dictionary file for FutureSQL v2)

See fsql_main.cgi for Author and Copyright Information

!!! SEE THIS SCRIPT'S comments below, within the code.
  => this file runs separately from the others; allowing you
     to create the dictionary files even before you setup the
     other files. Thus, you have to setup your connection vars below.

=> run this script as "fsql_make.cgi <db_name> <table_name> <ds>"
=> the output file will OVERWRITE an existing file of the same name.
                        =========

=head1 METHODOLOGY NOTE

This program uses the output from the MySQL 'describe' command.
As such, it sets up a basic dictionary file, allowing one to edit
a table. However, it can't know the admin's intent for special fields
like popup fields. Thus, you almost assuredly will want to fine tune
the output. See the 'fsql_doc.txt' file for documentation about
things like setting up popup fields, pre and post processing, etc.

This is some sample output from 'describe', for the 'FutureForum' table
(an Open Source web discussion forum that I wrote.)

Database: forum  Table: forum  Rows: 45
+------------+------------------+------+-----+------------+----------------+
| Field      | Type             | Null | Key | Default    | Extra          |
+------------+------------------+------+-----+------------+----------------+
| message_no | int(10) unsigned |      | PRI | 0          | auto_increment |
| parent_no  | int(10) unsigned |      |     | 0          |                |
| branch_no  | text             |      |     |            |                |
| level      | int(10) unsigned |      | MUL | 0          |                |
| topic      | varchar(100)     |      | MUL |            |                |
| subject    | varchar(100)     |      | MUL |            |                |
| author     | varchar(100)     |      | MUL |            |                |
| email      | varchar(100)     |      |     |            |                |
| date       | date             |      | MUL | 0000-00-00 |                |
| time       | time             |      |     | 00:00:00   |                |
| seconds    | bigint(25)       |      | MUL | 0          |                |
| body       | text             |      |     |            |                |
| responses  | int(10) unsigned |      |     | 0          |                |
| send_email | char(1)          |      |     |            |                |
+------------+------------------+------+-----+------------+----------------+

=cut

##############################################################################

use DBI;
use locale;         # for case conversion for labels

$version = '2.1';

# the dictionary prefix and suffix should match in the futuresql_setup.pl file

$futuresql_dict_prefix = 'fsql_dict_';
$futuresql_dict_suffix = '.pl';

# this is where the dictionary file will be written
# this should match the setup file

# as it's written, we run it from the same directory from
# the other files (which could be in the web tree -- if so,
# I'd set the permissions to 700.) It's more secure, of course,
# to run it outside the web tree, still set to 700.

$setup_defaults_dir = '/typical_home_directory/futuresql/html/default_html_table';
$output_dir         = '.';

# color setup

$field_data_label_color                 = '#f3ecdc';
$view_list_table_start_data_label_color = '#f3ecdc';
$view_list_table_start_data_font_color  = 'darkblue';
$qbf_record_text_data_label_color       = '#f3ecdc';

$field_file         = "$setup_defaults_dir/blank_fields.html";

$qbf_table_start_file = "$setup_defaults_dir/custom.blank.qbf_table_start.html";
$qbf_record_text_file = "$setup_defaults_dir/custom.blank.qbf_record_text.html";
$qbf_table_end_file   = "$setup_defaults_dir/custom.blank.qbf_table_end.html";

$view_list_table_start_file = "$setup_defaults_dir/custom.blank.view_list_table_start.html";
$view_list_record_text_file = "$setup_defaults_dir/custom.blank.view_list_record_text.html";
$view_list_table_end_file   = "$setup_defaults_dir/custom.blank.view_list_table_end.html";

$output_field_file_suffix = '_fields.html';

$qbf_table_start_file_prefix = 'custom.';
$qbf_record_text_file_prefix = 'custom.';
$qbf_table_end_file_prefix   = 'custom.';

$qbf_table_start_file_suffix = '.qbf_table_start.html';
$qbf_record_text_file_suffix = '.qbf_record_text.html';
$qbf_table_end_file_suffix   = '.qbf_table_end.html';

$view_list_table_start_file_prefix = 'custom.';
$view_list_record_text_file_prefix = 'custom.';
$view_list_table_end_file_prefix   = 'custom.';

$view_list_table_start_file_suffix = '.view_list_table_start.html';
$view_list_record_text_file_suffix = '.view_list_record_text.html';
$view_list_table_end_file_suffix   = '.view_list_table_end.html';

# this should be usually set to '022' (resulting in a file set to '644')
# umask('022');   (depends on your system)

# $db_name is set at user prompt
$db_host           = 'localhost';

# you can store your user name and password in your cnf file

$cnf_file          = '~/.my.cnf';
$cnf_group         = 'client';

# or.. you can place the username and password in this file,
# but you should set this to 700 (rwx------) so that it's more secure.
# we assume here that you're using an 'admin' user name.

$w_user         = 'username';
$w_password     = 'password';

# set to 'cnf' or 'this_file' - the connection subroutine uses this
# to decide which method to use.

$password_location = 'cnf';

# this is used to set up the default size of textboxes
# use single quotes on the outside, and double on the inside

$textbox_coords = 'cols="50" rows="10"';
$max_html_size  = '30';  # this is 'size' var, not the maxlength var
                         # (fields will still scroll up to field_length)

if (-e "/dev/tty")
     {$clear = `clear`;}
else {$clear = `cls`;}

################################################################################
# END OF USER VARIABLES
################################################################################

# get input params here
# run this script as "futuresql_make.cgi <db_name> <table_name> <ds>"
#...............................................................................

if ($#ARGV == 0 and $ARGV[0] eq "-h")
	{
	&help_header;
	}
	
elsif ($#ARGV == 2)
	{
        $db_name    = $ARGV[0];
        $table_name = $ARGV[1];
        $ds         = $ARGV[2];
	}

else
	{
	&help_header;
	}

$dictionary_filename = $futuresql_dict_prefix . $ds . $futuresql_dict_suffix;
$dictionary_file     = $output_dir . "/" . $futuresql_dict_prefix . $ds . $futuresql_dict_suffix;

$status = 'does not exist yet.';
if ( -e $dictionary_file ){$status = 'EXISTS - do you want to overwrite it?';}

$| = 1;

# check for Unix or DOS, for console input

if (-e "/dev/tty")
     {$console = "/dev/tty";}
else {$console = "con";}

open(USER_PROMPT, "$console") || die print "Can't open $console.\n";

#..............................
$process = "false";
while ($process eq "false")
{

print $clear;

print qq~
FutureSQL Make Utility v$version
(creates dictionary files for FutureSQL)
         by Peter F. Brown; peterbrown\@worldcommunity.com
         Copyright 2001 Peter F. Brown. All Rights Reserved Worldwide.
         Open Source Software. [http://worldcommunity.com/opensource]

    You have specified:
         Database Name:     $db_name
            Table Name:     $table_name
Dictionary File String:     $ds
 (the part between the
    prefix and suffix)
     
        The file generated will OVERWRITE any existing file
        of the same name.

        [$dictionary_file] $status

        Do you wish to continue (enter only "y" or "n")? ~;

     $continue = <USER_PROMPT>;
     chop $continue;
     $continue = lc($continue);
        
        if ($continue eq "y")
             {
             $process = "true";
             }
        elsif ($continue eq "n")
             {
             $process = "false";
             close(USER_PROMPT);
	     print "\n\n ... Aborted make ...\n\n";
             exit;
             }
        else
             {
             $process = "false";
             }

        }

        close(USER_PROMPT);

print "\n\n ... processing ...\n\n";

#.....................................

&connect_to_db_in_make_file;

# get describe data
#.....................................

@describe = ();
%describe = ();

$sth = $dbh->prepare("describe $table_name")
                    or die print "Can't do select: $DBI::errstr\n";

$rv = $sth->execute or die print "Can't execute the query: $DBI::errstr\n";

while (
        (
         $field_, $type_, $null_,
         $key_, $default_, $extra_
        ) = $sth->fetchrow_array
      ) 
        {
        push (@describe, $field_);

        # check for values and set attributes

        $textarea = 'no';

        if    ( $type_ eq 'date' )      {$field_len = '10';}
        elsif ( $type_ eq 'datetime' )  {$field_len = '19';}
        elsif ( $type_ eq 'timestamp' ) {$field_len = '14';}
        elsif ( $type_ eq 'time' )      {$field_len = '10';}
        elsif ( $type_ eq 'year' )      {$field_len = '4';}

        elsif ( $type_ eq 'tinyblob' )  {$field_len = '255'; $textarea = 'yes';}
        elsif ( $type_ eq 'tinytext' )  {$field_len = '255'; $textarea = 'yes';}
        elsif ( $type_ eq 'blob' )      {$field_len = '65535'; $textarea = 'yes';}
        elsif ( $type_ eq 'text' )      {$field_len = '65535'; $textarea = 'yes';}
        elsif ( $type_ eq 'mediumblob' ){$field_len = '16777215'; $textarea = 'yes';}
        elsif ( $type_ eq 'mediumtext' ){$field_len = '16777215'; $textarea = 'yes';}
        elsif ( $type_ eq 'longblob' )  {$field_len = '4294967295'; $textarea = 'yes';}
        elsif ( $type_ eq 'longtext' )  {$field_len = '4294967295'; $textarea = 'yes';}

        elsif ( $type_ eq 'enum' )      {$field_len = '65535';}
        elsif ( $type_ eq 'set' )       {$field_len = '65535';}

        elsif ( $type_ =~ /float/ )
                {
                $field_len = $type_;
                $field_len =~ s/float\(//;
                $field_len =~ s/\,\d+\)//;
                }
        elsif ( $type_ =~ /\(/ )
                {
                $field_len = $type_;
                $field_len =~ s/\D+//g;
                }

        else  {$field_len = 1;}

        # double check
        if ( $field_len < 1 ){$field_len = 1;}

        $describe{$field_} = {
                             type      => $type_,
                             key       => $key_,
                             textarea  => $textarea,
                             field_len => $field_len,
                             extra     => $extra_
                             };
        }

#......................................................................

# parse through @describe and %describe and create output

$file_header = qq~# This is a dictionary setup file for FutureSQL
#############################################################################
# See the file 'fsql_setup.pl' for documentation.
#############################################################################
# field_setup
\sub field_setup
\{

~;

$file_footer = qq~
\}
#############################################################################

1;~;

##### NOW ASSEMBLE $output ########

$field_str = join ",\n                  ", @describe;

# key = 'PRI' ... what if 2? or 0? then use first field, and flag!
$key_field = $describe[0];

$output    = '';

##### here goes ######

$output .= qq~%dictionary = (

'table_name' => '$table_name',

'key_field'   => '$key_field',

'field_order' => '
                  $field_str
                 ',

'user_field_order' => '
                  $field_str
                 ',

'search_fields' => '
                  $field_str
                 ',

'fields'      => {
~;

#..............................................................

foreach $field_name ( @describe )
{
        $type      = $describe{$field_name}{type};
        $key       = $describe{$field_name}{key};
        $textarea  = $describe{$field_name}{textarea};
        $field_len = $describe{$field_name}{field_len};
        $extra     = $describe{$field_name}{extra};

        $label = $field_name;
        $label =~ s/\_/ /g;
        $label =~ s/(\w+)/\u\L$1/g;

        $html_type    = 'text';
        $html_size    = $field_len;
        $max_length   = $field_len;

        if ( $textarea eq 'yes' )
                {
                $html_type = 'textarea';
                $html_size = $textbox_coords;
                }
        else
                {
                if ( $html_size > $max_html_size ){$html_size = $max_html_size;}
                }

        if ( $type =~ /enum\(/i ){$html_type = 'popup_menu';}
        if ( $type =~ /set\(/i ) {$html_type = 'scrolling_list';}

        $display_only = 'no';
        if (
             $type  =~ /timestamp/ ||
             $extra =~ /auto_increment/
           )
            {$display_only = 'yes';}

        if ( $extra =~ /auto_increment/ )
                {
                $extra_insert_expression =                q~# we don't try to process
                                                          # a value, since the value
                                                          # is created after this, at
                                                          # insert time, since this
                                                          # is an auto-increment field~;
                }
        else
                {
                $extra_insert_expression =                qq~my (\$string) = \&join_array(' ');

                                                          return(\$string);~;
                }

$output .= qq~
                   '$field_name' => {
                                   'label'             => qq\~$label\~,
                                   'type'              => qq\~$type\~,
                                   'html_type'         => qq\~$html_type\~,
                                   'html_size'         => qq\~$html_size\~,
                                   'max_length'        => qq\~$max_length\~,
                                   'display_only'      => qq\~$display_only\~,
                                   'insert_expression' => \sub {
                                                          $extra_insert_expression
                                                          },

                                   'update_expression' => \sub {
                                                          my (\$string) = \&join_array(' ');

                                                          return(\$string);
                                                          },

                                   'default_value'     => \sub {
                                                          my (\$string) = '';

                                                          return(\$string);
                                                          },

                                   'display_process'   => \sub {
                                                          # my (\$field_name, \$field_value) = \@_;
                                                          # my \$return_value = \$field_value;
                                                          
                                                          # now do something with \$return_value
                                                          
                                                          # return(\$field_value, \$return_value);
                                                          },~;

        if ( $type =~ /enum\(/i )
                {
                # ENUM('value1','value2',...) popup_menu

                $value_str = $type;
                $value_str =~ s/enum\(//i;
                $value_str =~ s/\)//;
                $value_str = '(' . $value_str . ')';

$output .= qq~
                                   'field_setup'       =>  {
                                                           'values'   => sub {
                                                                         my \@array = $value_str;

                                                                         return(\\\@array);
                                                                         },
                                                           'default'  => sub {
                                                                         my (\$string) = \$field_value;

                                                                         return(\$string);
                                                                         },
                                                           'labels'   => sub {
                                                                         my (\$string) = '';

                                                                         return(\$string);
                                                                         },
                                                           'size'     => '',
                                                           'multiple' => '',
                                                           'linebreak'=> ''
                                                          },~;

                }
        elsif ( $type =~ /set\(/i )
                {
                # set('N','Y')

                $value_str = $type;
                $value_str =~ s/set\(//i;
                $value_str =~ s/\)//;
                $value_str = '(' . $value_str . ')';

$output .= qq~
                                   'field_setup'       =>  {
                                                           'values'   => sub {
                                                                         my \@array = $value_str;

                                                                         return(\\\@array);
                                                                         },
                                                           'default'  => sub {
                                                                         # array ref
                                                                         my (\$array_ref) = \&convert_to_array(' ');

                                                                         return(\$array_ref);
                                                                         },
                                                           'labels'   => sub {
                                                                         my (\$string) = '';

                                                                         return(\$string);
                                                                         },
                                                           'size'     => '4',
                                                           'multiple' => 'true',
                                                           'linebreak'=> ''
                                                          },~;

                }
        else
                {

$output .= qq~
                                   'field_setup'       =>  {
                                                           'values'   => sub {
                                                                         my (\$string) = '';

                                                                         return(\$string);
                                                                         },
                                                           'default'  => sub {
                                                                         my (\$string) = \$field_value;

                                                                         return(\$string);
                                                                         },
                                                           'labels'   => sub {
                                                                         my (\$string) = '';

                                                                         return(\$string);
                                                                         },
                                                           'size'     => '',
                                                           'multiple' => '',
                                                           'linebreak'=> ''
                                                          },~;
                 
                 }


$output .= qq~
                                   },
~;

}

$output .= qq~
                 },

          );
~;

# write to file
#.......................................................................

# open output file
unless ( open (OUT_FILE, ">$dictionary_file"))
    {
    die print "Error opening $dictionary_file for writing.\n";
    }

print OUT_FILE "$file_header";
print OUT_FILE "$output";
print OUT_FILE "$file_footer";

close OUT_FILE;

# now create default html templates
#.......................................................................

$field_file_text            = &include_file($field_file);
$view_list_table_start_text = &include_file($view_list_table_start_file);
$view_list_record_text_text = &include_file($view_list_record_text_file);
$qbf_record_text_text       = &include_file($qbf_record_text_file);

$field_data                 = '';
$view_list_table_start_data = '';
$view_list_record_text_data = '';
$qbf_record_text_data       = '';

foreach $field_name ( @describe )
        {
        $label = $field_name;
        $label =~ s/\_/ /g;
        $label =~ s/(\w+)/\u\L$1/g;

        #.......................................................................
        $field_data .= qq~
        <tr> 
        <td bgcolor="$field_data_label_color" align="right" valign="middle" colspan="2">
        <font face="Arial, Helvetica, sans-serif" size="-1"><b>
        $label\:
        </b></font>
        </td>
        <td valign="middle" colspan="2">
        :!:$field_name\:!:
        </td>
        </tr>
        ~;        
        #.......................................................................
        $view_list_table_start_data .= qq~
        <td bgcolor="$view_list_table_start_data_label_color">
        <font size="-1" face="Times New Roman,Georgia,Times" color="$view_list_table_start_data_font_color"><b>
        $label
        </b></font> 
        </td>
        ~;
        #.......................................................................
        $view_list_record_text_data .= qq~
        <td>
        <font size=-1 face="Times New Roman,Georgia,Times">
        :!:$field_name:!:
        </font>
        </td>
        ~;
        #.......................................................................
        $qbf_record_text_data .= qq~
        <tr> 
        <td bgcolor="$qbf_record_text_data_label_color" align="right" valign="top">
        <b><font face="Arial, Helvetica, sans-serif" size="2">
        $label\:
        </font></b>
        </td>
        <td>
        <font face="Arial, Helvetica, sans-serif" size="2">
        :!:$field_name:!:
        </font>
        </td>
        </tr>
        ~;
        #.......................................................................

        } # end of field loop

$field_file_text            =~ s/:!:data_row_tag_for_default_install:!:/$field_data/;
$view_list_table_start_text =~ s/:!:data_row_tag_for_default_install:!:/$view_list_table_start_data/;
$view_list_record_text_text =~ s/:!:data_row_tag_for_default_install:!:/$view_list_record_text_data/;
$qbf_record_text_text       =~ s/:!:data_row_tag_for_default_install:!:/$qbf_record_text_data/;

# print field_file_text
#..................................

$output_field_file = $output_dir . '/' . $table_name . $output_field_file_suffix;

unless ( open (OUT_FILE, ">$output_field_file"))
       {die print "Error opening $output_field_file for writing.\n";}

print OUT_FILE "$field_file_text";
close OUT_FILE;

# print qbf text
#...................................

$output_qbf_record_text_file = $output_dir . '/' .
                        $qbf_record_text_file_prefix .
                        $table_name .
                        $qbf_record_text_file_suffix;

unless ( open (OUT_FILE, ">$output_qbf_record_text_file"))
       {die print "Error opening $output_qbf_record_text_file for writing.\n";}

print OUT_FILE "$qbf_record_text_text";
close OUT_FILE;

# print view_list_table_start text
#...................................

$output_view_list_table_start_text_file = $output_dir . '/' .
                        $view_list_table_start_file_prefix .
                        $table_name .
                        $view_list_table_start_file_suffix;

unless ( open (OUT_FILE, ">$output_view_list_table_start_text_file"))
       {die print "Error opening $output_view_list_table_start_text_file for writing.\n";}

print OUT_FILE "$view_list_table_start_text";
close OUT_FILE;

# print view_list_record_text text
#...................................
           
$output_view_list_record_text_file = $output_dir . '/' .
                        $view_list_record_text_file_prefix .
                        $table_name .
                        $view_list_record_text_file_suffix;

unless ( open (OUT_FILE, ">$output_view_list_record_text_file"))
       {die print "Error opening $output_view_list_record_text_file for writing.\n";}

print OUT_FILE "$view_list_record_text_text";
close OUT_FILE;

## now copy standard files

$qbf_table_start_file_text     = &include_file($qbf_table_start_file);
$qbf_table_end_file_text       = &include_file($qbf_table_end_file);
$view_list_table_end_file_text = &include_file($view_list_table_end_file);

# write qbf_table_start_file
#...................................
           
$output_qbf_table_start_file = $output_dir . '/' .
                        $qbf_table_start_file_prefix .
                        $table_name .
                        $qbf_table_start_file_suffix;

unless ( open (OUT_FILE, ">$output_qbf_table_start_file"))
       {die print "Error opening $output_qbf_table_start_file for writing.\n";}

print OUT_FILE "$qbf_table_start_file_text";
close OUT_FILE;

# write qbf_table_end_file
#...................................
           
$output_qbf_table_end_file = $output_dir . '/' .
                        $qbf_table_end_file_prefix .
                        $table_name .
                        $qbf_table_end_file_suffix;

unless ( open (OUT_FILE, ">$output_qbf_table_end_file"))
       {die print "Error opening $output_qbf_table_end_file for writing.\n";}

print OUT_FILE "$qbf_table_end_file_text";
close OUT_FILE;

# print view_list_table_end text
#...................................

$output_view_list_table_end_text_file = $output_dir . '/' .
                        $view_list_table_end_file_prefix .
                        $table_name .
                        $view_list_table_end_file_suffix;

unless ( open (OUT_FILE, ">$output_view_list_table_end_text_file"))
       {die print "Error opening $output_view_list_table_end_text_file for writing.\n";}

print OUT_FILE "$view_list_table_end_file_text";
close OUT_FILE;

# end of html creation
# ...................................................................................

print qq~

   Dictionary file $dictionary_file and html files
   successfuly created!

   Please check them, and fine-tune them, if necessary.

~;

exit;

##################################################################
# connect_to_db_in_make_file
sub connect_to_db_in_make_file
{

if ( $password_location eq 'this_file' )
{
$dbh = DBI->connect("DBI:mysql:$db_name:$db_host", $w_user, $w_password)
        || die print qq~
        You were unable to connect to the database.
        Please check your setup.
        $DBI::errstr
         ~;
}

elsif ( $password_location eq 'cnf' )

{

$dbh = DBI->connect("DBI:mysql:$db_name:$db_host"
                     . ";mysql_read_default_file=$cnf_file"
                     . ";mysql_read_default_group=$cnf_group",
                     $user, $password)
        || die print qq~
        You were unable to connect to the database.
        Please check your setup.
        $DBI::errstr
         ~;

}

else
{

print qq~
You were unable to connect to the database.
Please check your setup.
~;
exit;

}

}
##################################################################
# help_header
sub help_header
{

print $clear;

print qq~
FutureSQL Make Utility v$version
(creates dictionary files for FutureSQL)
         by Peter F. Brown; peterbrown\@worldcommunity.com
         Copyright 2001 Peter F. Brown. All Rights Reserved Worldwide.
         Open Source Software. [http://worldcommunity.com/opensource]

Syntax: "fsql_make.cgi <db_name> <table_name> <ds> [-h]"
        - <db_name> is the name of the MySQL database
        - <table_name> is the name of the MySQL table
        - <ds> is the phrase in between the dictionary
          file prefix and suffix phrases. This usually is NOT
          the name of the table; allowing you to have multiple
          dictionary files for each table (e.g: 't.1', 't.2', etc.)
        - [-h] shows you this help header.

        The file generated OVERWRITES an existing file.
        Be careful, and make sure to check the generated file
        BEFORE you use it. Make sure the KEY_FIELD IS CORRECT!

~;

exit;

}

###########################################################################
# include_file
sub include_file
{

my($i_file) = @_;
my($filesize);
my($i_text);

#obtain size of file
$filesize = -s $i_file;

# open file
unless ( open(INCLUDE_FILE, $i_file ))
{
    die print
    qq~Error!<p>
    Can't open File $i_file.~;
}

# read whole file into variable
read (INCLUDE_FILE, $i_text, $filesize);

close(INCLUDE_FILE);

return($i_text);
    
}
###################################################################

