#!/usr/bin/perl
#    Last change: PFB 2001-10-07 19:55:34
#############################################################################
# POD Documentation

=head1 SETUP FILE

fsql_setup.pl - version 2.2
Build Date: October 7, 2001

VERSION NOTE:
  I've released this version without too much additional documentation,
  simply because many, many people wanted it.
  Version 2 has been tested in SEVERAL production environmnents,
  quite extensively, and works well.

(See Copyright & Author Info in 'fsql_main.cgi')

This is the MAIN SETUP FILE FOR fsql_main.cgi
(not counting the site specific info in 'fsql_site_db.pl')

! Read the file Readme.1st, please, and
  see Instructions and Notes in the documentation file: 'fsql_doc.txt'
  (which admittedly are awfully thin (anybody want to volunteer?) :-)

    - 'futuresql.cgi' is run, and
    - then loads       'fsql_main.cgi'
    - which then loads 'fsql_site_db.pl'
    - which then loads 'fsql_setup.pl' (this file).

    - other files below are loaded from calls in 'fsql_main.cgi'

!!! Note: based on the script parameters 's_' and 'dc_'
    additional files are required: e.g:
    (assuming $s_ and $dc_ = 'f.1' ('f' might be the first letter of the table))
    (if $s_ is set to 'default' then you can use one spec file for all tables)
    (the default specvar file 'phrase' is set below in this file)

    - 'fsql_dict_f.1.pl'     - (you ALWAYS need a unique data dictionary file)
    - 'fsql_specvars_f.1.pl' - (or 'fsql_specvars_default.pl')
    - 'fsql_spec_f.1.pl'     - (or 'fsql_spec_default.pl')

... and more files are required:

    - 'fsql_dictionary_vars.pl'
    - 'fsql_action_vars.pl'
    - 'fsql_user_subs.pl'

       note the order of loading these files, in the main script
      ...............................................................................
       1 - dictionary vars, to load arrays and hashes for all dictionary files
       2 - dictionary,      to get $table_name and $key_field, etc (used in spec file)

       3 - action vars,     variables used in more than one table, but note that
                            they need the values from the dictionary files,
                            thus, we load the action file AFTER the dictionary files,
                            and PRIOR to the specvar and action spec files

       4 - user subs file,  user subroutines

       5 - specvars,        to get special vars and TABLE specific customized
                            action spec vars (loaded AFTER the action file and
                            BEFORE the 'spec' file)

       6 - spec,            to manage user interface and actions
      ...............................................................................

... with many additional html template files

=cut

####################################################################################
# This file is used for GLOBAL variable setup.
# It contains variables and subroutines that can be MODIFIED by the administrator.
#
# => For user / site / db specific variables - use 'fsql_site_db.pl'
# => For user subroutines,                     use 'fsql_user_subs.pl'
####################################################################################

use POSIX qw(strftime);
use Time::Local;                  # place special library calls here

srand;                            # Used for session id routine
                                  # sometimes used for password generation
                                  # in subroutines.

$global_todays_date             = strftime("%A, %B %e, %Y - %l:%M%p %Z", localtime);
                                  # this is a handy variable to initialize for date display
                                  # it requires the POSIX and Time::Local calls above
                                  # I use this because I got tired of messing with date
                                  # routines the sample variable below is displayed as:
                                  # "Monday, December 18, 2000 - 7:00AM EST" (for example)
                                  # see strftime notes at end of file

$view_with_html_escape          = 'no';
                                  # => Set to 'no' if you want your data to show the
                                  #    RESULTS of html code
                                  # => Set to 'yes' if you want to see the RAW html code.
                                  # if 'yes', then html code in fields will be escaped
                                  # when displayed in viewing subroutines.
                                  # (Fields are always html_escaped in the edit routine.)
                                  # => if you don't want to make this global, you can use
                                  #    the 'display_process' dictionary key to control
                                  #    display (see the subroutine 'get_display_field_value'
                                  #    in the main file, for notes about this.)

$default_display_method         = '_external_template_';
# $default_display_method         = '_standard_';
# $default_display_method         = '_custom_page_';

                                  # this variable determines the display
                                  # method for data.

                                  # '_standard_' uses an internal method
                                  # of display that's hard to modify

                                  # '_custom_page_' may be deprecated in
                                  # the future, and is kind of in-between
                                  # '_standard_' and the BEST OPTION:

                                  # 'external_template_', which uses
                                  # external html templates that are
                                  # TOTALLY customizable.

                                  # If you want to be up and running quickly,
                                  # use '_standard_' first.

                                  # DEFAULT DISPLAY is set here, but
                                  # can be overridden in the 'spec' files

# color vars
# ...............................

$top_menu_left_bg_color         = '#6699cc';
$top_menu_right_bg_color        = '#e6e6e6';
$top_menu_backup_bg_color       = '#990066';
$top_menu_backup_fg_color       = 'white';
                                  # colors for the admin menu

$header_lbl_bg                  = 'darkblue';
$header_fld_bg                  = 'white';
$lbl_bg                         = '#f3ecdc';
$fld_bg                         = '';
$td_bg                          = '#f3ecdc';
                                  # colors for non-html output
                                  # header_lbl_bg, header_fld_bg, lbl_bg, fld_bg, td_bg
                                  # are used for anything other than html template output

$button_cell_width              = 30;
                                  # button_cell_width is for the <td> cells with menu buttons

$wrap                           = 'virtual';
                                  # use 'off', 'virtual', or 'physical'
                                  # used for textboxes

$admin_access                   = 'admin';
                                  # (usually set to 'admin')

                                  # 'admin' is the default value for testing
                                  # for 'administrative' access
                                  # the script looks for 'admin',
                                  # 'intermediate' and 'public'
                                  # in some cases (such as this demo script)
                                  # one needs to open up all admin functions,
                                  # thus we set this to 'public' here
                                  # and in the spec files

# secondary setup files
# ...............................

$futuresql_dictionary_vars_file = "$preferences_dir/fsql_dictionary_vars.pl";
$futuresql_action_vars_file     = "$preferences_dir/fsql_action_vars.pl";
$futuresql_user_subs_file       = "$preferences_dir/fsql_user_subs.pl";

$futuresql_dict_prefix          = "$preferences_dir/fsql_dict_";
$futuresql_specvars_prefix      = "$preferences_dir/fsql_specvars_";
$futuresql_spec_prefix          = "$preferences_dir/fsql_spec_";

$futuresql_specfile_suffix      = '.pl';

$futuresql_specvars_default     = 'default';
                                  # default name of specvars 'atom',
                                  # if regular one doesn't exist,
                                  # e.g: 'fsql_specvars_default.pl'

# template file variables
# ...............................

$error_file                     = "$template_dir/error.html";
$message_file                   = "$template_dir/message.html";

$password_mail_file             = "$preferences_dir/password_mail.txt";
$claim_record_mail_file         = "$preferences_dir/claim_record_mail.txt";
$add_record_mail_file           = "$preferences_dir/add_record_mail.txt";
$user_pass_mail_file            = "$preferences_dir/user_pass_mail.txt";

                                  # these variables are used for the 'user'
                                  # routines (see the doc file)

                                  # Note about the $rt template variable:
                                  # ........................................
                                  # the incoming template parameter '$rt_' must be
                                  # placed in the initial login form, but you can
                                  # declare other template variables
                                  # here, and then replace the value of $rt_ anywhere
                                  # in your spec setup file. For example, you could
                                  # declare a variable like
                                  # $view_record_template = 'file.htm' and then put
                                  # $view_record_template in the hidden field section
                                  # of the view_admin_list action hash element
                                  # If you do this, you may also want to reset '$rt_'
                                  # to the original value in one of the 'pre_op' sections,
                                  # so that when you go back to the top_menu, it will pull
                                  # up the right template.
                                  #........................................................

$zero_records_text              = qq~
                                    <b>Zero Records Found</b><br>
                                    (using the count function)<br>
                                    with the selection parameters of:
                                    <p>
                                    :!:select_count:!:
                                    <p>
                                    Please hit BACK and try again.
                                    <p>
                                    :!:zero_records_custom_message:!:
                                    ~;

                                  # zero_records_text is used
                                  # when a selection query results in an empty set
                                  # The tag :!:select_count:!: (if you have it in
                                  # the variable below) is replaced with the SQL
                                  # selection query text.
                                  # Note that you may also add your own tags here
                                  # as long as you define them and parse this
                                  # variable in a pre_op that uses &get_records.
                                  # I've also created a second default tag;
                                  # :!:zero_records_custom_message:!:
                                  # that the code looks for and parses -- thus
                                  # you can define $zero_records_custom_message
                                  # in a get_records pre_op.
                                  # - a get_records pre_op is a spec key like
                                  # 'view_record' or 'view_custom_record', etc.
                                    
$use_field_defaults_for_qbf     = 'no';
                                  # use_field_defaults_for_qbf will place the default
                                  # values in the qbf fields if this var is set to 'yes'.
                                  # this can be disconcerting: if one doesn't notice that
                                  # a field has a value and then hits submit, a search
                                  # might fail.
                                  # set this to 'no' if you don't want to use it.
                                  
@skip_tables                    = qw[];
                                  # array of table names that should be skipped in
                                  # &show_database_structure, which shows the
                                  # structure of the CURRENT DATABASE.
                                  # Leave it blank if you don't want to use it., i.e:
                                  # qw[]; (then ALL TABLES will be displayed)

#....................................
# default vars for returning records

$max_rows                       = 200; # used if single page output is selected
$max_rows_per_page              = 100; # used if $r_ is missing or > than this
$default_rows_per_page          = 30;  # used as default for $r_

# TEMPLATE TAGS
#....................................
                                  # These notes connect with the %template_tags hash below.
                                  # These tags are replaced in the template files,
                                  # with the real values.

$text_tag                       = ':!:body_text:!:';

                                  #!!! we keep text_tag out of the hash, because it's
                                  #    so critical that it works
                                  #!!! this is the main tag for the entire program
                                  #->> you can change the value of ':!:body_text:!:', but
                                  #->> DON'T change the name of the variable '$text_tag'.
                                  #    (text_tag is replaced with $body_all)

                                  # Template tags are used so that you can have an unlimited
                                  # and flexible method of replacing tags in the template
                                  # that is actually OUTSIDE of the database functionality;
                                  # for example, you might want to have banner ads
                                  # dynamically generated via a subroutine, or you
                                  # might wish to create the <title> tag based on the
                                  # name of the database, etc.

                                  # 1. each key (e.g: 'special_tag') is parsed and two
                                  #    variables are created and used. The first is
                                  #   'tag_code_in_template'

                                  # 2. 'tag_value_variable_name' is the name of the
                                  #    $variable that YOU must initialize, in either this
                                  #    file, or in one of the 'pre_process' sections of a
                                  #    spec file

                                  # 3. 'tag_process_status' is mainly for those who wish
                                  #    to set up a hash of variables, and then turn one of
                                  #    them off, for some reason. If the value is set to
                                  #   'active', they'll be processed.
                                  
                                  # the actual command that parses these tags in the
                                  # templates, is:
                                  # $template_text =~ s/$tag_code_in_template/$$tag_value_variable_name/g;
                                  # EXAMPLE: :!:special_tag:!: is replaced with $special_text

                                  # Remember: You must place the actual tag(s) in your
                                  #           template file, and you must initialize the
                                  #           value of the replacing variable in either
                                  #           this file, or one of the spec files, usually
                                  #           in the 'pre_process' section.

%template_tags = (
                 'custom_list_header' => {
                                'tag_code_in_template'    => ':!:custom_list_header:!:',
                                'tag_value_variable_name' => 'custom_list_header',
                                'tag_process_status'      => 'active',
                               },

                 'special_tag' => {
                                'tag_code_in_template'    => ':!:special_tag:!:',
                                'tag_value_variable_name' => 'special_text',
                                'tag_process_status'      => 'not_active',
                               },
                 'ad_graphic' => {
                                'tag_code_in_template'    => ':!:ad_graphic:!:',
                                'tag_value_variable_name' => 'ad_graphic_text',
                                'tag_process_status'      => 'not_active',
                               },
                 );

                                  # CUSTOM_LIST_HEADER is a great example of this flexible system.
                                  # By putting the tag in the table start area, and then
                                  # calling the function with a hidden field, like this:
                                  # <input type="hidden" name="custom_list_header" value="Report">
                                  # you can easily add various custom tags without actually
                                  # modifying the main code.
                                  # HOWEVER, since this was such a great tag, I put the variable
                                  # init call inside &get_records:view_custom_list, so that
                                  # you wouldn't have to put a call to param() in all the
                                  # view_custom_list:pre_op sections of your setup files.
                                  # I left the tag HERE, as an example - Note however, that it's
                                  # a WORKING example. :-)

$special_text                   = qq~
                                  <a href="http://worldcommunity.com">
                                  <img src="http://worldcommunity.com/pfbsmall.gif" border="0" alt="photo of author"></a>
                                  <br><font size=-1><i>
                                  (an example of special tag processing,<br>
                                  used by FutureSQL, leading to the author's<br>
                                  Perl / MySQL programming site.)
                                  <p></i></font>~;

                                  # $special_text is, of course, an example :-)
                                  # template tag value variables can get their values from
                                  # subroutines, or just straight, like this example.

foreach my $new_tag ( @extra_template_tags )
        {
        $template_tags{$new_tag} = {
                                    'tag_code_in_template'    => ":!:$new_tag:!:",
                                    'tag_value_variable_name' => "$new_tag",
                                    'tag_process_status'      => "active",
                                   };
        }

                                  # @extra_template_tags is a quicker method than the above.
                                  # By adding a tag to that array, the tag is automatically
                                  # added to the hash above. (The array is in
                                  # fsql_site_db.pl because it often contains table
                                  # specific items.)

                                  # end of template tags section
                                  #........................................................

$email_expr                     = qq~
                                  if(email != '',
                                  concat('<a href="mailto:', email, '">Send Email</a>'),
                                  '') as email_url
                                  ~;

$email_gif_expr                 = qq~
                                  if(email != '',
                                  concat('<a href="mailto:', email, '"><img src="$image_url/email.gif" border="0" alt="Send Email"></a><br>'),
                                  '')
                                  as email_gif_url
                                  ~;

                                  # sample expressions for field and record processing
                                  # use in the spec file, if you wish

$base_url_search                = qq~
                                  <img src="$image_url/blue.gif">
                                  <a href="$script_name\?s_=$s_\&dc_=$dc_\&r_=$r_\&p_=$p_\&rt_=$rt_\&user_level=public&w_=$w_\&a_=view_custom_list
                                  ~;

                                  # base for generic custom searches (url style)
                                  # use this in the spec file, under 'query_by_form_custom'

                                  # (NOTE: since v2.0, this has been deprecated. It's much
                                  # easier to code a search in the html templates. See one
                                  # of the 'qbf' template files, e.g: 'custom.books.qbf_record_text.html')

                                  # you can also put this in the top of the spec file
                                  # if you want to customize any variables for each table
                                  # (like 'user_level')
                                  # if you place it in the spec file, then that value will
                                  # overwrite this value - thus, you can leave this value as
                                  # the default, for tables that don't need customization
                                  # e.g: make this user_level equal to 'public', and make it
                                  # 'admin' for an admin type table.
                                  
                                  # then... add on this (for example):
                                  # like this:  $base_url_search\&wh_=body%20like%20'%25e%25'&or_=author,subject">Messages Containing the Letter 'e' :-)</a>

# mysqldump variables
# ..........................

$mysqlbin                       = '/usr/bin';
$mysqlshow                      = "$mysqlbin/mysqlshow";

$mysql_dump_file_ext            = 'txt';
$mysqldump_params               = '--quick --add-drop-table -c -l';
                                  # I used '-v' (for verbose on this, and it
                                  # crashed. still checking this...)

$backup_type                    = 'mysqldump';
# $backup_type                    = 'outfile';
# $backup_type                    = 'normal_select';

                                  # set $backup_type to 'mysqldump',
                                  # 'outfile', or 'normal_select'
                                  # 'normal_select' uses a normal
                                  # select/write process; it's clunky,
                                  # but some hosts don't provide access to
                                  # mysqldump or 'select into outfile'
                                  # (sometimes mysqldump is on a different
                                  # server, and sometimes a user doesn't have
                                  # 'file_privileges' for mysql.)

                                  # (username and password are added to the
                                  # params in the main script)
                                  # ! note: you must have MySQL file priv
                                  # status, or the file will not be written
                                  # (it will be 0 bytes long)

                                  # NOTE: for LARGE data sets, 'normal_select'
                                  # may not work well, because of either
                                  # a) memory problems, or b) browser
                                  # timeouts. this is something I want to
                                  # work on.

$backup_field_terminate         = '|';
$backup_field_enclosed_by       = '';
$backup_line_terminate          = "\n";
                                  # params for 'normal_select' file writing
                                  # note that the "\n" must be interpolated
                                  # via " double quotes or the qq method

$outfile_params                 = qq~ fields terminated by '|' lines terminated by '\n' ~;
                                  # params for 'select * from $table_name
                                  # into $outfile ($outfile is created in
                                  # the backup routine)

$chmod_backup_file              = 'no'; # set to 'yes' if you want to use it
                                  # NOTE: chmoding the backup file to 600 is
                                  # only useful if one is using cgiwrap, or
                                  # the site is chrooted as the user normally,
                                  # the file is saved as 'nobody' (you DO NOT
                                  # want to set the backup file to 600 as
                                  # 'nobody' or you, as a normal user, won't
                                  # be able to ftp the file)


#############################################################################
# GLOBAL BUT MODIFIABLE SUBROUTINE SECTION
#############################################################################
# These subroutines are separated from the main file because they can be
# modified without necessarily breaking the main core code.
# I recommend that you add user subroutines to the 'fsql_user_subs.pl' file.

# The next three routines are critical, and are used by the program.
# Be careful when editing them. The routines are:

# - connect_to_db       - (necessary to edit)
# - check_authorization - (not necessary to edit unless you want to)
# - sub_function        - (edit if you add a function)
#############################################################################
# connect_to_db
sub connect_to_db
{

# there are a LOT of ways to do this, and everyone's security needs
# are different. This is just one way:

# This method is based on a number of concepts:

# 1. The login form will use a variable called 'user_level',
#    set to 'admin', 'intermediate' or 'public' - causing different
#    things to happen here and in the script.

#    Note that the EXTRA Admin functions on the Home menu will only
#    be displayed:
#    if ( $w_user_type eq 'admin' and $user_level eq 'admin' )
#    (Of course, all of this can be modified freely.)

# 2. There will be a 'user' table (not the mysql.user table) but
#    rather a custom user table containing authorization details.
#    In this demo, it's contained in the 'books.readers' table.

# 3. The mysql.users and mysql.db tables are initialized with the
#    corresponding user names, passwords and edit levels.
#    (Some servers don't allow this; they only allow one user, and
#    give that user super user permission. This really stinks and
#    is bad for security. You need at least 3 different user levels.)
#    i.e: 'admin' (all permissions); 'user' (select, insert, update, delete);
#    and 'public' (only select permission)

my($q) = @_;

$w_user       = param('w_user');
$user_level   = param('user_level');

# we check both the password and session id
# the password will be retreived from the first login

$w_password   = param('w_password');
$w_session_id = param('w_si');

$create_session_id = 'no';

# only use session ids if user_level is NOT public

if ( $user_level ne 'public' )
      {
      if ( $w_session_id )
            {
            $w_password = &decrypt_session_id($w_session_id);
            }
      else
            {
            # must create session id (after logging in)
            $create_session_id = 'yes';
            }
      }

#.........................................................................

if (
    $user_level ne 'public' and
    ($w_user    !~ /\w+/     or $w_password !~ /\w+/)
   )
        {
        &error_message(qq~Error!<p>You must type a user name and
                       password. Please hit BACK and try again.~);
        }

if ( $w_user eq $admin_user )
        {
        # we assume that the admin user should NOT use a .cnf file, but should
        # be forced to login (as super user)
        # This allows a log in, even if this user is NOT in an additional
        # users table.

        $dbh = DBI->connect("DBI:mysql:$db_main:$db_host:$db_port", $w_user, $w_password)
                || die
                &error_message(qq~Error!<p>
                You were unable to connect to the database<br>
                It may be that you typed in your username or password
                incorrectly. Please contact
                <a href="mailto:$admin_email">$admin_email</a>
                $DBI::errstr~);

        $w_user_type = $admin_access;
        $w_first     = $admin_user;
        $w_last      = '';
        $w_status    = 'active';
        $w_u_id      = '0';

        }

#.................................

elsif ( $w_user ne $admin_user and $user_level eq $admin_access )
        {
        # we assume that logins from a form with user_level
        # set to $admin_access (that are not $admin_user ) will not log in
        # to the MySQL DB but will use a .cnf file.
        # However, we KEEP $w_user and $w_password,
        # to check edit levels, in the application user table

        $cnf_group = $cnf_group_admin; # select, insert, update, delete

        $dbh = DBI->connect("DBI:mysql:$db_main:$db_host:$db_port"
                             . ";mysql_read_default_file=$cnf_file"
                             . ";mysql_read_default_group=$cnf_group",
                             $user, $password)
                             || die
                             &error_message(qq~Error!<p>
                             ... connecting to the Database.<br>
                             Please contact <a href="mailto:$admin_email">
                             $admin_email</a>
                             $DBI::errstr
                             ~);

        # get more info
        # user_type, first, last, status

        ( $w_user_type, $w_first, $w_last, $w_status, $w_u_id ) =
          $dbh->selectrow_array("select user_type,
                                 first_name,last_name,
                                 status,r_id
                                 from readers where
                                 user = '$w_user' and
                                 password = '$w_password'");

        if ( $w_user_type ne $admin_access ){&error_message(qq~Error!<p>
                       You're User Type is not set to Admin.
                       You therefore can't
                       log in via the Admin Form. Please log in
                       with the User Log in Form.<p>~);}

        if ( $w_status ne 'active' ){&error_message(qq~Error!<p>
                       You either typed the wrong user name and password
                       or your account is no longer active.<p>~);}

        }

#.................................

elsif ( $user_level eq 'intermediate' )

        {
        # We assume that logins from a form with user_level
        # set to 'intermediate' will not log in
        # to the MySQL DB but will use a .cnf file.
        # We assume that these users are HAVE LIMITED PERMISSIONS
        # We KEEP $w_user and $w_password,
        # to check edit levels, in the readers table

        $cnf_group = $cnf_group_admin;  # select, insert, update, delete

        $dbh = DBI->connect("DBI:mysql:$db_main:$db_host:$db_port"
                             . ";mysql_read_default_file=$cnf_file"
                             . ";mysql_read_default_group=$cnf_group",
                             $user, $password)
                             || die
                             &error_message(qq~Error!<p>
                             ... connecting to the Database.<br>
                             Please contact <a href="mailto:$admin_email">
                             $admin_email</a>
                             $DBI::errstr
                             ~);

        # get more info
        # user_type, first, last, status

        ( $w_user_type, $w_first, $w_last, $w_status, $w_u_id ) =
          $dbh->selectrow_array("select user_type,
                                 first_name,last_name,
                                 status,r_id
                                 from readers where
                                 user = '$w_user' and
                                 password = '$w_password'");

        if ( $w_status ne 'active' ){&error_message(qq~Error!<p>
                       You either typed the wrong user name and password
                       or your account is no longer active.<p>~);}

        }

#..................................

elsif ( $user_level eq 'public' )
        {
        # we assume that public users will not log in, but use a
        # .cnf file, and we set $w_user and $w_password to something
        # simple, to get beyond the script requirements that these
        # vars must be set.

        # Note that 'books_public' is set to allow 'select, insert,
        # and update' in the 'mysql.db' table because we have a
        # routine in this demo that allows one to register as a new
        # user. We would normally have another 'public' user for
        # searches, etc, for security.

        $cnf_group  = $cnf_group;
        $w_user     = 'public';
        $w_password = 'hidden';

        $dbh = DBI->connect("DBI:mysql:$db_main:$db_host:$db_port"
                             . ";mysql_read_default_file=$cnf_file"
                             . ";mysql_read_default_group=$cnf_group",
                             $user, $password)
                             || die
                             &error_message(qq~Error!<p>
                             ... connecting to the Database.<br>
                             Please contact <a href="mailto:$admin_email">
                             $admin_email</a>
                             $DBI::errstr
                             ~);
        }

#..................................

else
        {
        &error_message(qq~Error!<p>
                        ... connecting to the Database.<br>
                        Please contact <a href="mailto:$admin_email">
                        $admin_email</a>
                        ~);
        }

#..................................

# passing the PASSWORD is not the best way... using a session id is
# better. note that the mysql_backup routine that uses mysqldump with
# a password would have to be rewritten to grab the password from the
# session id. also note that the w_user and user_level vars can and
# should be passed for various script options.
# At a minimum, the PASSWORD should be muffled.

# <input type="hidden" name="w_password" value="$w_password">
# <input type="hidden" name="w_s" value="$w_session_id">

if ( $create_session_id eq 'yes' )
      {
      # create session_id and file from above password
      $w_session_id = &create_session_id($w_password, $w_u_id);
      }

$security_fields = qq~
<input type="hidden" name="w_user"     value="$w_user">
<input type="hidden" name="w_si"       value="$w_session_id">
<input type="hidden" name="user_level" value="$user_level">
~;

$dbh->{PrintError} = 1;
$dbh->{RaiseError} = 1;

}
########################################################################
# create_session_id
sub create_session_id
{

# $w_session_id = &session_id($w_password, $w_u_id);

my ($w_password, $w_u_id) = @_;
my $w_session_id;
my $session_file_str, $session_file;
my $session_file_write;

# init vars
#.............................................

my @xor_array,@random_array;
my $len = length($w_password);
my $xor_password_string, $xor_password_key;
my $random_num, $ra, $sa;
my $count = 0;

undef(@xor_array);
undef(@random_array);

# create $xor_password_string and $xor_password_key
#......................................................................

while ( $count < $len )
        {
        $random_num = int(rand(9999999999));
        $ra = $random_array[$count] = $random_num;
        $sa = ord(substr($w_password,$count,1));
        $xor_array[$count] = int($sa) ^ int($ra);
        $count++;
        }

$xor_password_string = join(".", @xor_array);
$xor_password_key    = join(".", @random_array);

# first delete old files, this way, the script is reasonably self cleaning

&delete_old_files($session_dir, $session_file_suffix, $session_seconds_to_save);

#...........................................................................
# we use sysopen with O_EXCL to avoid overwriting other person's file
# we use a mask of 0600 to create a file only readable by apache,
# and not a normal shell user, as extra security

$session_file_write = 'no';

use Fcntl qw/O_WRONLY O_CREAT O_EXCL/;

while ( $session_file_write eq 'no' )
      {
      # loop until we find a file name that's unique

      $session_file_str = $w_u_id . '.' . time . '.' . int(rand(999999999999999));
      $session_file     = $session_dir . '/' . $session_file_str . $session_file_suffix;

      $rv = sysopen(FH, $session_file, O_WRONLY | O_EXCL | O_CREAT, 0600);
      if ( $rv )
            {
            $session_file_write = 'yes';
            }
      }

print FH $xor_password_string;
close FH;

# create session id as "session_file_name:xor_password_key"
# so that 'key' is not stored on disk,
# user name is not stored on disk
# disk file only has session_file name and encrypted password,
# (not second half of session_id),
# thus, even if password is decrypted, user name is not known.

$w_session_id = $session_file_str . ':' . $xor_password_key;

return($w_session_id);
     
}
########################################################################
# decrypt_session_id
sub decrypt_session_id
{

# $w_password = &decrypt_session_id($w_session_id);

my ($w_session_id) = @_;
my $session_file_str, $session_key;
my $session_file;
my $w_password;
my $xor_password_string;
my $len,$count,$sa,$ra,$xa,$unxa,$ca;
my @xor_array,@random_array;

undef(@xor_array);
undef(@random_array);

# check file and decrypt

($session_file_str, $session_key) = split /:/, $w_session_id, 2;

$session_file = $session_dir . '/' . $session_file_str . $session_file_suffix;

if ( -e "$session_file" )
      {
      # get password for login check below

      $xor_password_str = &include_file("$session_file");

      # we decrypt here

      @xor_array    = split /\./, $xor_password_str;
      @random_array = split /\./, $session_key;
      $len = @random_array;

      $count = 0;
      $sa = "";

      while ( $count < $len )
              {
              $ra = $random_array[$count];
              $xa = $xor_array[$count];
              $unxa = int($xa) ^ int($ra);

              $ca = chr($unxa);
              $sa .= $ca;

              $count++;
              }

      $w_password = $sa;
      }
else
      {
      # w_session_id exists, but session files doesn't; login again
      &error_message(qq~Error<p>
                        You Session ID file must have expired.<p>
                        Please Login Again.<p>
                        $login_page
                        <p>
                       ~);
      }

return($w_password);
     
}
########################################################################
# delete_old_files
sub delete_old_files
{
     
# &delete_old_files($delete_directory, $delete_suffix, $seconds_to_save);

my ($delete_directory, $delete_suffix, $seconds_to_save) = @_;

my $save_time = time() - $seconds_to_save;
my $modtime;

opendir (DIRHANDLE, $delete_directory);

# we use $delete_suffix to make it safer; we don't want to delete
# any files except those matching the file spec

my @filelist = grep { /$delete_suffix$/ } readdir(DIRHANDLE);

closedir (DIRHANDLE);

my @sortlist   = sort(@filelist);

# loop through directory
foreach my $infile (@sortlist)
        {
        $infile = "$delete_directory/$infile";

        # check if file is a directory
        next if ( -d $infile );

        ($modtime) = (stat($infile))[9];

        if ( $modtime < $save_time )
                {
                # file is older, so delete it

                my $delete_count = unlink "$infile";
        
                if ( -e $infile and $delete_count != 1 )
                        {
                        &error_message(qq~Problem Removing File: $infile~);
                        }
                }
        }

}
########################################################################
# check_authorization
sub check_authorization
{

# &check_authorization('admin');

my ($check_level) = @_;
my ($bad_level_msg);

$bad_level_msg = qq~
Error!
<p>
<b>Either...</b> you didn't type in a user_name,<br>
<b>or...</b> you may not be authorized to do this operation.
For example, you may have clicked on a "search" link as a
"public" user, thus allowing you to view records. In order
to edit records or do other functions, you usually have to
"log in" as an authorized user. You may wish to go back to
the starting page, and see if there was a "log in" option.
<p>
If you have questions, please contact the<br>
System Admininstrator at $admin_email\.
~;

#.......

if ( $w_user !~ /\w+/ )
     { &error_message($bad_level_msg); exit; }

if ( ($check_level eq 'admin') and ($user_level ne 'admin') )
     { &error_message($bad_level_msg); exit; }

if ( ($check_level eq 'intermediate') and
     ($user_level ne 'admin' and $user_level ne 'intermediate')
   )
     { &error_message($bad_level_msg); exit; }

if (
     ($check_level eq 'public') and
             ($user_level ne 'admin' and
              $user_level ne 'intermediate' and
              $user_level ne 'public')
   )
     { &error_message($bad_level_msg); exit; }

}
##################################################################
# sub_function
sub sub_function
{

# this subroutine is used to process the pre and post action
# subroutine calls contained in the 'public_function' and 'admin_function'
# action specs

my ($response_text);

$response_text = '<p>Error!<p>No Function Selected.<p>';

# be careful: these are used by the system.
# add your own functions at the bottom of this if/else branch

if ( $sub_function eq 'global_update' )
        {
        $response_text = &global_update;
        }
elsif ( $sub_function eq 'global_delete' )
        {
        $response_text = &global_delete;
        }
if ( $sub_function eq 'confirm_global_update' )
        {
        $response_text = &confirm_global_update;
        }
elsif ( $sub_function eq 'confirm_global_delete' )
        {
        $response_text = &confirm_global_delete;
        }
elsif ( $sub_function eq 'mysql_backup' )
        {
        $response_text = &mysql_backup;
        }

# here's where you can add your own user functions:

elsif ( $sub_function eq 'user_login_page' )
        {
        $response_text = &user_login_page;
        }

elsif ( $sub_function eq 'retrieve_password' )
        {
        $response_text = &retrieve_password;
        }

elsif ( $sub_function eq 'claim_record' )
        {
        $response_text = &claim_record;
        }

if ( $sub_function eq 'make_user_pass' )
        {
        $response_text = &make_user_pass;
        }

elsif ( $sub_function eq 'sample_report' )
        {
        $response_text = &sample_report;
        }

elsif ( $sub_function eq 'admin_menu' )
        {
        $response_text = &admin_menu;
        }

elsif ( $sub_function eq 'reports_menu' )
        {
        $response_text = &reports_menu;
        }

elsif ( $sub_function eq 'author_select' )
        {
        $response_text = &author_select;
        }

elsif ( $sub_function eq 'show_database_structure' )
        {
        $response_text = &show_database_structure;
        }

elsif ( $sub_function eq 'secure_login' )
        {
        $response_text = &secure_login;
        }

else {}

return($response_text);

}
########################################################################
# secure_login
sub secure_login
{

my $response_text;

$response_text = qq~
<br><br>
<b><font size=+1>Successful Login</font></b><br>
You have logged in successfully, using the Encrypted Secure Server.<br>
Now that your password is protected,<br>
please hit continue to proceed to the normal menu.
<p>
After hitting Continue, your browser may display a message
stating that the submission is insecure. This is because the script
is "breaking out" of the secure server method, since your password
has already been securely processed. You may safely ignore any
browser security message.
<p>
<FORM method="post" action="$futuresql_url">
$security_fields
<input type="hidden" name="s_"  value="$s_">
<input type="hidden" name="dc_" value="$dc_">
<input type="hidden" name="r_"  value="$r_">
<input type="hidden" name="p_"  value="$p_">
<input type="hidden" name="rt_" value="$rt_">
<input type="hidden" name="w_"  value="1">
<input type="hidden" name="a_"  value="admin_functions_._admin_menu">
<input type="image" src="$image_url/continue.gif"  border=0 alt="Continue" name="continue">
</form>
~;

&print_message($response_text);
     
}
########################################################################
# admin_menu
sub admin_menu
{

my $response_text;

$admin_menu_main_text =~ s/:!:w_first:!:/$w_first/g;
$admin_menu_main_text =~ s/:!:w_last:!:/$w_last/g;
$admin_menu_main_text =~ s/:!:security_fields:!:/$security_fields/g;
$admin_menu_main_text =~ s/:!:futuresql_url:!:/$futuresql_url/g;
$admin_menu_main_text =~ s/:!:image_url:!:/$image_url/g;
$admin_menu_main_text =~ s/:!:default_rows_per_page:!:/$default_rows_per_page/g;

$response_text = qq~
$admin_menu_main_text
~;

if ( $w_user_type eq 'admin' and $user_level eq 'admin' )
        {
        $admin_menu_extra_text =~ s/:!:security_fields:!:/$security_fields/g;
        $admin_menu_extra_text =~ s/:!:futuresql_url:!:/$futuresql_url/g;
        $admin_menu_extra_text =~ s/:!:image_url:!:/$image_url/g;
        $admin_menu_extra_text =~ s/:!:default_rows_per_page:!:/$default_rows_per_page/g;

        $response_text .= qq~
        $admin_menu_extra_text
        ~;
        }
else
        {
        $response_text .= qq~
        <hr>
        ~;
        }

return($response_text);

}
########################################################################
# mail_to
sub mail_to
{

# this is a SAMPLE function, and may have to be modified
#............................................................

# &mail_to($email_to, $email_from, $subject, $mail_body, $reply_to);

my ($email_to, $email_from, $subject, $mail_body, $reply_to) = @_;

if ( $reply_to !~ /\@/ ){$reply_to = $email_from;}

open (MAIL, "|$mailprog") || &error_message(qq~Error!<p>
                             Can't open $mailprog!~);

print MAIL "To: $email_to\n";
print MAIL "From: $email_from\n";
print MAIL "Subject: $subject\n";
print MAIL "Reply-To: $reply_to\n";
print MAIL "\n";
print MAIL "$mail_body";
print MAIL "\n";
close (MAIL);

}
########################################################################
# user_login_page
sub user_login_page
{

# this is a SAMPLE function, and may have to be modified
#............................................................

my ($response_text);

$response_text = qq~
<font size=+1 color="darkblue"><b>
Add a Record<br>
or Access a Current Record
</b></font>
<p>
<font size=-1>
<FORM method="post" action="futuresql.cgi">
Select one of the options below, and type in
data as appropriate. From this page, you can edit, claim, add,
or request a forgotten password for an existing record.
<ol>
<li> <input type="radio" name="a_" value="edit_record" checked>
     LOGIN: to your record (for editing)<br>
     (Just type in your <b>UserName and Password</b>, and you'll be
     taken automatically to your record.)
<p>
<li> <input type="radio" name="a_" value="public_functions_._retrieve_password">
     RETRIEVE YOUR PASSWORD: If you've lost your password, and
     have a valid user name, type your user name and email in, and your
     password will be emailed to you. Note that it will only be
     emailed to the email address that we have on file, to
     protect our users' security. If you've changed your email
     address, please email us at $admin_email.
<p>
<li> <input type="radio" name="a_" value="public_functions_._claim_record">
     CLAIM A RECORD: If you want to "claim" a record that
     doesn't have a UserName or Password, then
     type in <b>your Email Address</b>; and a <b>UserName
     </b> that you want to use. After confirming
     that the ID is not already taken, the
     program will email you a password. You'll then
     be able to login, and edit that record.
<p>
<li> <input type="radio" name="a_" value="add_record">
     ADD: a New Record. After the record is
     approved by our administrator, you'll be given
     access to edit it further, and it will be added
     to the public list.  Note that you must
     use a <b>different</b> UserName for each record (if you
     have multiple records.) You must type in the <b>User
     Name</b> on <b>THIS</b> page, and then hit the submit
     button. After filling in your data on the add record page,
     your password will be emailed to you, so that you can
     access the new record.
</ol>
Select an Option Button above, type in the appropriate data,
and hit Log In. If you have any questions, please email us at
$admin_email.
</font>
<p>

<div align=center>
<p>
<table border="0" cellpadding="8" cellspacing="2" width="95%">

<tr>

<td valign="top" bgcolor="#485798">
<font color="white" face="Times New Roman,Georgia,Times">
<b>Record ID<br>
</b></font>
</td>

<td valign="top" bgcolor="#485798">
<INPUT type="text" name="w_" size="10" maxlength="10" value="$w_"><br>
<font color="white" face="Times New Roman,Georgia,Times" size="-1">
Note that depending on the user name
and password that you type in, you may
be taken to a different record. You can also type
in a different ID in order to claim that record.
</font>
</td>
</tr>

<td valign="top" bgcolor="#485798">
<font color="white" face="Times New Roman,Georgia,Times">
<b>User Name<br>
</b></font>
</td>

<td valign="top" bgcolor="#485798">
<INPUT type="text" name="w_user" size="16" maxlength="16"><br>
<font color="white" face="Times New Roman,Georgia,Times" size="-1">
(Note that user names are case sensitive -
username is NOT UserName is NOT USERNAME)
</font>
</td>
</tr>

<tr>
<td valign="top" bgcolor="#485798">
<font face="Times New Roman,Georgia,Times" color="white">
<b>Password<br>
</b>
</font>
</td>

<td valign="top" bgcolor="#485798">
<INPUT type="text" name="w_password" size="16" maxlength="16"><br>
<font color="white" face="Times New Roman,Georgia,Times" size="-1">
(Note that passwords are also case sensitive -
password is NOT PassWord is NOT PASSWORD)
</font>
</td>
</tr>

<tr>
<td valign="top" bgcolor="#485798">
<font face="Times New Roman,Georgia,Times" color="white">
<b>Email<br>
</b>
</font>
</td>

<td valign="top" bgcolor="#485798">
<INPUT type="text" name="email" size="20" maxlength="50">
</td>
</tr>

<tr>
<td colspan="2" align=center valign="top" bgcolor="#485798">
<input type="hidden" name="s_" value="default">
<input type="hidden" name="dc_" value="u.1">
<input type="hidden" name="r_" value="10">
<input type="hidden" name="p_" value="m">
<input type="hidden" name="rt_" value="t1">
<input type="hidden" name="user_level" value="public">
<input type="submit" value="Log In">
</tr>
<tr>
<td colspan=2>
</form>
</div>
<p>
</td>
</tr>
</table>
~;

return($response_text);

}
##################################################################
# setup_authorized_edit
sub setup_authorized_edit
{

# this is a SAMPLE function, and may have to be modified
#............................................................

# &setup_authorized_edit;

if ( $user_level ne 'admin' )
{

if ( $w_user !~ /\w+/ or $w_user eq 'public' )
     {  &error_message(qq~Error!<p>
        Invalid or Missing User Name!~); }

my $sth = $dbh->prepare("select
                         $key_field, user_name, password, status
                         from $table_name
                         where
                         user_name = '$w_user' and
                         password  = '$w_password'
                       ")
                       or die
                       &error_message(qq~Error!<p>
                       Can't do select: $DBI::errstr~);

my $rv = $sth->execute or die
                       &error_message(qq~Error!<p>
                       Can't execute the query: $DBI::errstr~);

($$key_field, $user_name, $password, $status ) = $sth->fetchrow_array;

if ( $$key_field )
        {
        if ( $status eq 'approved' )
                {
                $w_ = $$key_field; param('w_',$w_);

                &get_hash_vars;
                for $attrib_name ( keys %{$s_spec{action}{$a_}} )
                        {
                        $attrib_value = $s_spec{action}{$a_}{$attrib_name};
                        $$attrib_name = $attrib_value;
                        }

                &assemble_header;
                }
        else
                {
                &error_message(qq~Error!<p>
                The record for user_name [$w_user] in the $table_name table
                is not yet 'approved'.
                <p>
                Please try accessing the record at a later date.
                ~);
                exit;
                }
        }
else
        {
        &error_message(qq~Error!<p>
        <b>Either...</b> the user_name [$w_user] has no record in the $table_name table,<br>
        <b>... or,</b> you may have typed in the password incorrectly,<br>
        <b>... or,</b> you might not have typed in the password.<br>
        <b>... remember,</b> user names and passwords are Case Sensitive.
        <p>
        Please hit BACK and try again.
        ~);
        exit;
        }

}

}
##################################################################
# retrieve_password
sub retrieve_password
{

# this is a SAMPLE function, and may have to be modified
#............................................................

my ($response_text);
my $email = param('email');

unless ( $w_user and $email )
        {
        $response_text = qq~
        You must type in a user name and an email address.
        <p>
        Please hit BACK and try again.
        ~;
        return($response_text);
        }

my $sth = $dbh->prepare("select
                         password, status, contact_email
                         from $table_name
                         where
                         user_name = '$w_user'
                       ")
                       or die
                       &error_message(qq~Error!<p>
                       Can't do select: $DBI::errstr~);

my $rv = $sth->execute or die
                       &error_message(qq~Error!<p>
                       Can't execute the query: $DBI::errstr~);

@row = $sth->fetchrow_array;

$password = $row[0];
$status   = $row[1];
$db_email = $row[2];

if ( scalar(@row) > 0 )
        {
        if ( $status ne 'approved' )
                {
                $response_text = qq~
                Your record has not been approved.
                <p>
                Please us at $admin_email.
                ~;
                return($response_text);
                }

        if ( ($db_email ne $email) or ( $db_email !~ /\w+\@\w+\.\w+/ ) )
                {
                $response_text = qq~
                Your email address doesn't match, or is invalid.
                <p>
                You typed [$email].
                <p>
                Please hit BACK and try again, or contact us at $admin_email.
                ~;
                return($response_text);
                }

        if ( $password !~ /\w+/ )
                {
                $response_text = qq~
                Your password was blank or invalid.
                <p>
                Please contact us at $admin_email.
                ~;
                return($response_text);
                }

        if ( ($status eq 'approved') and
             ($db_email eq $email)   and
             ($password =~ /\w+/ )
           )
                {
                # email...

                my $mail_body = &include_file("$password_mail_file");
                $mail_body =~ s/:!:table_name:!:/$table_name/g;
                $mail_body =~ s/:!:user_name:!:/$w_user/g;
                $mail_body =~ s/:!:password:!:/$password/g;

                &mail_to($email, $admin_email, 'Your Password', $mail_body);

                $response_text = qq~
                Your password is being emailed to you!
                ~;

                return($response_text);
                }
        else
                {
                $response_text = qq~
                Unknown Error!
                <p>
                Please contact us at $admin_email.
                ~;
                return($response_text);
                }
        }
else
        {
        $response_text = qq~
        <p><br>Error!<p>
        The user_name [$w_user] has no record in the $table_name table.
        <p>
        Please hit BACK and try again.
        <p>
        ~;
        return($response_text);
        }

}
##################################################################
# claim_record
sub claim_record
{

# this is a SAMPLE function, and may have to be modified
#............................................................

my ($response_text);
my $email = param('email');
my $record_id = $w_;

unless ( $w_user and $email and $record_id )
        {
        $response_text = qq~
        You must type in a user name, email address and ID.
        <p>
        Please hit BACK and try again.
        ~;
        return($response_text);
        }

# check to see if record id is taken

my $sth = $dbh->prepare("select
                         user_name, password, status
                         from $table_name
                         where
                         $key_field = '$record_id'
                       ")
                       or die
                       &error_message(qq~Error!<p>
                       Can't do select: $DBI::errstr~);

my $rv = $sth->execute or die
                       &error_message(qq~Error!<p>
                       Can't execute the query: $DBI::errstr~);

@row = $sth->fetchrow_array;

$user_name = $row[0];
$password  = $row[1];
$status    = $row[2];

if ( $status eq 'approved' )
        {
        $status_str = "This Record is Approved.";
        }
else
        {
        $status_str = "This Record is NOT Approved. Please contact $admin_email.";
        }

if ( scalar(@row) > 0 )
        {
        if ( ($user_name eq '' ) and ($password eq '' ) )
                {

                # first, check if that new user name is taken

                my $sth = $dbh->prepare("select
                                         user_name
                                         from $table_name
                                         where
                                         user_name = '$w_user'
                                       ")
                                       or die
                                       &error_message(qq~Error!<p>
                                       Can't do select: $DBI::errstr~);

                my $rv = $sth->execute or die
                                       &error_message(qq~Error!<p>
                                       Can't execute the query: $DBI::errstr~);

                @row = $sth->fetchrow_array;

                $user_name = $row[0];

                if ( scalar(@row) > 0 )
                        {
                        $response_text = qq~
                        The record [$record_id] is available, but the<br>
                        User Name [$w_user] is already in use.
                        <p>
                        Try using a variation, like [$w_user\.421]<br>
                        (as one example)
                        <p>
                        Hit BACK and try again.
                        <p>
                        If you have questions, please contact us at $admin_email.
                        ~;
                        return($response_text);
                        }

                # next, generate password

                $new_password = $record_id . $w_user;
                $new_password = substr($new_password, 0, 16);

                # claim it (update record)

                $update_str = "update $table_name
                               set
                               user_name     = '$w_user',
                               contact_email = '$email',
                               password      = '$new_password'
                               where
                               $key_field    = '$record_id'
                              ";

                $sth = $dbh->do("$update_str")
                             or die
                             &error_message(qq~Error!<p>
                             Can't update data: $DBI::errstr~);

                # email user name and password
                # email admin with claim data

                my $mail_body = &include_file("$claim_record_mail_file");
                $mail_body =~ s/:!:table_name:!:/$table_name/g;
                $mail_body =~ s/:!:user_name:!:/$w_user/g;
                $mail_body =~ s/:!:password:!:/$new_password/g;
                $mail_body =~ s/:!:record_id:!:/$record_id/g;
                $mail_body =~ s/:!:status:!:/$status_str/g;

                &mail_to($email, $admin_email, "$table_name Record Claimed", $mail_body);
                &mail_to($admin_email, $admin_email, "$table_name Record Claimed by User", $mail_body);

                $response_text = qq~
                You have successfully claimed record [$record_id]!<br>
                The password is being emailed to you!
                ~;

                return($response_text);
                }
        else
                {
                $response_text = qq~
                The record [$record_id] is already taken.
                <p>
                Please contact us at $admin_email.
                ~;
                return($response_text);
                }
        }
else
        {
        $response_text = qq~
        Error!<p>
        The ID [$record_id] is not in the $table_name table.
        <p>
        Please hit BACK and try again.
        <p>
        ~;
        return($response_text);
        }

}
##################################################################
# make_user_pass
sub make_user_pass
{

# this is a SAMPLE function, and may have to be modified
#............................................................

# Create user names and passwords on the fly for all records with email
# addresses.

my ($response_text) = '';
my ($count);

my $sth = $dbh->prepare("select count(*)
                         from $table_name
                         where
                         user_name = '' and
                         password  = '' and
                         contact_email <> ''
                       ")
                       or die
                       &error_message(qq~Error!<p>
                       Can't do select: $DBI::errstr~);

my $rv = $sth->execute or die
                       &error_message(qq~Error!<p>
                       Can't execute the query: $DBI::errstr~);

( $count ) = $sth->fetchrow_array;

$response_text = qq~
<p>
<b><font size=+1>
Confirmation of Make Password Routine
</font></b>
<p>
There are $count records that have no username, no password,
but DO have a 'contact_email' address.
<p>
~;

if ( $count == 0 )
{
$response_text .= qq~
Therefore, there is nothing to process...
<p>
~;
}

elsif ( $count > 0 )
{

$response_text .= qq~
Are you sure that you want to process these records?<br>
(Doing so will send emails to all of those people.)
<p>
If so, hit "Process".
<p>
<FORM method="post" action="futuresql.cgi">
<input type="hidden" name="s_" value="default">
<input type="hidden" name="dc_" value="u.1">
<input type="hidden" name="r_" value="10">
<input type="hidden" name="p_" value="m">
<input type="hidden" name="rt_" value="t1">
<input type="hidden" name="user_level" value="admin">
<input type="hidden" name="w_" value="1">
<input type="hidden" name="a_" value="admin_functions_._confirm_make_user_pass">
<DIV align=center>
<input type="submit" value=" Process ">
</div>
</form>
~;

}

return($response_text);

}
##################################################################
# confirm_make_user_pass
sub confirm_make_user_pass
{

# this is a SAMPLE function, and may have to be modified
#............................................................

# Create user names and passwords on the fly for all records with email
# addresses.

my ($response_text) = '';
my ($email, $user_name, $password, $record_id, $organization);

$status_str = "This Record is Approved.";

my $sth = $dbh->prepare("select
                         $key_field, contact_email, status, user_name, password, name
                         from $table_name
                         where
                         user_name = '' and
                         password  = '' and
                         contact_email <> ''
                       ")
                       or die
                       &error_message(qq~Error!<p>
                       Can't do select: $DBI::errstr~);

my $rv = $sth->execute or die
                       &error_message(qq~Error!<p>
                       Can't execute the query: $DBI::errstr~);

while (( $key_value, $email, $status, $user_name, $password, $organization ) = $sth->fetchrow_array )
{

if ( ($user_name eq '' ) and ($password eq '' ) and ($email ne '') )
        {

        # create new user name and password

        $new_user_name = 'fsql_' . $key_value . $table_name;
        $new_password  = 'fsql_' . $key_value . '_' . (time() + $key_value);

        $new_user_name = substr($new_user_name, 0, 16);
        $new_password  = substr($new_password, 0, 16);

        # check to make sure user name is not duped

        my $sth2 = $dbh->prepare("select
                                 user_name
                                 from $table_name
                                 where
                                 user_name = '$new_user_name'
                               ")
                               or die
                               &error_message(qq~Error!<p>
                               Can't do select: $DBI::errstr~);

        my $rv2 = $sth2->execute or die
                                 &error_message(qq~Error!<p>
                                 Can't execute the query: $DBI::errstr~);

        @row2 = $sth2->fetchrow_array;

        $user_name2 = $row[0];

        if ( scalar(@row2) > 0 )
                {
                $response_text = qq~
                The User Name [$new_user_name] is already in use.
                <p>
                ~;
                return($response_text);
                }

        # claim it (update record)

        $update_str = "update $table_name
                       set
                       user_name = '$new_user_name',
                       password  = '$new_password',
                       status    = 'approved'
                       where
                       $key_field = '$key_value'
                      ";

        $sth3 = $dbh->do("$update_str")
                     or die
                     &error_message(qq~Error!<p>
                     Can't update data: $DBI::errstr~);

        # email user name and password

        my $mail_body = &include_file("$user_pass_mail_file");
        $mail_body =~ s/:!:table_name:!:/$table_name/g;
        $mail_body =~ s/:!:user_name:!:/$new_user_name/g;
        $mail_body =~ s/:!:password:!:/$new_password/g;
        $mail_body =~ s/:!:record_id:!:/$key_value/g;
        $mail_body =~ s/:!:status:!:/$status_str/g;
        $mail_body =~ s/:!:organization:!:/$organization/g;

        &mail_to($email, $admin_email, "Welcome to Our Database!", $mail_body);

        $response_text .= qq~
        The record [$key_value] has been updated.<br>
        ~;

        }
}

return($response_text);

}
##################################################################
# new_record_password
sub new_record_password
{

# this is a SAMPLE function, and may have to be modified
#............................................................

my ($response_text);

# generate password

$new_password = $key_value . $user;
$new_password = substr($new_password, 0, 16);

# claim it (update record)

$update_str = "update $table_name
               set
               password  = '$new_password'
               where
               $key_field = '$key_value'
              ";

$sth = $dbh->do("$update_str")
             or die
             &error_message(qq~Error!<p>
             Can't update data: $DBI::errstr~);

# email user name and password
# email admin with claim data

if ( $status eq 'approved' )
        {
        $status_str = "This Record is Approved.";
        }
else
        {
        $status_str = qq~
This is a New Record, so it's not yet Approved.
Our staff will email you after we approve it.
For questions,  please email $admin_email.
Thanks!
~;
        }

my $mail_body = &include_file("$add_record_mail_file");
$mail_body =~ s/:!:table_name:!:/$table_name/g;
$mail_body =~ s/:!:user_name:!:/$user_name/g;
$mail_body =~ s/:!:password:!:/$new_password/g;
$mail_body =~ s/:!:record_id:!:/$key_value/g;
$mail_body =~ s/:!:status:!:/$status_str/g;

&mail_to($contact_email, $admin_email, "$table_name Record Added", $mail_body);

$response_text = qq~
Your password is being emailed to you!
~;

return($response_text);

}
##################################################################
# reports_menu
sub reports_menu
{

# this is a SAMPLE function, and may have to be modified
#............................................................

my $response_text;

$response_text = qq~

<table border=0 width="80%" cellpadding=10>
<tr>
<td valign=top align=left>
<b><font size=+1>Reports</font></b>
<p>
... reports will go here ...
<p>
</td>
</tr>
</table>
~;

return($response_text);

}
########################################################################
# sample_report
sub sample_report
{

# this is a SAMPLE function, and may have to be modified
#............................................................

my ($response_text);

        # put sample report here
        $response_text = qq~
        <p>
        This is one example of where you can run a
        specialized report, all from the same interface!
        <p>
        You can also run functions that operate on a set
        of records, like emailing all of your users, or
        other administrative functions.
        ~;

return($response_text);

}
##################################################################
# keep_old_value
sub keep_old_value
{

# this is a SAMPLE function, and may have to be modified
#............................................................

# this is a routine that you can put into the 'update_expression'
# in a field, in the dictionary file, to allow admin users to update
# a field, but not anyone else. For example:

        # if ( $user_level ne 'admin' )
        #     {
        #     $string = &keep_old_value('status');
        #     }

# using the 'status' field as an example, this essentially throws
# away any value that unauthorized folks try to put in.
# the $where_str is set to the key field of the record being edited.

# in the same way, this can be put into the 'insert_expression' of the
# same field, so that new records get the correct value

        # if ( $user_level ne 'admin' )
        #     {
        #     $string = 'not_approved';
        #     }

my ($field) = @_;
my ($field_value);

my $sth = $dbh->prepare("select
                         $field
                         from $table_name
                         where
                         ( $hard_coded_where_str )
                         and
                         ( $where_str )
                       ")
                       or die
                       &error_message(qq~Error!<p>
                       Can't do select: $DBI::errstr~);

my $rv = $sth->execute or die
                       &error_message(qq~Error!<p>
                       Can't execute the query: $DBI::errstr~);

($field_value ) = $sth->fetchrow_array;

return($field_value);

}
##################################################################
# reject_duplicate
sub reject_duplicate
{

# this is a SAMPLE function, and may have to be modified
#............................................................

# &reject_duplicate('user_name', $string);

# this routine can be used to block duplicate entries.
# put something like this in the 'pre_op' section of the 'add_record' action.

        # if ( $user_level eq 'public' and $w_user =~ /\w+/ )
        #        {
        #        &reject_duplicate('user_name', $w_user);
        #        }

my ($field, $field_value) = @_;
my ($db_field_value);
my ($select_str);
my ($key_value);

$key_value = param($key_field);

# determine if this is a new record or an old one
if ( $key_value < 1 )
        {
        $select_str = qq~
        select
        $field
        from $table_name
        where
        $field = '$field_value'
        ~;
        }
else
        {
        $select_str = qq~
        select
        $field
        from $table_name
        where
        $field = '$field_value' and
        $key_field <> '$key_value'
        ~;
        }

my $sth = $dbh->prepare($select_str)
                or die
                &error_message(qq~Error!<p>
                Can't do select: $DBI::errstr~);

my $rv = $sth->execute or die
                       &error_message(qq~Error!<p>
                       Can't execute the query: $DBI::errstr~);

@row = $sth->fetchrow_array;

$db_field_value = $row[0];

if ( (scalar(@row) > 0) and $field_value = $db_field_value )
        {
        &error_message(qq~Error!<p>
        The value [$field_value] in the [$field] field already exists.
        <p>
        It cannot be duplicated.
        <p>
        Please hit BACK and try again.<br>
        <i>(Even a slight edit may allow this record to be
        accepted. For example, a user name might be changed
        from 'joe' to 'joe_02'.)</i>
        ~);
        exit;
        }

}
##################################################################
# get_lookup_values
sub get_lookup_values
{

# this is a SAMPLE function, and may have to be modified
#............................................................

# this is a routine to get a list of foreign table lookup id's and labels
# see the routine 'update_table' above for more notes

# &get_lookup_values('labels/values', 'foreign_table_name',
#                    'foreign_where_str', 'foreign_order_str',
#                    qw[foreign_key_field foreign_field1 foreign_field2]
#                   ),

# this would be placed in the 'field_setup/values' subroutine in the dict file
# for the field where you want the value list to popup:

# my ($array_ref) = &get_lookup_values('values', 'groups', "group_id > '0'", 'group_name', qw[group_id]);
# return($array_ref);

# this would be placed in the 'field_setup/labels' subroutine in the dict file
# for the field where you want the value list to popup:

# my ($hash_ref) = &get_lookup_values('labels', 'groups', "group_id > '0'", 'group_name', qw[group_id group_name]);
# return($array_ref);

my ($type, $foreign_table, $foreign_where_str, $foreign_order_str, @foreign_fields) = @_;

my ($foreign_field_str) = join ',', @foreign_fields;
my @lookup_values_array = ();
my %lookup_values_hash  = ();

my $sth = $dbh->prepare("select
                         $foreign_field_str
                         from $foreign_table
                         where
                         $foreign_where_str
                         order by
                         $foreign_order_str
                       ")
                       or die
                       &error_message(qq~Error!<p>
                       Can't do select: $DBI::errstr~);

my $rv = $sth->execute or die
                       &error_message(qq~Error!<p>
                       Can't execute the query: $DBI::errstr~);

while( my $hash_ref = $sth->fetchrow_hashref )
        {
        my $key_value = $hash_ref->{$foreign_fields[0]};
        if ( $type eq 'values' )
                {
                push (@lookup_values_array, $key_value);
                }
        elsif ( $type eq 'labels' )
                {
                my $label_str = '';
                foreach my $foreign_field_name ( @foreign_fields )
                        {
                        my $foreign_field_value = $hash_ref->{$foreign_field_name};
                        $label_str .= $foreign_field_value . ' - ';
                        }

                if ( substr($label_str,-3,3) eq ' - ')
                        {
                        substr($label_str, -3,3) = '';
                        }
                $lookup_values_hash{$key_value} = $label_str;
                }
        }

if ( $type eq 'values' )
        {
        return (\@lookup_values_array);
        }
elsif ( $type eq 'labels' )
        {
        return (\%lookup_values_hash);
        }
else
        {
        &error_message(qq~Error!<p>
        The subroutine 'get_lookup_values'<br>
        has been called with incorrect parameters.
        ~);
        exit;
        }

}
##################################################################
# check_required_fields
sub check_required_fields
{

# &check_required_fields(\@required_fields);

# version 1.1 - added ability to strip out empty elements from
#               multiple select popups, so that the empty element
#               wouldn't cause an error on the 'empty check'
#             - added ability to display hash labels for fields
# ................................................................
     
my ($rq_field_ref, $rq_label_ref) = @_;
my $rq_string = '';

if ( $rq_label_ref )
        {
        foreach my $rq ( @{$rq_field_ref} )
                {
                $rq_string .= $$rq_label_ref{$rq} . ' - ';
                }
        if ( substr($rq_string,-3,3) eq ' - '){substr($rq_string, -3,3) = '';}
        }
else
        {
        $rq_string = join ' - ', @{$rq_field_ref};
        }

foreach my $rq ( @{$rq_field_ref} )
     {
     my @rq_array = param($rq);
     my $rq_array_str = join ' ', @rq_array;
     my $rq_field_label;

     if ( $rq_label_ref )
        {
        $rq_field_label = $$rq_label_ref{$rq};
        }
     else
        {
        $rq_field_label = $rq;
        }

     if ( $rq_array_str !~ /\w+/ )
        {
        # print error and return
        &error_message(qq~
                        <p>
                        The following fields are required:
                        <br>
                        [$rq_string\]
                        <p>
                        Please hit back and try again.<br>
                        The field that generated this error is<br>
                        [$rq_field_label\]
                        <p>
                        ~);
        }
     else
        {
        # check arrays, strip out empty elements, restore array, and keep going
        # (since there's at least one array element, it passes the empty field test,
        # but we don't want to pass empty values to the program.
        # This wouldn't be necessary, except for the fact that people tend to
        # select the initial empty string in a multiple pop up.

        if ( @rq_array > 1 )
                {
                my @temp_array = ();
                foreach my $rq_atom ( @rq_array )
                        {
                        if ( $rq_atom =~ /\w+/ )
                                {
                                push (@temp_array, $rq_atom);
                                }
                        }

                param($rq, @temp_array);
                }
        }
     }

}
##################################################################
# get_foreign_values
sub get_foreign_values
{

# this subroutine is used to place static foreign values on a relational
# page when you only need to display the values - such as on an edit page,
# where you're editing the primary table but not the related table, but
# need to show related values. (how's that for a long sentence :-)

# &get_foreign_values('local_table', 'local_rec_id_field', 'local_rec_id_value',
#                         'local_parent_key_field', 'local_parent_key_value',
#                         'foreign_table', 'foreign_key_field', 'foreign_tag_prefix',
#                         qw[foreign_field1 foreign_field2]
#                        )
# e.g:
# &get_foreign_values('reporter_stats', 'rs_id', $w_,
#                     'reporter_id', '',
#                     'users', 'user_id', 'u_',
#                     qw[byline username email user_category status edit_level]
#                    )

# NOTE: 'local_parent_key_value' is usually passed as '', and then
#       looked up below. You can preset it, if you don't want to
#       pass it as blank.

my ($local_table, $local_rec_id_field, $local_rec_id_value,
    $local_parent_key_field, $local_parent_key_value,
    $foreign_table, $foreign_key_field, $foreign_tag_prefix,
    @foreign_fields
   ) = @_;


# print qq~
# <blockquote>
# local_table           [ $local_table             ]<br>
# local_rec_id_field    [ $local_rec_id_field      ]<br>
# local_rec_id_value    [ $local_rec_id_value      ]<br>
# local_parent_key_field [ $local_parent_key_field  ]<br>
# local_parent_key_value [ $local_parent_key_value  ]<br>
# foreign_table         [ $foreign_table           ]<br>
# foreign_key_field     [ $foreign_key_field       ]<br>
# foreign_tag_prefix    [ $foreign_tag_prefix      ]<br>
# foreign_fields        [ @foreign_fields          ]<br>
# ~;

my ($foreign_field_str) = join ',', @foreign_fields;

# first confirm local values

if ( $local_parent_key_value !~ /\w+/ )
        {
        if ( $local_rec_id_value !~ /\w+/ )
                {
                &error_message(qq~Error!<p>
                Blank key value passed in replace_foreign_values.
                ~); exit;
                }

        my $sth = $dbh->prepare("select
                                $local_parent_key_field
                                from $local_table
                                where
                                $local_rec_id_field = '$local_rec_id_value'
                                limit 1
                               ")
                  or die
                  &error_message(qq~Error!<p>
                  Can't do select: $DBI::errstr~);

        my $rv = $sth->execute or die
                               &error_message(qq~Error!<p>
                                Can't execute the query: $DBI::errstr~);

        while( my $hash_ref = $sth->fetchrow_hashref )
                {
                $local_parent_key_value = $hash_ref->{$local_parent_key_field};
                }
        }

# now get foreign values

# print qq~
# lookup:
# $foreign_key_field : $local_parent_key_value <br>
# ~;

my $sth = $dbh->prepare("select
                         $foreign_field_str
                         from $foreign_table
                         where
                         $foreign_key_field = '$local_parent_key_value'
                         limit 1
                       ")
                       or die
                       &error_message(qq~Error!<p>
                       Can't do select: $DBI::errstr~);

my $rv = $sth->execute or die
                       &error_message(qq~Error!<p>
                       Can't execute the query: $DBI::errstr~);

while( my $hash_ref = $sth->fetchrow_hashref )
        {
        foreach my $foreign_field_name ( @foreign_fields )
                {
                $foreign_field_value = $hash_ref->{$foreign_field_name};

               # print qq~
               # $foreign_field_name : $foreign_field_value <br>
               # ~;

                $varname = $foreign_tag_prefix . $foreign_field_name;
                $$varname = $foreign_field_value;

                # my $tag = ':!:' . $foreign_tag_prefix . $foreign_field_name . ':!:';
                # $template_text =~ s/$tag/$foreign_field_value/g;

                # replacing tags here only works when the post_op is just before
                # the printing of the template text (in save and update, it's not)
                # so I now use the @extra_template_tags array
                # at the top of this file

                }
        }

}
################################################################################
# check_date_format
sub check_date_format
{

my ($field) = @_;
my $date    = param($field);

unless (
    $date =~ /^\d{4}\-\d{2}\-\d{2}$/ and
    $date =~ /[1-9]+/
   )
        {
        &error_message(qq~Error!<p>
                          Your Date must be in the format
                          "YYYY-MM-DD", and cannot be empty.
                          You typed [$date\].
                          Please hit BACK and edit the format.
                         ~);
        }

}
################################################################################
# get_code_titles
sub get_code_titles
{

# &get_code_titles($split_value, $code_string, \%code_title_hash_ref);
# &get_code_titles(',', $a_tags, \%a_tags_hash);

my ($split_value, $code_string, $code_title_hash_ref) = @_;
my $title_string = '';
my $code;

foreach $code ( split $split_value, $code_string )
        { $title_string .= $$code_title_hash_ref{$code} . $split_value; }
          $title_string =~ s/${split_value}$//;

return($title_string);

}
################################################################################
# date_to_seconds
sub date_to_seconds
{

# $seconds = &date_to_seconds($month, $day, $year, 'gm/local');

my ($month, $day, $year, $type) = @_;
my $seconds;

if ( $month < 1 or $day < 1 or $year < 1 or
     ( $type ne 'gm' and $type ne 'local' )
   )
        {
        confess "Error in date_to_seconds subroutine.";
        }

if ( $type eq 'gm' )
        {
        $seconds = timegm('00', '00', '00',
                             $day, ($month - 1), ($year - 1900) );
        }
else
        {
        $seconds = timelocal('00', '00', '00',
                             $day, ($month - 1), ($year - 1900) );
        }

return($seconds);

}
################################################################################
# show_database_structure
sub show_database_structure
{

# => this subroutine assumes that:
# -  you've already logged into ONE database, with the database handle $dbh
# -  you're calling it with: $text_var = &show_database_structure;
#    ... and then you print the $text_var
# - you've already printed the [print "Content-type: text/html\n\n";] line
# - you're passing user permissions or session id's to access the database
# - $site_name is set outside of this subroutine
# - $header_lbl_bg, $header_fld_bg, $lbl_bg, $fld_bg,
#   @skip_tables, $mysqlshow are defined outside of this subroutine
#...................................................................................

my $body_text = qq~
<b><font size=+1>Displaying Database Table Structures for $site_name
</font></b>
<br>
~;

# test and create the initial database array
# first convert the exception database and table arrays
# to hashes for speed searching
#............................................................................

my %skip_tables    = ();

foreach my $table_name ( @skip_tables )
        {
        $skip_tables{$table_name} = $table_name;
        }

$body_text .= "<br><b><font size=+1>[ Database: $db_main ]</b></font><br><br>";

# now grab table names for this databases
# we use 'show tables' to avoid problems with mysqlshow % with older versions
# ............................................................................

my $sth = $dbh->prepare("show tables") or &error_message(qq~Error!<br>
                                       Can't execute the query: $DBI::errstr~);

my $rv = $sth->execute or &error_message(qq~Error!<br>
                       Can't execute the query: $DBI::errstr~);

while ( ( my $table_name ) = $sth->fetchrow_array )
        {
        if ( exists $skip_tables{$table_name} )
                {
                next;
                }

        # now display structure of the table

        my $sth2, $rv2;
        my $field_, $type_, $null_;
        my $key_, $default_, $extra_;

        my $ucfirst_table = ucfirst($table_name);

        $sth2 = $dbh->prepare("describe $table_name")
                            or die print "<br>Can't do select: $DBI::errstr<br>";

        $rv2 = $sth2->execute or die print "<br>Can't execute the query: $DBI::errstr<br>";

        $body_text .= qq~
        <table width="100%" border="1" cellpadding="3">
        <tr>
        <td colspan="6" bgcolor="$lbl_bg"><font size=+1 color="$fld_bg"><b>Table: $ucfirst_table</b></font></td>
        </tr>

        <tr>
        <td bgcolor="$header_lbl_bg"><font color="$header_fld_bg"><b>Field</b></font></td>
        <td bgcolor="$header_lbl_bg"><font color="$header_fld_bg"><b>Type</b></font></td>
        <td bgcolor="$header_lbl_bg"><font color="$header_fld_bg"><b>Null</b></font></td>
        <td bgcolor="$header_lbl_bg"><font color="$header_fld_bg"><b>Key</b></font></td>
        <td bgcolor="$header_lbl_bg"><font color="$header_fld_bg"><b>Default</b></font></td>
        <td bgcolor="$header_lbl_bg"><font color="$header_fld_bg"><b>Extra</b></font></td>
        </tr>
        ~;

        while (
                (
                 $field_, $type_, $null_,
                 $key_, $default_, $extra_
                ) = $sth2->fetchrow_array
              )
                {
                $body_text .= qq~
                <tr>
                <td bgcolor="$lbl_bg"><font color="$fld_bg">$field_ &nbsp;</font></td>
                <td bgcolor="$lbl_bg"><font color="$fld_bg">$type_ &nbsp;</font></td>
                <td bgcolor="$lbl_bg"><font color="$fld_bg">$null_ &nbsp;</font></td>
                <td bgcolor="$lbl_bg"><font color="$fld_bg">$key_ &nbsp;</font></td>
                <td bgcolor="$lbl_bg"><font color="$fld_bg">$default_ &nbsp;</font></td>
                <td bgcolor="$lbl_bg"><font color="$fld_bg">$extra_ &nbsp;</font></td>
                </tr>\n
                ~;
                }

        $body_text .= qq~
        </table>
        <br>
        <br>
        ~;

        # end of structure display
        }

return($body_text);

}
################################################################################
# make_record_updated_test_field
sub make_record_updated_test_field
{

( $record_updated_test ) =
     $dbh->selectrow_array(
     "select $record_updated_field from $table_name
     where $key_field = '$w_'
     ");

$record_updated_test = qq~
<input type="hidden" name="$record_updated_field" value="$record_updated_test">
~;

}
################################################################################
# record_updated_test
sub record_updated_test
{

$record_updated_test = param('record_updated_test');

( $record_updated_old ) =
     $dbh->selectrow_array(
     "select $record_updated_field from $table_name
     where $key_field = '$w_'
     ");

if ( "$record_updated_test" ne "$record_updated_old" )
   {
   &error_message(qq~
   <font size=+1 color="red"><b>
   Editing Conflict Error!
   </b></font>
   <p>
   Someone must have edited this record<br>
   since you started your edit.<br><br>
   This is a multi-user system, so another user<br>
   could be logged in. Their changes may not be obvious,<br>
   so you'll have to check all the fields carefully --<br>
   or just hit Edit when you view the record,<br>
   to re-initiate a <b>New Edit</b>.<br><br>
   Please click on the view button<br>
   to review the record again.
   <br>
   <FORM method="post" action="$script_name">
   <input type="hidden" name="rt_" value="$rt_">
   <input type="hidden" name="w_"  value="$w_">
   <input type="hidden" name="s_"  value="$s_">
   <input type="hidden" name="dc_" value="$dc_">
   <input type="hidden" name="r_"  value="1">
   <input type="hidden" name="p_"  value="$p_">
   $security_fields
   <input type="hidden" name="a_"  value="view_custom_record">
   <input type="image" src="$image_url/view.gif" border=0 alt="View" name="View">
   </form>
   ~);
   }
else
    {
    $record_updated_test = '';
    }

}
################################################################################
# prep_fast_search
sub prep_fast_search
{

if (
    ( $fast_search{$table_name}{use_fast_search} eq 'yes' ) and
    (
    $a_ eq 'view_custom_record' or
    $a_ eq 'update_record'      or
    $a_ eq 'save_new_record'
    )
   )
      {
      my $fast_search_fields = $fast_search{$table_name}{fast_search_fields};
      my $fast_search_order  = $fast_search{$table_name}{fast_search_order};

      $fast_search_form =~ s/:!:fast_search_fields:!:/$fast_search_fields/;
      $fast_search_form =~ s/:!:fast_search_order:!:/$fast_search_order/;

      $fast_search_form =~ s/:!:default_rows_per_page:!:/$default_rows_per_page/;
      $fast_search_form =~ s/:!:key_field:!:/$key_field/;
      $fast_search_form =~ s/:!:image_url:!:/$image_url/;
      }
else
      {
      $fast_search_form = '';
      }

}
################################################################################
# strftime notes:
# %A - full weekday name
# %B - full month name
# %d - day with leading zero
# %e - day with no leading zero
# %Y - 4 digit year
# %l - hour using 12 hour clock, with no leading zero
# %M - minute as decimal, 00-59
# %p - AM or PM
# %Z - time zone letters

# keep the 1; on the line below

1;
