#!/usr/bin/perl
#    Last change: PFB 2001-10-07 23:26:24
#############################################################################
# POD Documentation

=head1 DETAILS

fsql_site_db.pl

Primary Setup file for FutureSQL
................................

        - contains all the 'site specific' and 'database specific' variables.
        => This gets LOADED FIRST, by 'fsql_main.cgi'.
           This file then loads the main setup file: 'fsql_setup.pl'.

        => Note that you should edit the &connect_to_db subroutine in
           the fsql_setup.pl file.

=cut

#############################################################################

$script_name               = 'futuresql.cgi';
                             # this is the name of the top stub script,
                             # which should be in your web tree.
                             # => By using different stub scripts, you can
                             #    run different applications which all use
                             #    the main engine script 'fsql_main.cgi'
                             #    as well as the main setup file 'fsql_setup.pl'.
                             # => For example, I use a stub script called
                             #    'futuresql_publish.cgi' that runs a completely
                             #    different set of files.
                             # => By specifying a different preferences directory,
                             #    you can keep the same setup file naming syntax,
                             #    and view a completely separate set of files.

$preferences_dir           = '/typical_home_directory/futuresql';
                             # this is the directory where you place
                             # all of your .pl files and the /html
                             # template files. This should be
                             # OUTSIDE of your web tree!!!
                             # The main stub script (futuresql.cgi) should
                             # be in the web tree, or in the cgi-bin
                             # directory, and point to this setup file.
                             # => the $preferences_dir var should be the
                             #    full path, in orderto accomodate the
                             #    change of directory for mysqldump

$domain_url                = 'yourdomain.com';
                             # used for internal script urls

$return_url_and_text       = qq~
                             <a href="http://$domain_url">
                             Back to the Top Page</a>~;
                             # $return_url_and_text is a handy var

$login_page                = qq~<a href="https://$domain_url/login.html">Secure Log In Page</a>~;

$futuresql_url             = "http://$domain_url/$script_name";
                             # used for internal script urls

$site_name                 = 'Your Site Name';
                             # used in &show_database_structure

$default_page_title        = 'A FutureSQL Application';
                             # for the title bar of html files,
                             # when they're not defined elsewhere

$image_url                 = '/graphics';
                             # image_dir for graphics

$db_main                   = 'books';
$db_host                   = 'localhost';
$db_port                   = '3066';
$cnf_file                  = "$preferences_dir/.www.my.cnf";

$cnf_group                 = 'books_public';
                             # database connection variables
                             # we set $cnf_group here to the lowest
                             # level of security. See the subroutine
                             # &connect_to_db

$cnf_group_admin           = 'books_user';
                             # used for admin access

$admin_user                = 'books_admin';
                             # this is the admin user,
                             # used in the &connect_to_db subroutine

# email variables

$mailprog                  = "/var/qmail/bin/qmail-inject -h";
# $mailprog                  = '/usr/lib/sendmail -t -oi';
                             # sendmail is more common - but I like qmail :-)
                             # (see 'qmail.org')

$admin_email_to            = "you\@yourdomain.com";
$admin_email_from          = $admin_email_to;
                             # the email addresses for error messages, etc.

$fsql_setup_file           = "$preferences_dir/fsql_setup.pl";
                             # the primary setup file, after this initial file

$template_dir              = "$preferences_dir/html";
$template_ext              = 'html';
$use_table_subdir          = 'yes';
                             # if $use_table_subdir is set to 'yes',
                             # then table specific html files will be
                             # looked for in a subdirectory of the
                             # template_dir, based on the table name,
                             # e.g: $template_dir/users. if it's not
                             # 'yes', then the files will be looked for
                             # in the template dir. note that
                             # $rt_ files (such as t1.html) are looked
                             # for in the template dir.

$mysql_backup_dir          = "$preferences_dir/backup";
                             # the backup dir should normally be
                             # OUTSIDE of your web document root
                             # this directory is only an example
                             # this directory must be writable by the script

$session_dir               = "$preferences_dir/sessions";
                             # session_dir is where session files are stored

$session_file_suffix       = '.si';
                             # suffix of session id files - used for deleting old ones

$session_seconds_to_save   = 3600 * 4;
                             # $session_seconds_to_save is the length of time in
                             # seconds to save session files -- after which
                             # session files are deleted

                             # 3600 equals one hour (60 seconds * 60 minutes)
                             # one could assume that sessions won't last longer
                             # than 4 hours. Note that session files may exist
                             # longer than this if someone doesn't login again
                             # (unless you run a separate cron script to delete them)

# $ENV{'MYSQL_UNIX_PORT'}    = "/virtual/home/worldcommunity.com/mysql/mysql.sock";
                             # $ENV{'MYSQL_UNIX_PORT'} is used because
                             # we have multiple instances of the MySQL
                             # daemon running on our host
                             # (worldcommunity.com) and each instance
                             # has its own mysql.sock file - Therefore
                             # the script needs to find it.
                             # !!! If you use the normal MySQL
                             # installation, then COMMENT OUT this line

$record_updated_field        = 'record_updated';

$default_custom_records_sort_order = qq~$record_updated_field desc~;
                             # this is the name of the timestamp field that,
                             # when used as a sort order in reverse, will
                             # display the most recently updated fields first
                             # use this in the action_vars file,
                             # in the variable '$view_custom_records_sort_order'
                             # => This assumes that you've setup a standard
                             #    field name for this (a good idea!)
                             # => If you haven't, customize it for each table
                             #    in the action vars file by testing for the
                             #    table name, or set it above to:
                             # => qq~:!:key_field:!: desc~;
                             #    (which will parse as your PRIMARY key field name)

# the following arrays and hashes are based on your table set up files
# and are used by the Administration Menu

@table_spec_array          = (
                             'r.1',
                             'a.1',
                             'b.1',
                             'default'
                             );

%table_spec_hash           = (
                             'r.1'     => 'Readers Spec',
                             'a.1'     => 'Authors Spec',
                             'b.1'     => 'Books Spec',
                             'default' => 'Default Spec'
                             );

@table_dict_array          = (
                             'r.1',
                             'a.1',
                             'b.1',
                             );

%table_dict_hash           = (
                             'r.1'     => 'Readers Dictionary',
                             'a.1'     => 'Authors Dictionary',
                             'b.1'     => 'Books Dictionary',
                             );

# fast_search_form table tag hash
# see $fast_search_form in the action_vars file and
# the test in the top of the spec file
# add :!:fast_search_form:!: to your regular view_custom_record
# html template file (the table_name_fields.html file)

%fast_search = (
             "authors" => {
                        'use_fast_search'    => 'yes',
                        'fast_search_fields' => qq~
                                                  Last: <input type="text" name="last_name" size="10" maxlength="50">
                                                  First: <input type="text" name="first_name" size="10" maxlength="50">
                                                  ~,
                        'fast_search_order'  => qq~
                                                 <input type="hidden" name="sort_order" value="last_name,first_name">
                                                  ~,
                        },

             # in 'books' below,
             # note use of b_book_title (the underline in the field)
             # set to interact with the relational spec setup
             
             "books" => {
                        'use_fast_search'    => 'yes',
                        'fast_search_fields' => qq~
                                                  Book Title: <input type="text" name="b_book_title" size="10" maxlength="100">
                                                  ~,
                        'fast_search_order'  => qq~
                                                 <input type="hidden" name="sort_order" value="b_book_title">
                                                  ~,
                        },
             "readers" => {
                        'use_fast_search'    => 'yes',
                        'fast_search_fields' => qq~
                                                  Last: <input type="text" name="last_name" size="10" maxlength="50">
                                                  First: <input type="text" name="first_name" size="10" maxlength="50">
                                                  ~,
                        'fast_search_order'  => qq~
                                                 <input type="hidden" name="sort_order" value="last_name,first_name">
                                                  ~,
                        },
               );

# @extra_template_tags is for template parsing
# see 'TEMPLATE TAGS' in fsql_setup.pl
# By adding a tag name here, the tag is automatically added
# to the hash %template_tags

# Note that the first section list standard tags and the
# second section consists of table specific tags.

@extra_template_tags = qw[
                          w_user
                          w_first
                          w_last
                          global_todays_date
                          image_url
                          report_title
                          table_name
                          ucfirst_table

                          site_name
                          login_page
                          fast_search_form
                          record_updated_test

                          r_user
                          a_a_id
                          a_first_name
                          a_last_name
                          new_a_id_hidden_fields
                          author_current_b_id_update_button
                          b_b_id
                          author_action
                          author_select_mode
                          b_a_id
                         ];


# Now load the main setup file
#...............................................................................

if ( -e $fsql_setup_file )
     {
     require $fsql_setup_file;
     }
else
     {
     print qq~
     <p><br>Error!<p>
     Primary Setup File ($fsql_setup_file\) not found.<br>
     Please contact the system administrator.
     <p>
     ~;
     exit;
     }

#############################################################################

1;
