# This is a spec setup file for FutureSQL v2.0
#    Last change: PFB 2001-10-07 03:46:57
#############################################################################
# This is a spec setup file for FutureSQL
# See the file 'fsql_setup.pl' for documentation.
#############################################################################
# spec_setup
sub spec_setup
{

# NOTE: Unless you customize RECORD RULES, this spec file can be used for
# all of your tables. You can place table specific field html in the
# 'fsql_action_vars.pl' and then set them to 'y' in the 'specvars' files.

# place any custom vars in the next section, before the hash
#################################################################################

# DEFAULT DISPLAY is set in the fsql_setup.pl, but
# can be overridden here, by uncommenting the appropriate var

# $default_display_method = '_standard_';
# $default_display_method = '_custom_page_';
# $default_display_method = '_external_template_';

# I've included this if statement in the spec file, rather
# than the main code, in order to preserve the concept that
# the main code should almost never be modified. The variable
# 'from_admin_list' is set up in the action_vars file.
# This code sets up the administration menu to use the
# standard method for editing, updating, and adding records,
# even if the default_display_method is set to external_template
# everywhere else.

if ( param('from_admin_list') eq 'yes' and
     ( $a_ eq 'edit_record' or $a_ eq 'update_record' or $a_ eq 'add_record' )
   )
     { $default_display_method = '_standard_'; }

# prep fast_search

&prep_fast_search;

# prep record_updated test

$record_updated_test = '';

#################################################################################
# SPEC HASH OF HASHES/SUBROUTINES

%s_spec = (

'action' => {
            'top_menu' => {
                            'pre_op'            => sub {
                                                   &check_authorization('admin');

                                                   },

                            'post_op'           => sub {

                                                   },

                            'where_strings'    => {
                                                   'ALL' => qq~
                                                            $key_field > 0
                                                            ~,

                                                   },

                            'order_strings'    => {
                                                   'Most Recently Added First' => qq~
                                                            $key_field desc
                                                            ~,

                                                   },
                            'menu_body'         => qq~
                                                   $admin_menu_body_text
                                                   ~,

                            },

            'query_by_form' => {
                            'pre_op'            => sub {
                                                   &check_authorization('admin');

                                                   },
                            'post_op'           => sub {

                                                   },

                            'table_start'          => qq~
                                                      $query_by_form_table_start
                                                      ~,

                            'record_text'          => qq~
                                                      $query_by_form_record_text
                                                      ~,

                            'table_end'            => qq~
                                                      $query_by_form_table_end
                                                      ~,

                            },

            'query_by_form_custom' => {
                            'pre_op'            => sub {
                                                   &check_authorization('public');

                                                   },
                            'post_op'           => sub {

                                                   },

                            'show_select_params'    => 'yes',
                            'select_param'          => 'like',
                            'show_sort_params'      => 'yes',
                            'sort_order'            => qq~$key_field desc~,

                            'table_start'          => qq~
                                                      $query_by_form_custom_table_start
                                                      ~,

                            'record_text'          => qq~
                                                      $query_by_form_custom_record_text
                                                      ~,

                            'table_end'            => qq~
                                                      $query_by_form_custom_table_end
                                                      ~,

                            },

            'public_functions' => {
                            'pre_op'            => sub {
                                                   &check_authorization('public');
                                                   $response_text = &sub_function;

                                                   },

                            'post_op'           => sub {

                                                   },

                            'response_body'     => qq~
                                                   $public_functions_response_body
                                                   ~,
                            },

            'admin_functions' => {
                            'pre_op'            => sub {
                                                   &check_authorization('intermediate');
                                                   $response_text = &sub_function;

                                                   },

                            'post_op'           => sub {

                                                   },

                            'response_body'     => qq~
                                                   $admin_functions_response_body
                                                   ~,

                            },

            'view_admin_list' => {
                            'pre_op'            => sub {
                                                   &check_authorization('admin');
                                                   &check_for_qbf;

                                                   },
                            'post_op'           => sub {

                                                   },
                            'field_expressions' => qq~
                                                   1=1 as expression_a_name,
                                                   1=1 as expression_b_name
                                                   ~,

                            'field_expression_names' => qq~
                                                   expression_a_name,
                                                   expression_b_name
                                                   ~,

                            'hard_coded_where_str' => qq~
                                                      ( $key_field > '0' )
                                                      ~,

                            'where_str'            => qq~
                                                      ( $key_field > '0' )
                                                      ~,

                            'order_str'            => qq~
                                                      $key_field desc
                                                      ~,

                            'sub_head'             => qq~ ~,
                            'sub_head_test_field'  => qq~$key_field~,
                            'sub_head_print_field' => qq~$key_field~,

                            'table_start'          => qq~
                                                      $view_admin_list_table_start
                                                      ~,

                            'table_header'          => qq~
                                                      $view_admin_list_table_header
                                                      ~,

                            'record_text'          => qq~
                                                      $view_admin_list_record_text
                                                      ~,

                            'record_cols'          => qq~
                                                      $view_admin_list_record_cols
                                                      ~,

                            'table_end'            => qq~
                                                      $view_admin_list_table_end
                                                      ~,

                            },

            'view_custom_list' => {
                            'custom_list_files'    => {
                                                      'author' => qq~custom.authors.view_list.authors.pl~,
                                                      },

                            'pre_op'            => sub {
                                                   &check_authorization('public');
                                                   &check_for_qbf;

                                                   },
                            'post_op'           => sub {

                                                   },

                            'field_expressions' => qq~
                                                   1=1 as expression_a_name,
                                                   1=1 as expression_b_name
                                                   ~,

                            'field_expression_names' => qq~
                                                   expression_a_name,
                                                   expression_b_name
                                                   ~,

                            'hard_coded_where_str' => qq~
                                                      ( $key_field > '0' )
                                                      ~,

                            'where_str'            => qq~
                                                      ( $key_field > '0' )
                                                      ~,

                            'order_str'            => qq~
                                                      $key_field desc
                                                      ~,

                            'sub_head'             => qq~ ~,
                            'sub_head_test_field'  => qq~$key_field~,
                            'sub_head_print_field' => qq~$key_field~,

                            'table_start'          => qq~
                                                      $view_custom_list_table_start
                                                      ~,

                            'record_text'          => qq~
                                                      $view_custom_list_record_text
                                                      ~,

                            'table_end'            => qq~
                                                      $view_custom_list_table_end
                                                      ~,

                            },

            'view_record' => {
                            'pre_op'            => sub {
                                                   &check_authorization('admin');

                                                   },
                            'post_op'           => sub {

                                                   },
                            'field_expressions' => qq~
                                                   1=1 as expression_a_name,
                                                   1=1 as expression_b_name
                                                   ~,

                            'field_expression_names' => qq~
                                                   expression_a_name,
                                                   expression_b_name
                                                   ~,

                            'hard_coded_where_str' => qq~
                                                      ( $key_field > '0' )
                                                      ~,

                            'where_str'            => qq~
                                                      $key_field = '$w_'
                                                      ~,

                            'order_str'            => qq~
                                                      $key_field asc
                                                      ~,

                            'sub_head'             => qq~ ~,
                            'sub_head_test_field'  => qq~$key_field~,
                            'sub_head_print_field' => qq~$key_field~,

                            'table_start'          => qq~
                                                      $view_record_table_start
                                                      ~,

                            'record_text'          => qq~
                                                      $view_record_record_text
                                                      ~,

                            'table_end'            => qq~
                                                      $view_record_table_end
                                                      ~,

                            },

            'view_custom_record' => {
                            'pre_op'            => sub {
                                                   &check_authorization('public');

                                                   },
                            'post_op'           => sub {

                                                   },

                            'field_expressions' => qq~
                                                   1=1 as expression_b_name
                                                   ~,

                            'field_expression_names' => qq~
                                                   expression_b_name
                                                   ~,

                            'hard_coded_where_str' => qq~
                                                      ( $key_field > '0' )
                                                      ~,

                            'where_str'            => qq~
                                                      $key_field = '$w_'
                                                      ~,

                            'order_str'            => qq~
                                                      $key_field asc
                                                      ~,

                            'sub_head'             => qq~ ~,
                            'sub_head_test_field'  => qq~$key_field~,
                            'sub_head_print_field' => qq~$key_field~,

                            'display_method'    => qq~$default_display_method~, # (_standard_, _custom_page_, _external_template_)
                            'external_template' => qq~$default_external_template~,

                            'page_title'        => qq~$view_custom_record_page_title~,

                            'page_header'       => qq~
                                                   $view_custom_record_page_header
                                                   ~,
                            'menu_control_text'    => qq~
                                                   $view_custom_record_menu_control_text
                                                   ~,

                            'page_form_start'   => qq~
                                                   $view_custom_record_page_form_start
                                                   ~,

                            'custom_page'       => qq~
                                                   $view_custom_record_custom_page
                                                   ~,

                            'page_form_system_fields' => qq~
                                                   $view_custom_record_page_form_system_fields
                                                   ~,

                            'page_form_end'       => qq~
                                                   $view_custom_record_page_form_end
                                                   ~,

                            'page_footer'       => qq~
                                                   $view_custom_record_page_footer
                                                   ~,

                            'table_start'         => qq~
                                                     $view_custom_record_table_start
                                                     ~,

                            'record_text'         => qq~
                                                     $view_custom_record_record_text
                                                     ~,

                            'table_end'            => qq~
                                                      $view_custom_record_table_end
                                                      ~,

                            },

            'batch_process' => {
                            'pre_op'            => sub {
                                                   &check_authorization('admin');

                                                   },
                            'post_op'           => sub {

                                                   },

                            'where_str'            => qq~
                                                      $key_field = '$w_'
                                                      ~,

                            'table_start'          => qq~
                                                      $batch_process_table_start
                                                      ~,

                            'table_header'         => qq~
                                                      $batch_process_table_header
                                                      ~,

                            'record_text'          => qq~
                                                      $batch_process_record_text
                                                      ~,

                            'record_cols'          => qq~
                                                      $batch_process_record_cols
                                                      ~,

                            'table_end'            => qq~
                                                      $batch_process_table_end
                                                      ~,

                            },

            'confirm_batch_process' => {
                            'pre_op'            => sub {
                                                   &check_authorization('admin');
                                                   $batch_response = '';

                                                   },
                            'post_op'           => sub {

                                                   },
                            'batch_commands'    => {
                                                   'notes'    => sub {
                                                                 my $note = qq~
                                                                 scalars available:
                                                                 key_field (name of field)
                                                                 key_value
                                                                 field_name
                                                                 field_value
                                                                 process_name
                                                                 table_name
                                                                 where_str
                                                                 and more ...
                                                                 ~;
                                                                 },

                                                   'delete'   => sub {

                                                                 my $command_str = "
                                                                 delete
                                                                 from
                                                                 $table_name
                                                                 where
                                                                 $key_field = '$key_value'
                                                                 ";

                                                                 my $sth = $dbh->do($command_str) or
                                                                                 die print "Can't do $command_str: $DBI::errstr\n";

                                                                 },

                                                   'approve'  => sub {

                                                                 my $command_str = "
                                                                 update
                                                                 $table_name
                                                                 set status = 'approved'
                                                                 where
                                                                 $key_field = '$key_value'
                                                                 ";

                                                                 my $sth = $dbh->do($command_str) or
                                                                                 die print "Can't do $command_str: $DBI::errstr\n";

                                                                 },

                                                   'sample_process'   => sub {
                                                                 # - You could run a subroutine here,
                                                                 # or any kind of process.
                                                                 # e.g: email each record, or
                                                                 # change a field value in each record
                                                                 # (for example setting records to 'approved').
                                                                 # - You can even run an SQL command that
                                                                 # uses a field value of this record to
                                                                 # operate on another range of records.
                                                                 # e.g: in our forum, I'll grab the 'branch_no'
                                                                 # value in each batch record and then delete all
                                                                 # other records that have branch_no values
                                                                 # that are longer, but start with the same value
                                                                 # (in other words, the children of this record.)
                                                                 # - You can even operate on other tables.
                                                                 # - You con't have to use the $batch_response
                                                                 # variable - if it's unused, it won't display

                                                                 $batch_response .= qq~
                                                                 Processed $key_field - $key_value<br>
                                                                 ~;

                                                                 },

                                                   },

                            'where_str'            => qq~
                                                      $key_field = '$w_'
                                                      ~,

                            'table_start'          => qq~
                                                      $confirm_batch_process_table_start
                                                      ~,

                            'table_header'         => qq~
                                                      $confirm_batch_process_table_header
                                                      ~,

                            'record_text'          => qq~
                                                      $confirm_batch_process_record_text
                                                      ~,

                            'record_cols'          => qq~
                                                      $confirm_batch_process_record_cols
                                                      ~,

                            'table_end'            => qq~
                                                      $confirm_batch_process_table_end
                                                      ~,

                            },

            'edit_record' => {
                            'pre_op'            => sub {
                                                   &check_authorization('intermediate');
                                                   &check_edit_authorization;

                                                   },
                            'post_op'           => sub {
                                                   &make_record_updated_test_field;

                                                   },
                            'field_expressions' => qq~
                                                   1=1 as expression_a_name,
                                                   1=1 as expression_b_name
                                                   ~,

                            'field_expression_names' => qq~
                                                   expression_a_name,
                                                   expression_b_name
                                                   ~,

                            'hard_coded_where_str' => qq~
                                                   (1=1)
                                                   ~,

                            'where_str'            => qq~
                                                      $key_field = '$w_'
                                                      ~,

                            'order_str'            => qq~
                                                      $key_field asc
                                                      ~,

                            'sub_head'             => qq~ ~,
                            'sub_head_test_field'  => qq~$key_field~,
                            'sub_head_print_field' => qq~$key_field~,

                            'display_method'    => qq~$default_display_method~, # (_standard_, _custom_page_, _external_template_)
                            'external_template' => qq~$default_external_template~,
                            'page_title'        => qq~
                                                   $edit_record_page_title
                                                   ~,
                            'page_header'       => qq~
                                                   $edit_record_page_header
                                                   ~,

                            'menu_control_text' => qq~
                                                   $edit_record_menu_control_text
                                                   ~,

                            'page_form_start'  => qq~
                                                  $edit_record_page_form_start
                                                  ~,

                            'custom_page'       => qq~
                                                   $edit_record_custom_page
                                                   ~,

                            'page_form_system_fields' => qq~
                                                   $edit_record_page_form_system_fields
                                                   ~,

                            'page_form_end'       => qq~
                                                   $edit_record_page_form_end
                                                   ~,

                            'page_footer'       => qq~
                                                   $edit_record_page_footer
                                                   ~,

                            'table_start'       => qq~
                                                   $edit_record_table_start
                                                   ~,

                            'record_text'       => qq~
                                                   $edit_record_record_text
                                                   ~,

                            'table_end'         => qq~
                                                   $edit_record_table_end
                                                   ~,

                            },

            'update_record' => {
                            'pre_op'            => sub {
                                                   &check_authorization('intermediate');
                                                   &record_updated_test;

                                                   },

                            'post_op'           => sub {
                                                   # sample call: see setup notes

                                                   &update_authors_table;

                                                   },

                            'where_str'         => qq~
                                                   $key_field = '$w_'
                                                   ~,

                            'display_method'    => qq~$default_display_method~, # (_standard_, _custom_page_, _external_template_)
                            'external_template' => qq~$default_external_template~,

                            'page_title'        => qq~$update_record_page_title~,
                            'page_header'       => qq~
                                                   $update_record_page_header
                                                   ~,

                            'menu_control_text' => qq~
                                                   $update_record_menu_control_text
                                                   ~,

                            'page_form_start'  => qq~
                                                  $update_record_page_form_start
                                                  ~,

                            'custom_page'       => qq~
                                                   $update_record_custom_page
                                                   ~,

                            'page_form_system_fields' => qq~
                                                   $update_record_page_form_system_fields
                                                   ~,

                            'page_form_end'       => qq~
                                                   $update_record_page_form_end
                                                   ~,

                            'page_footer'       => qq~
                                                   $update_record_page_footer
                                                   ~,

                            'table_start'       => qq~
                                                   $update_record_table_start
                                                   ~,

                            'record_text'       => qq~
                                                   $update_record_record_text
                                                   ~,

                            'table_end'         => qq~
                                                   $update_record_table_end
                                                   ~,

                            },

            'add_record' => {
                            'pre_op'            => sub {
                                                   &check_authorization('intermediate');

                                                   # if ( $w_user eq 'admin' )
                                                   #        {
                                                   #        print qq~
                                                   #        <p>Error!<p>
                                                   #        You must type in a user name before
                                                   #        adding a record.
                                                   #        <p>
                                                   #        ~;
                                                   #        exit;
                                                   #        }
                                                   #
                                                   # if ( $user_level eq 'admin' and $w_user =~ /\w+/ )
                                                   #        {
                                                   #        &reject_duplicate('user_name', $w_user);
                                                   #        }

                                                   },
                            'post_op'           => sub {

                                                   },

                            'display_method'    => qq~$default_display_method~, # (_standard_, _custom_page_, _external_template_)
                            'external_template' => qq~$default_external_template~,

                            'page_title'        => qq~
                                                   $add_record_page_title
                                                   ~,
                            'page_header'       => qq~
                                                   $add_record_page_header
                                                   ~,

                            'menu_control_text' => qq~
                                                   $add_record_menu_control_text
                                                   ~,

                            'page_form_start'  => qq~
                                                  $add_record_page_form_start
                                                  ~,

                            'custom_page'       => qq~
                                                   $add_record_custom_page
                                                   ~,

                            'page_form_system_fields' => qq~
                                                   $add_record_page_form_system_fields
                                                   ~,

                            'page_form_end'       => qq~
                                                   $add_record_page_form_end
                                                   ~,

                            'page_footer'       => qq~
                                                   $add_record_page_footer
                                                   ~,

                            'table_start'       => qq~
                                                   $add_record_table_start
                                                   ~,

                            'record_text'       => qq~
                                                   $add_record_record_text
                                                   ~,

                            'table_end'         => qq~
                                                   $add_record_table_end
                                                   ~,

                            },

            'save_new_record' => {
                            'pre_op'            => sub {
                                                   &check_authorization('intermediate');

                                                   # if ( $user_level ne 'admin' )
                                                   #        {
                                                   #         if ( (param('user_name') eq '') and (param('contact_email') eq '') )
                                                   #                {
                                                   #                print qq~
                                                   #                <p>
                                                   #                <br>
                                                   #                Error!
                                                   #                <p>
                                                   #                You must type in a user name and a contact email address.
                                                   #                <p>
                                                   #                Please hit BACK and try again.
                                                   #                ~;
                                                   #                exit;
                                                   #                }
                                                   #        }

                                                   },
                            'post_op'           => sub {
                                                   # old sample call: see setup notes
                                                   # &update_table;

                                                   # if ( $user_level ne 'admin' )
                                                   #        {
                                                   #        $key_value = $field_values{$key_field};
                                                   #
                                                   #        # this is set to display in the
                                                   #        # template process in the main script
                                                   #
                                                   #        $body_all .= &new_record_password;
                                                   #        $body_all .= "<p>";
                                                   #
                                                   #        $admin_note = "A New Record has been added to the Database:\n\n";
                                                   #        $admin_note .= "Table     = $table_name\n";
                                                   #        $admin_note .= "Record_ID = $key_value\n";
                                                   #        $admin_note .= "User      = $user_name\n";
                                                   #        $admin_note .= "Password  = $new_password\n\n";
                                                   #
                                                   #        &mail_to($admin_email, $admin_email, 'New Record Added', $admin_note);
                                                   #        }

                                                   &update_authors_table;

                                                   },

                            'display_method'    => qq~$default_display_method~, # (_standard_, _custom_page_, _external_template_)
                            'external_template' => qq~$default_external_template~,

                            'page_title'        => qq~
                                                   $save_new_record_page_title
                                                   ~,
                            'page_header'       => qq~
                                                   $save_new_record_page_header
                                                   ~,

                            'menu_control_text' => qq~
                                                   $save_new_record_menu_control_text
                                                   ~,

                            'page_form_start'  => qq~
                                                  $save_new_record_page_form_start
                                                  ~,

                            'custom_page'       => qq~
                                                   $save_new_record_custom_page
                                                   ~,

                            'page_form_system_fields' => qq~
                                                   $save_new_record_page_form_system_fields
                                                   ~,

                            'page_form_end'       => qq~
                                                   $save_new_record_page_form_end
                                                   ~,

                            'page_footer'       => qq~
                                                   $save_new_record_page_footer
                                                   ~,

                            'table_start'       => qq~
                                                   $save_new_record_table_start
                                                   ~,

                            'record_text'       => qq~
                                                   $save_new_record_record_text
                                                   ~,

                            'table_end'         => qq~
                                                   $save_new_record_table_end
                                                   ~,

                            },
            }

); # end of hash

}
#############################################################################

1;
