# This is a spec setup file for FutureSQL
#    Last change: PFB 2001-10-07 03:57:39
#############################################################################
# See the file 'fsql_setup.pl' for documentation.
#############################################################################
# spec_setup
sub spec_setup
{

# NOTE: Unless you customize RECORD RULES, this spec file can be used for
# all of your tables. You can place table specific field html in the
# 'fsql_action_vars.pl' and then set them to 'y' in the 'specvars' files.

# place any custom vars in the next section, before the hash
#################################################################################

# DEFAULT DISPLAY is set in the fsql_setup.pl, but
# can be overridden here, by uncommenting the appropriate var

# $default_display_method = '_standard_';
# $default_display_method = '_custom_page_';
# $default_display_method = '_external_template_';

# I've included this if statement in the spec file, rather
# than the main code, in order to preserve the concept that
# the main code should almost never be modified. The variable
# 'from_admin_list' is set up in the action_vars file.
# This code sets up the administration menu to use the
# standard method for editing, updating, and adding records,
# even if the default_display_method is set to external_template
# everywhere else.

if ( param('from_admin_list') eq 'yes' and
     ( $a_ eq 'edit_record' or $a_ eq 'update_record' or $a_ eq 'add_record' )
   )
     { $default_display_method = '_standard_'; }

### set up of relational fields
### Note that these vars are read prior to the hash below,
### so you don't have to put these relational vars in any of the
### custom report files
### ...............................................................

$join_field_expressions = qq~
                          b.b_id                as b_b_id,
                          b.a_id                as b_a_id,
                          b.r_id                as b_r_id,
                          b.book_title          as b_book_title,
                          b.book_rating         as b_book_rating,
                          b.book_description    as b_book_description,
                          b.reader_comments     as b_reader_comments,
                          a.a_id                as a_a_id,
                          a.a_category          as a_a_category,
                          a.first_name          as a_first_name,
                          a.last_name           as a_last_name,
                          r.r_id                as r_r_id,
                          r.user                as r_user
                          ~;

$join_field_order = qq~
                          b_b_id,
                          b_a_id,
                          b_r_id,
                          b_book_title,
                          b_book_rating,
                          b_book_description,
                          b_reader_comments,
                          a_a_id,
                          a_a_category,
                          a_first_name,
                          a_last_name,
                          r_r_id,
                          r_user
                          ~;

# assemble larger strings for view_custom_record
# this assumes that we start with the current 'dc_' file to the field order
# for the primary table (this one)

$primary_field_order = $field_order;
$primary_field_order =~ s/\s+//g;
@primary_field_order = split /,/, $primary_field_order;
@primary_field_order = sort (@primary_field_order);

$join_all_field_expressions = '';
$join_all_field_order       = '';

foreach $pri_fld ( @primary_field_order )
        {
        # b_b_id,
        # b.b_id                as b_b_id,
        $join_all_field_order       .= 'b_' . $pri_fld . ',';
        $join_all_field_expressions .= 'b.' . $pri_fld . ' as b_' . $pri_fld . ',';
        }

# now append foreign fields by hand
# we assume here that we don't need to get all the foreign fields,
# because we generally just want to grab some of the fields for display

$join_all_field_order .= qq~
                         a_a_id,
                         a_a_category,
                         a_first_name,
                         a_last_name,
                         r_r_id,
                         r_user
                         ~;

$join_all_field_expressions .= qq~
                         a.a_id                as a_a_id,
                         a.a_category          as a_a_category,
                         a.first_name          as a_first_name,
                         a.last_name           as a_last_name,
                         r.r_id                as r_r_id,
                         r.user                as r_user
                         ~;

# in a pre_op below (like view_custom_record), set the regular vars to
# the above 'all' vars, like this:

# $join_field_order = $join_all_field_order;
# $join_field_expressions = $join_all_field_expressions;

# $join_field_order is also used for custom searches
#..............................................................

$join_table_names = qq~
                          books b,
                          authors a,
                          readers r
                          ~;

%join_dc_prefix_hash = (
                          'b_' => 'b.1',
                          'a_' => 'a.1',
                          'r_' => 'r.1'
                          );

# %join_fields_alias_hash is used by check_for_qbf to replace :!:t_field_names:!:
# with the proper sql 't.field_name' syntax (the '.' is added at run time)
# (pointing to the TABLE ALIAS used in $join_table_names)

%join_fields_alias_hash = (
                          'b_' => 'b',
                          'a_' => 'a',
                          'r_' => 'r'
                          );

$join_hard_coded_where_string = qq~
                          ( (b.a_id = a.a_id) and (b.r_id = r.r_id) )
                          ~;

$join_where_string = '( 1 = 1 )';

$join_key_field_where_string = qq~b.b_id = '$w_'~;

$join_order_string = qq~
                          a.last_name, a.first_name, b.book_title
                          ~;

$join_external_template = 'books_fields_join.bar.html';

if ( $a_ eq 'edit_record'     or
     $a_ eq 'update_record'   or
     $a_ eq 'add_record'      or
     $a_ eq 'save_new_record'
   )
        {
        $default_external_template = 'books_fields_join.bar_edit.html';
        }

### end of relational setup
### ...............................................................

# swap out various defaults

$create_menu = qq~
<td valign=bottom align=right width="30">
$menu_control_header
<input type="hidden" name="author_select_mode"  value="add_record">
<input type="hidden" name="a_" value="admin_functions_._author_select">
<input type="image" src="$image_url/create.gif" border=0 alt="Create Record" name="create">
$menu_control_footer
</td>
~;

$view_custom_record_menu_control_text = qq~
$home_menu $create_menu $edit_menu $find_menu_text
~;

$view_custom_record_menu_control_text =~ s/:!:key_field:!:/:!:b_b_id:!:/g;

$author_current_b_id_update_button = '';

if ( $a_ eq 'view_custom_record' )
        {
        $author_current_b_id_update_button = qq~
                <table border=0 cellpadding=0 cellspacing=0>
                <tr>
                <td>
                <FORM method="post" action="$script_name">
                </td>
                </tr>
                <tr>
                <td>
                :!:security_fields:!:
                <input type="hidden" name="rt_" value="$rt_">
                <input type="hidden" name="w_"  value=":!:key_field:!:">
                <input type="hidden" name="s_"  value="$s_">
                <input type="hidden" name="dc_" value="$dc_">
                <input type="hidden" name="r_"  value="1">
                <input type="hidden" name="p_"  value="m">
                <input type="hidden" name="author_select_mode"  value="edit">
                <input type="hidden" name="a_" value="admin_functions_._author_select">
                <input type="image" src="$image_url/change_author.gif" border=0 alt="Change Author" name="change_author">
                </td>
                </tr>
                <tr>
                <td>
                </form>
                </td>
                </tr>
                </table>
                ~;
        }

# note additional JOIN elements in hash below

# prep fast_search

&prep_fast_search;

# prep record_updated test

$record_updated_test = '';

#################################################################################
# SPEC HASH OF HASHES/SUBROUTINES

%s_spec = (

'action' => {
            'top_menu' => {
                            'pre_op'            => sub {
                                                   &check_authorization('admin');

                                                   },

                            'post_op'           => sub {

                                                   },

                            'where_strings'    => {
                                                   'ALL' => qq~
                                                            $key_field > 0
                                                            ~,

                                                   },

                            'order_strings'    => {
                                                   'Most Recently Added First' => qq~
                                                            $key_field desc
                                                            ~,

                                                   },
                            'menu_body'         => qq~
                                                   $admin_menu_body_text
                                                   ~,

                            },

            'query_by_form' => {
                            'pre_op'            => sub {
                                                   &check_authorization('admin');

                                                   },
                            'post_op'           => sub {

                                                   },

                            'table_start'          => qq~
                                                      $query_by_form_table_start
                                                      ~,

                            'record_text'          => qq~
                                                      $query_by_form_record_text
                                                      ~,

                            'table_end'            => qq~
                                                      $query_by_form_table_end
                                                      ~,

                            },

            'query_by_form_custom' => {
                            'pre_op'            => sub {
                                                   &check_authorization('public');
                                                   $use_join_str = 'yes';

                                                   },
                            'post_op'           => sub {

                                                   },

                            'show_select_params'    => 'no',
                            'select_param'          => 'like',
                            'show_sort_params'      => 'yes',
                            'sort_order'            => qq~$join_order_string~,

                            'table_start'          => qq~
                                                      $query_by_form_custom_table_start
                                                      ~,

                            'record_text'          => qq~
                                                      $query_by_form_custom_record_text
                                                      ~,

                            'table_end'            => qq~
                                                      $query_by_form_custom_table_end
                                                      ~,

                            },

            'public_functions' => {
                            'pre_op'            => sub {
                                                   &check_authorization('public');
                                                   $response_text = &sub_function;

                                                   },

                            'post_op'           => sub {

                                                   },

                            'response_body'     => qq~
                                                   $public_functions_response_body
                                                   ~,
                            },

            'admin_functions' => {
                            'pre_op'            => sub {
                                                   &check_authorization('intermediate');
                                                   $response_text = &sub_function;

                                                   },

                            'post_op'           => sub {

                                                   },

                            'response_body'     => qq~
                                                   $admin_functions_response_body
                                                   ~,

                            },

            'view_admin_list' => {
                            'pre_op'            => sub {
                                                   &check_authorization('admin');
                                                   &check_for_qbf;

                                                   },
                            'post_op'           => sub {

                                                   },
                            'field_expressions' => qq~
                                                   1=1 as expression_a_name,
                                                   1=1 as expression_b_name
                                                   ~,

                            'field_expression_names' => qq~
                                                   expression_a_name,
                                                   expression_b_name
                                                   ~,

                            'hard_coded_where_str' => qq~
                                                      ( $key_field > '0' )
                                                      ~,

                            'where_str'            => qq~
                                                      ( $key_field > '0' )
                                                      ~,

                            'order_str'            => qq~
                                                      $key_field desc
                                                      ~,

                            'sub_head'             => qq~ ~,
                            'sub_head_test_field'  => qq~$key_field~,
                            'sub_head_print_field' => qq~$key_field~,

                            'table_start'          => qq~
                                                      $view_admin_list_table_start
                                                      ~,

                            'table_header'          => qq~
                                                      $view_admin_list_table_header
                                                      ~,

                            'record_text'          => qq~
                                                      $view_admin_list_record_text
                                                      ~,

                            'record_cols'          => qq~
                                                      $view_admin_list_record_cols
                                                      ~,

                            'table_end'            => qq~
                                                      $view_admin_list_table_end
                                                      ~,

                            },

            'view_custom_list' => {
                            'custom_list_files'    => {
                                                      'multi'  => qq~custom.books.view_list.multi.pl~,
                                                      'export' => qq~custom.books.view_list.export.pl~,
                                                      },

                            'pre_op'            => sub {
                                                   &check_authorization('public');

                                                   $use_join_str = 'yes';
                                                   # place $use_join_str BEFORE
                                                   # &check_for_qbf so that the
                                                   # $search_fields string can be
                                                   # replaced by
                                                   # $join_field_order

                                                   &check_for_qbf;

                                                   },
                            'post_op'           => sub {

                                                   },

                            'field_expressions' => qq~
                                                   1=1 as expression_a_name,
                                                   1=1 as expression_b_name
                                                   ~,

                            'field_expression_names' => qq~
                                                   expression_a_name,
                                                   expression_b_name
                                                   ~,

                            'hard_coded_where_str' => qq~
                                                      $join_hard_coded_where_string
                                                      ~,

                            'where_str'            => qq~
                                                      $join_where_string
                                                      ~,

                            'order_str'            => qq~
                                                      $join_order_string
                                                      ~,

                            'sub_head'             => qq~ ~,
                            'sub_head_test_field'  => qq~$key_field~,
                            'sub_head_print_field' => qq~$key_field~,

                            'table_start'          => qq~
                                                      $view_custom_list_table_start
                                                      ~,

                            'record_text'          => qq~
                                                      $view_custom_list_record_text
                                                      ~,

                            'table_end'            => qq~
                                                      $view_custom_list_table_end
                                                      ~,

                            },

            'view_record' => {
                            'pre_op'            => sub {
                                                   &check_authorization('admin');

                                                   },
                            'post_op'           => sub {

                                                   },
                            'field_expressions' => qq~
                                                   1=1 as expression_a_name,
                                                   1=1 as expression_b_name
                                                   ~,

                            'field_expression_names' => qq~
                                                   expression_a_name,
                                                   expression_b_name
                                                   ~,

                            'hard_coded_where_str' => qq~
                                                      ( $key_field > '0' )
                                                      ~,

                            'where_str'            => qq~
                                                      $key_field = '$w_'
                                                      ~,

                            'order_str'            => qq~
                                                      $key_field asc
                                                      ~,

                            'sub_head'             => qq~ ~,
                            'sub_head_test_field'  => qq~$key_field~,
                            'sub_head_print_field' => qq~$key_field~,

                            'table_start'          => qq~
                                                      $view_record_table_start
                                                      ~,

                            'record_text'          => qq~
                                                      $view_record_record_text
                                                      ~,

                            'table_end'            => qq~
                                                      $view_record_table_end
                                                      ~,

                            },

            'view_custom_record' => {
                            'pre_op'            => sub {
                                                   &check_authorization('public');

                                                   $use_join_str           = 'yes';
                                                   $join_field_order       = $join_all_field_order;
                                                   $join_field_expressions = $join_all_field_expressions;

                                                   if ( param('author_select_mode') eq 'edit' )
                                                        {
                                                        &update_book_author;
                                                        }

                                                   },
                            'post_op'           => sub {
                                                       $author_current_b_id_update_button =~ s/:!:key_field:!:/$w_/g;
                                                       },

                            'field_expressions' => qq~
                                                   1=1 as expression_b_name
                                                   ~,

                            'field_expression_names' => qq~
                                                   expression_b_name
                                                   ~,

                            'hard_coded_where_str' => qq~
                                                      $join_hard_coded_where_string
                                                      ~,

                            'where_str'            => qq~
                                                      $join_key_field_where_string
                                                      ~,

                            'order_str'            => qq~
                                                      $join_order_string
                                                      ~,

                            'sub_head'             => qq~ ~,
                            'sub_head_test_field'  => qq~$key_field~,
                            'sub_head_print_field' => qq~$key_field~,

                            'display_method'    => qq~$default_display_method~, # (_standard_, _custom_page_, _external_template_)
                            'external_template' => qq~$join_external_template~,

                            'page_title'        => qq~$view_custom_record_page_title~,

                            'page_header'       => qq~
                                                   $view_custom_record_page_header
                                                   ~,
                            'menu_control_text'    => qq~
                                                   $view_custom_record_menu_control_text
                                                   ~,

                            'page_form_start'   => qq~
                                                   $view_custom_record_page_form_start
                                                   ~,

                            'custom_page'       => qq~
                                                   $view_custom_record_custom_page
                                                   ~,

                            'page_form_system_fields' => qq~
                                                   $view_custom_record_page_form_system_fields
                                                   ~,

                            'page_form_end'       => qq~
                                                   $view_custom_record_page_form_end
                                                   ~,

                            'page_footer'       => qq~
                                                   $view_custom_record_page_footer
                                                   ~,

                            'table_start'         => qq~
                                                     $view_custom_record_table_start
                                                     ~,

                            'record_text'         => qq~
                                                     $view_custom_record_record_text
                                                     ~,

                            'table_end'            => qq~
                                                      $view_custom_record_table_end
                                                      ~,

                            },

            'batch_process' => {
                            'pre_op'            => sub {
                                                   &check_authorization('admin');

                                                   },
                            'post_op'           => sub {

                                                   },

                            'where_str'            => qq~
                                                      $key_field = '$w_'
                                                      ~,

                            'table_start'          => qq~
                                                      $batch_process_table_start
                                                      ~,

                            'table_header'         => qq~
                                                      $batch_process_table_header
                                                      ~,

                            'record_text'          => qq~
                                                      $batch_process_record_text
                                                      ~,

                            'record_cols'          => qq~
                                                      $batch_process_record_cols
                                                      ~,

                            'table_end'            => qq~
                                                      $batch_process_table_end
                                                      ~,

                            },

            'confirm_batch_process' => {
                            'pre_op'            => sub {
                                                   &check_authorization('admin');
                                                   $batch_response = '';

                                                   },
                            'post_op'           => sub {

                                                   },
                            'batch_commands'    => {
                                                   'notes'    => sub {
                                                                 my $note = qq~
                                                                 scalars available:
                                                                 key_field (name of field)
                                                                 key_value
                                                                 field_name
                                                                 field_value
                                                                 process_name
                                                                 table_name
                                                                 where_str
                                                                 and more ...
                                                                 ~;
                                                                 },

                                                   'delete'   => sub {

                                                                 my $command_str = "
                                                                 delete
                                                                 from
                                                                 $table_name
                                                                 where
                                                                 $key_field = '$key_value'
                                                                 ";

                                                                 my $sth = $dbh->do($command_str) or
                                                                                 die print "Can't do $command_str: $DBI::errstr\n";

                                                                 },

                                                   'approve'  => sub {

                                                                 my $command_str = "
                                                                 update
                                                                 $table_name
                                                                 set status = 'approved'
                                                                 where
                                                                 $key_field = '$key_value'
                                                                 ";

                                                                 my $sth = $dbh->do($command_str) or
                                                                                 die print "Can't do $command_str: $DBI::errstr\n";

                                                                 },

                                                   'sample_process'   => sub {
                                                                 # - You could run a subroutine here,
                                                                 # or any kind of process.
                                                                 # e.g: email each record, or
                                                                 # change a field value in each record
                                                                 # (for example setting records to 'approved').
                                                                 # - You can even run an SQL command that
                                                                 # uses a field value of this record to
                                                                 # operate on another range of records.
                                                                 # e.g: in our forum, I'll grab the 'branch_no'
                                                                 # value in each batch record and then delete all
                                                                 # other records that have branch_no values
                                                                 # that are longer, but start with the same value
                                                                 # (in other words, the children of this record.)
                                                                 # - You can even operate on other tables.
                                                                 # - You con't have to use the $batch_response
                                                                 # variable - if it's unused, it won't display

                                                                 $batch_response .= qq~
                                                                 Processed $key_field - $key_value<br>
                                                                 ~;

                                                                 },

                                                   },

                            'where_str'            => qq~
                                                      $key_field = '$w_'
                                                      ~,

                            'table_start'          => qq~
                                                      $confirm_batch_process_table_start
                                                      ~,

                            'table_header'         => qq~
                                                      $confirm_batch_process_table_header
                                                      ~,

                            'record_text'          => qq~
                                                      $confirm_batch_process_record_text
                                                      ~,

                            'record_cols'          => qq~
                                                      $confirm_batch_process_record_cols
                                                      ~,

                            'table_end'            => qq~
                                                      $confirm_batch_process_table_end
                                                      ~,

                            },

            'edit_record' => {
                            'pre_op'            => sub {
                                                   &check_authorization('intermediate');
                                                   &check_edit_authorization;

                                                   },
                            'post_op'           => sub {
                                                        &get_foreign_values('books', 'b_id', $w_, 'r_id', '',
                                                                            'readers', 'r_id', 'r_',
                                                                            qw[user]
                                                                           );

                                                        &get_foreign_values('books', 'b_id', $w_,
                                                                            'a_id', '',
                                                                            'authors', 'a_id', 'a_',
                                                                            qw[first_name last_name]
                                                                           );

                                                   &make_record_updated_test_field;

                                                   },
                            'field_expressions' => qq~
                                                   1=1 as expression_a_name,
                                                   1=1 as expression_b_name
                                                   ~,

                            'field_expression_names' => qq~
                                                   expression_a_name,
                                                   expression_b_name
                                                   ~,

                            'hard_coded_where_str' => qq~
                                                   (1=1)
                                                   ~,

                            'where_str'            => qq~
                                                      $key_field = '$w_'
                                                      ~,

                            'order_str'            => qq~
                                                      $key_field asc
                                                      ~,

                            'sub_head'             => qq~ ~,
                            'sub_head_test_field'  => qq~$key_field~,
                            'sub_head_print_field' => qq~$key_field~,

                            'display_method'    => qq~$default_display_method~, # (_standard_, _custom_page_, _external_template_)
                            'external_template' => qq~$default_external_template~,
                            'page_title'        => qq~
                                                   $edit_record_page_title
                                                   ~,
                            'page_header'       => qq~
                                                   $edit_record_page_header
                                                   ~,

                            'menu_control_text' => qq~
                                                   $edit_record_menu_control_text
                                                   ~,

                            'page_form_start'  => qq~
                                                  $edit_record_page_form_start
                                                  ~,

                            'custom_page'       => qq~
                                                   $edit_record_custom_page
                                                   ~,

                            'page_form_system_fields' => qq~
                                                   $edit_record_page_form_system_fields
                                                   ~,

                            'page_form_end'       => qq~
                                                   $edit_record_page_form_end
                                                   ~,

                            'page_footer'       => qq~
                                                   $edit_record_page_footer
                                                   ~,

                            'table_start'       => qq~
                                                   $edit_record_table_start
                                                   ~,

                            'record_text'       => qq~
                                                   $edit_record_record_text
                                                   ~,

                            'table_end'         => qq~
                                                   $edit_record_table_end
                                                   ~,

                            },

            'update_record' => {
                            'pre_op'            => sub {
                                                   &check_authorization('intermediate');
                                                   &record_updated_test;

                                                   },

                            'post_op'           => sub {
                                                   &update_books_table;
                                                   &get_foreign_values('books', 'b_id', $w_, 'r_id', '',
                                                                       'readers', 'r_id', 'r_',
                                                                       qw[user]
                                                                      );

                                                   &get_foreign_values('books', 'b_id', $w_,
                                                                       'a_id', '',
                                                                       'authors', 'a_id', 'a_',
                                                                       qw[first_name last_name]
                                                                      );

                                                   },

                            'where_str'         => qq~
                                                   $key_field = '$w_'
                                                   ~,

                            'display_method'    => qq~$default_display_method~, # (_standard_, _custom_page_, _external_template_)
                            'external_template' => qq~$default_external_template~,

                            'page_title'        => qq~$update_record_page_title~,
                            'page_header'       => qq~
                                                   $update_record_page_header
                                                   ~,

                            'menu_control_text' => qq~
                                                   $update_record_menu_control_text
                                                   ~,

                            'page_form_start'  => qq~
                                                  $update_record_page_form_start
                                                  ~,

                            'custom_page'       => qq~
                                                   $update_record_custom_page
                                                   ~,

                            'page_form_system_fields' => qq~
                                                   $update_record_page_form_system_fields
                                                   ~,

                            'page_form_end'       => qq~
                                                   $update_record_page_form_end
                                                   ~,

                            'page_footer'       => qq~
                                                   $update_record_page_footer
                                                   ~,

                            'table_start'       => qq~
                                                   $update_record_table_start
                                                   ~,

                            'record_text'       => qq~
                                                   $update_record_record_text
                                                   ~,

                            'table_end'         => qq~
                                                   $update_record_table_end
                                                   ~,

                            },

            'add_record' => {
                            'pre_op'            => sub {
                                                   &check_authorization('intermediate');

                                                   },
                            'post_op'           => sub {
                                                   if ( param('author_select_mode') eq 'add_record' )
                                                        {
                                                        $a_id         = param('a_id');
                                                        $a_first_name = param('a_first_name');
                                                        $a_last_name  = param('a_last_name');
                                                        $new_a_id_hidden_fields = qq~
                                                        <input type="hidden" name="a_id" value="$a_id">
                                                        <input type="hidden" name="author_select_mode" value="save_new_record">
                                                        ~;
                                                        }

                                                   },

                            'display_method'    => qq~$default_display_method~, # (_standard_, _custom_page_, _external_template_)
                            'external_template' => qq~$default_external_template~,

                            'page_title'        => qq~
                                                   $add_record_page_title
                                                   ~,
                            'page_header'       => qq~
                                                   $add_record_page_header
                                                   ~,

                            'menu_control_text' => qq~
                                                   $add_record_menu_control_text
                                                   ~,

                            'page_form_start'  => qq~
                                                  $add_record_page_form_start
                                                  ~,

                            'custom_page'       => qq~
                                                   $add_record_custom_page
                                                   ~,

                            'page_form_system_fields' => qq~
                                                   $add_record_page_form_system_fields
                                                   ~,

                            'page_form_end'       => qq~
                                                   $add_record_page_form_end
                                                   ~,

                            'page_footer'       => qq~
                                                   $add_record_page_footer
                                                   ~,

                            'table_start'       => qq~
                                                   $add_record_table_start
                                                   ~,

                            'record_text'       => qq~
                                                   $add_record_record_text
                                                   ~,

                            'table_end'         => qq~
                                                   $add_record_table_end
                                                   ~,

                            },

            'save_new_record' => {
                            'pre_op'            => sub {
                                                   &check_authorization('intermediate');

                                                   },
                            'post_op'           => sub {
                                                   &update_books_table;
                                                   &get_foreign_values('books', 'b_id', $w_, 'r_id', '',
                                                                       'readers', 'r_id', 'r_',
                                                                       qw[user]
                                                                      );

                                                   &get_foreign_values('books', 'b_id', $w_,
                                                                       'a_id', '',
                                                                       'authors', 'a_id', 'a_',
                                                                       qw[first_name last_name]
                                                                      );

                                                   },

                            'display_method'    => qq~$default_display_method~, # (_standard_, _custom_page_, _external_template_)
                            'external_template' => qq~$default_external_template~,

                            'page_title'        => qq~
                                                   $save_new_record_page_title
                                                   ~,
                            'page_header'       => qq~
                                                   $save_new_record_page_header
                                                   ~,

                            'menu_control_text' => qq~
                                                   $save_new_record_menu_control_text
                                                   ~,

                            'page_form_start'  => qq~
                                                  $save_new_record_page_form_start
                                                  ~,

                            'custom_page'       => qq~
                                                   $save_new_record_custom_page
                                                   ~,

                            'page_form_system_fields' => qq~
                                                   $save_new_record_page_form_system_fields
                                                   ~,

                            'page_form_end'       => qq~
                                                   $save_new_record_page_form_end
                                                   ~,

                            'page_footer'       => qq~
                                                   $save_new_record_page_footer
                                                   ~,

                            'table_start'       => qq~
                                                   $save_new_record_table_start
                                                   ~,

                            'record_text'       => qq~
                                                   $save_new_record_record_text
                                                   ~,

                            'table_end'         => qq~
                                                   $save_new_record_table_end
                                                   ~,

                            },
            }

); # end of hash

}
#############################################################################

1;
